/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "bsdport.h"
#include "bsdportglobal.h"

/********************** class QBsdPort ******************************/

QBsdPort::QBsdPort(QString origin)
{
    m_inst = 0;
    m_avSrc = 0;
    m_avBin = 0;
    m_origin = origin;
}

QBsdPort::QBsdPort(const QBsdPort& orig)
    : QObject()
{
    /** THIS COPY CONSTRUCTOR CREATES A SHALLOW COPY
        IT ONLY COPIES THE POINTERS TO ..PKG **/

    m_origin = orig.origin();
    m_inst  = orig.hasInst() ? orig.inst() : 0;
    m_avSrc = orig.hasAvSrc() ? orig.avSrc() : 0;
    m_avBin = orig.hasAvBin() ? orig.avBin() : 0;
}

void QBsdPort::findInst()
{
    setInst(0);

    QBsdPortInstPkg *pkg = 0;
    QString cmd = "pkg_info -qO " + origin().toLatin1();
    QStringList grep = global->runCmd(cmd);
    QString Buffer;

    if (grep.count() > 1) /// there are multiple installed pkgs with same origin
                QBsdPort::global->emitError("findInst() : " + origin() + " has multiple" +
                            " installed packages. all but the first ignored.");
    if (!grep.first().isEmpty())
    {
        Buffer = grep.first();
        if (Buffer.contains('-'))
        {
            pkg = new QBsdPortInstPkg(this);
            pkg->setName(Buffer.section('-', 0, -2));
            pkg->setVersion(Buffer.section('-',-1,-1));
            pkg->setCategories(origin().section('/', 0,0));

            setInst(pkg);
        }
    }
}

void QBsdPort::findAvSrc()
{
    setAvSrc(0);
    if (!QFile::exists(global->portsDir().absolutePath() + "/" + origin()))
    /// no avSrc
        return;

    QBsdPortAvSrcPkg* pkg = new QBsdPortAvSrcPkg(this);

    QString pkgname = global->getMakeEnv(global->portsDir().absolutePath() +
                                         "/" + origin(), "PKGNAME");
    if (!pkgname.contains('-'))
    {
        QBsdPort::global->emitError("findAvSrc() : couldnt retrieve PKGNAME from " + origin() +
                    "'s Make-Environment");
        return;
    }

    pkg->setName(pkgname.section('-', 0,-2));
    pkg->setVersion(pkgname.section('-',-1,-1));

    setAvSrc(pkg);
}

void QBsdPort::findAvBin()
{
    setAvBin(0);

    /** we grep the FTPINDEX ( i think its faster than reading it with qfile**/
    /// an indexless version could do this instead:
    /*  echo "rstatus `cd /usr/ports/ origin() && make -V LATEST_LINK`.tbz"
        | ftp -v \
        ftp://ftp.freebsd.org/pub/FreeBSD/ports/i386/packages-6-stable/Latest/ |
        grep "LATEST_LINK.tbz" | awk '{print $11 } **/

    //TODO find a better place to store ftpIndexFile
    QBsdPortAvBinPkg *pkg = 0;
    QString cmd = "/usr/bin/grep -e \"/" + origin() + "|\"" +
                  QBsdPort::global->avBinThread()->ftpIndexFile().fileName() + "\"";
    QStringList grep = global->runCmd(cmd);

    if (grep.count() > 1) /// there are multiple installed pkgs with same origin
        QBsdPort::global->emitError("findAvBin() : " + origin() + " has multiple" +
                                 " avBin-packages. all but the first ignored.");
    if (!grep.first().isEmpty())
    {
        QStringList lbuffer = grep.first().split('|');
        if (lbuffer.at(1).section('/',3,4) == origin())
        {
            pkg     = new QBsdPortAvBinPkg(this);
            /// since we got all the info right here we set it!
            pkg->setVersion(lbuffer.at(0).section('-',-1,-1));
            pkg->setName(lbuffer.at(0).section('-',0,-2));
            #ifndef SPONTANEOUS
            pkg->setComment(lbuffer.at(3));
            pkg->setMaint(lbuffer.at(5));
            pkg->setCategories(lbuffer.at(6));
            pkg->setWWW(lbuffer.at(9));
            pkg->setDescr(QString());
            #endif
//            pkg->setReqs(lbuffer.at(8).split(' '));

            setAvBin(pkg);
        }
    }

}

QBsdPortInstPkg* QBsdPort::inst() const
{
    return hasInst() ? m_inst : global->errorPort()->inst();
}

QBsdPortAvSrcPkg* QBsdPort::avSrc() const
{
    return hasAvSrc() ? m_avSrc : global->errorPort()->avSrc();
}

QBsdPortAvBinPkg* QBsdPort::avBin() const
{
    return hasAvBin() ? m_avBin : global->errorPort()->avBin();
}

void    QBsdPort::setInst(QBsdPortInstPkg* newValue)
{
    if (m_inst)
        global->cleanDelete(m_inst);
    m_inst = newValue;
    if (m_inst)
        inst()->setParent(this);

    // 1 central
    global->emitPortState(this);
    // 2 new single
//  emitPkgState();
    // 3 original single
//    emitInstState();
}
void    QBsdPort::setInstQObject(QObject *newValue)
{
    setInst((QBsdPortInstPkg*)newValue);
}

void    QBsdPort::setAvSrc(QBsdPortAvSrcPkg* newValue)
{
    if (m_avSrc)
        global->cleanDelete(m_avSrc);
    m_avSrc = newValue;
    if (m_avSrc)
        avSrc()->setParent(this);

    // 1 central
   global->emitPortState(this);
    // 2 new single
//  emitPkgState();
    // 3 original single
//     emitAvSrcState();
}

void    QBsdPort::setAvBin(QBsdPortAvBinPkg* newValue)
{
    if (m_avBin)
        global->cleanDelete(m_avBin);
    m_avBin = newValue;
    if (m_avBin)
        avBin()->setParent(this);

    // 1 central
   global->emitPortState(this);
    // 2 new single
//	emitPkgState();
    // 3 original single
//     emitAvBinState();
}

void    QBsdPort::emitPortState()
{
    global->emitPortState(this);
}

void    QBsdPort::setAvBinQObject(QObject* newValue)
{
    setAvBin((QBsdPortAvBinPkg*)newValue);
}
/*
void    QBsdPort::emitPkgState()
{
	emit pkgState(hasInst() ? inst()->nv() : "",
					hasAvSrc() ? avSrc()->nv() : "",
					hasAvBin() ? avBin()->nv() : "");
}

void    QBsdPort::emitStatusChanged()
{
    // NOT USED RIGHT NOW
    QString i = hasInst() ? inst()->name()+"-"+inst()->version() : "";
    QString s = hasAvSrc() ? avSrc()->name()+"-"+avSrc()->version() : "";
    QString b = hasAvBin() ? avBin()->name()+"-"+avBin()->version() : "";

    emit statusChanged(this, i, s, b);
}

void    QBsdPort::emitInstState()
{
    emit instState(hasInst() ? inst()->name()+"-"+inst()->version() : "");
}

void    QBsdPort::emitAvSrcState()
{
    emit avSrcState(hasAvSrc() ? avSrc()->name()+"-"+avSrc()->version() : "");
}

void    QBsdPort::emitAvBinState()
{
    emit avBinState(hasAvBin() ? avBin()->name()+"-"+avBin()->version() : "");
}*/

/************ QBsdPortPkg ********************************/

QBsdPortPkg::QBsdPortPkg(QBsdPort* parent)
{
    m_parent = parent;

#ifndef SPONTANEOUS
    reset();
#endif
    m_proc      = 0;
}

QBsdPortPkg::QBsdPortPkg(const QBsdPortPkg& orig)
            : QObject()
{
    reset();


    //TODO WARNING
    // this copy constructor doesnt really copy anthing
    // it is just there so registerMetaTyp works

}


QStringList QBsdPortPkg::files()
{
//     QBsdPort::global->emitError(
//         "QBsdPortPkg::files() should never be called");
    return QStringList();
}

QPortList QBsdPortPkg::reqsPtrList()
{
    QBsdPort::global->emitError(
        "QBsdPortPkg::reqsPtrList() should never be called");
    return m_reqsPtrList;
}
#ifdef PORTAUDIT
QStringList QBsdPortPkg::vulna()
{
    if (QBsdPort::global->portaudit() < 0)
        return QStringList();
  #ifndef SPONTANEOUS
    if (m_vulna.count() == 0)
        return m_vulna;
    if (m_vulna.at(0) != "bpk_no_init")
        return m_vulna;
  #endif

    QStringList strl = QBsdPort::global->runCmd("/usr/local/sbin/portaudit " +
                                                    name() + "-" + version() );
    QStringList ret;

    for (int i = 0; i < strl.count(); ++i)
    {
        if (strl.at(i).startsWith("Type of problem: "))
            ret.append(QString(strl.at(i)).remove("Type of problem: "));
        else if (strl.at(i).startsWith("Reference: "))
            ret.last().append("|" +
             QString(strl.at(i)).remove("Reference: ").remove('<').remove('>'));
    }

  #ifndef SPONTANEOUS
    m_vulna = ret;
  #endif

    return ret;
}
#endif
/*
QBsdPort::QPortList QBsdPortPkg::reqsPtrList()
{
    qDebug("base reqsPtrList called for " + name().toLatin1() + "-" +
            version().toLatin1());
    QPortList ret;
    QPortList l;
    for (int i = 0; i < reqs().count(); ++i)
    {
        /// package kind doesnt matter -> 0
        l = QBsdPort::global->portWithPkgName(reqs().at(i),
                                              QBsdPortGlobal::ANYPKG);
        if (l.count()==1)
            ret.append(l.at(0));
        else if (l.count()>1)
        {
            ret.append(l.at(0));
            QBsdPort::global->emitWarning("reqsPtrList() : " + name() + "-" + version() +
                         " has more than one match for " + reqs().at(i) + ".");
        } else
            QBsdPort::global->emitWarning("reqsPtrList() : " + name() + "-" + version() +
                         " has no match for " + reqs().at(i) + ".");
    }
    return ret;
}
*/
int	QBsdPortPkg::reqsMet()/// check whether all requirements are met
{
    bool notUpToDate = false;
    QPortList list = reqsPtrList();
    for (int i = 0; i < list.count(); ++i)
    {
        if (!list.at(i)->hasInst())
            return 0;
        if (list.at(i)->hasAvSrc() && !notUpToDate)
            if (QBsdPort::global->compareVersions(
                                    list.at(i)->inst()->version(),
                                    list.at(i)->avSrc()->version()) == 2)
                notUpToDate = true;
        if (list.at(i)->hasAvBin() && !notUpToDate)
            if (QBsdPort::global->compareVersions(
                                    list.at(i)->inst()->version(),
                                    list.at(i)->avBin()->version()) == 2)
                notUpToDate = true;
    }

    return (notUpToDate ? 2 : 1);
}

void    QBsdPortPkg::stopProc()
{
    if (proc())
    {
        if (proc()->state() > 0)
        {
            proc()->terminate();
            proc()->waitForFinished(1000);
            proc()->kill();
        }

        QBsdPort::global->cleanDelete(proc());
    }
}

/************ QBsdPortInstPkg ********************************/
QString QBsdPortInstPkg::name()
{
  #ifndef SPONTANEOUS
    if (m_name != "bpk_no_init")
        return m_name;
  #endif

    QString str = QBsdPort::global->runCmd("/usr/sbin/pkg_info -QO " +
                                                   parent()->origin(), 1).at(0);

    QString ret = str.section('-',0,-2);

  #ifndef SPONTANEOUS
    m_name = ret;
    m_version = str.section('-',-1,-1); /// set anyway since we got it now
  #endif

    return ret;
}

QString QBsdPortInstPkg::version()
{
  #ifndef SPONTANEOUS
    if (m_version != "bpk_no_init")
        return m_version;
  #endif

    QString str = QBsdPort::global->runCmd("/usr/sbin/pkg_info -QO " +
                                                   parent()->origin(), 1).at(0);

    QString ret = str.section('-',-1,-1);

  #ifndef SPONTANEOUS
    m_name = str.section('-',0,-2); /// set anyway since we got it now
    m_version = ret;
  #endif

    return ret;
}

QString QBsdPortInstPkg::comment()
{
  #ifndef SPONTANEOUS
    if (m_comment != "bpk_no_init")
        return m_comment;
  #endif

    QString ret = QBsdPort::global->runCmd("/usr/sbin/pkg_info -qc " + name() +
                                           "-" +version(), 1).at(0);
  #ifndef SPONTANEOUS
    m_comment = ret;
  #endif

    return ret;
}

QString QBsdPortInstPkg::www()
{
  #ifndef SPONTANEOUS
    if (m_www != "bpk_no_init")
        return m_www;
  #endif

    QString ret = QBsdPort::global->runCmd("/usr/sbin/pkg_info -qd " + name() +
                                           "-" + version() + " | grep \"WWW:" +
                                           "\" | awk '{ print $2 }'").at(0);
//                                           "&& /bin/sh -c '", 1).at(0);
    //VERY DIRTY HACK: ' escapes the sequence in runCmd
    ret.remove('\n');
    ret.remove(EOF);
    ret.remove(' ');
  #ifndef SPONTANEOUS
    m_www = ret;
  #endif

    return ret;
}

QString QBsdPortInstPkg::descr()
{
  #ifndef SPONTANEOUS
    if (m_descr != "bpk_no_init")
        return m_descr;
  #endif

    QString ret = QBsdPort::global->runCmd("/usr/sbin/pkg_info -qd " + name() +
                              "-" + version() + "| grep -v \"WWW:\"").join("\n");
  #ifndef SPONTANEOUS
    m_descr = ret;
  #endif

    return ret;
}

QString QBsdPortInstPkg::categories()
{
  #ifndef SPONTANEOUS
    if (m_categories != "bpk_no_init")
		return m_categories;
  #endif

    QString ret = parent()->origin().section('/',0,0);

  #ifndef SPONTANEOUS
    m_categories = ret;
  #endif

    return ret;
}

QStringList QBsdPortInstPkg::files()
{
  #ifndef SPONTANEOUS
    if (m_files.count() == 0)
	return m_files;
    if (m_files.at(0) != "bpk_no_init")
        return m_files;
//TODO tink about this whole contentsreading
    readContentsFile();

    //TODO TODO TODO CHANGE THIS
    return m_files;

  #else
    //THIS IS MUCH FASTER THAN THE ABOVE
    return QBsdPort::global->runCmd("/usr/bin/grep -v -e \"@\" -e \"%D\" " +
                 QBsdPort::global->pkgDbDir().absolutePath() + "/" +  name() +
                 "-" + version() + "/+CONTENTS");
  #endif
}
/*
QStringList QBsdPortInstPkg::reqs()
{
  #ifndef SPONTANEOUS
    if (m_reqs.count() == 0)
	return m_reqs;
    if (m_reqs.at(0) != "bpk_no_init")
        return m_reqs;

    readContentsFile();

    return m_reqs;

  #else

    return QBsdPort::global->runCmd("/usr/bin/grep -e \"@pkgdep\" " +
                  QBsdPort::global->pkgDbDir().absolutePath() + "/" + name() +
                  "-" + version() + "/+CONTENTS | /usr/bin/awk '{ print $2 }'");
  #endif
}*/

QBsdPort::QPortList QBsdPortInstPkg::reqsPtrList()
{
  #ifndef SPONTANEOUS
    if (m_reqsPtrList.isEmpty())
	return m_reqsPtrList;
    if (m_reqsPtrList.at(0) != 0)
        return m_reqsPtrList;

    readContentsFile();

    return m_reqsPtrList;

  #else

    QStringList strl = QBsdPort::global->runCmd("/usr/bin/grep -e \"DEPORIGIN\" " +
                   QBsdPort::global->pkgDbDir().absolutePath() + "/" + name() +
                   "-" + version() + "/+CONTENTS");
    QBsdPort* p;
    QPortList ret;
    for (int i = 0; i < strl.count(); ++i)
    {
        p = portWithOrigin(strl.at(i).remove("@comment DEPORIGIN:");
        if (p)
            ret.append(p);
    }

    return ret;
  #endif
}

QBsdPort::QPortList QBsdPortInstPkg::depsPtrList()
/// dependencies as qbsdport-pointers
{
    QPortList ret;
    QPortList l;
    for (int i = 0; i < deps().count(); ++i)
    {
        l = QBsdPort::global->portWithPkgName(deps().at(i),
                                              QBsdPortGlobal::INSTPKG);
        if (l.count()==1)
            ret.append(l.at(0));
        else if (l.count()>1)
        {
            ret.append(l.at(0));
            QBsdPort::global->emitWarning("depsPtrList() : " + name() + "-" + version() +
                         " has more than one match for " + deps().at(i) + ".");
        } else
            QBsdPort::global->emitWarning("depsPtrList() : " + name() + "-" + version() +
                         " has no match for " + deps().at(i) + ".");
    }
    return ret;
}

qint64  QBsdPortInstPkg::size()
{
  #ifndef SPONTANEOUS
    if (m_size != -1)
        return m_size;
  #endif

    qint64 ret = QBsdPort::global->runCmd("/usr/sbin/pkg_info -qs " + name() +
                                       "-" + version(), 1).at(0).toLongLong();
  #ifndef SPONTANEOUS
    m_size = ret;
  #endif

    return ret;
}

#ifndef SPONTANEOUS
void    QBsdPortInstPkg::readContentsFile()
{

    QFile contentsFile(QBsdPort::global->pkgDbDir().absolutePath()+ "/" + name()
                        + "-" + version() + "/+CONTENTS");
    char    buffer[512] = "";
    QString qbuffer;
    int     lineLength = 0;
    QString prefix="";
    QStringList /*reqsL,*/ filesL;
    QPortList reqsPtrListL;
    QBsdPort *p = 0;

    if ( (!contentsFile.exists()) || (!contentsFile.open(QFile::ReadOnly)) )
    {
        QBsdPort::global->emitCritical("readContentsFile() : "+ parent()->origin() +
                    " has No +CONTENTS or not readble.");
    } else
    {
        while (!contentsFile.atEnd())
        {
            lineLength = contentsFile.readLine(buffer, sizeof(buffer));

            if (lineLength == -1) /// error reading from +CONTENTS
            {
                contentsFile.close();
                QBsdPort::global->emitCritical("readContentsFile() : " + name() + "-" +
                            version() + "'s pkgdb entry broken.\nTry to " +
                            "pkg_delete -f ");
                return;
            } else
            {
                qbuffer = buffer;
                if (!qbuffer.endsWith('\n'))
                    QBsdPort::global->emitWarning("findAllInstalled() : "
                                            "a line not completely read.");
                else
                    qbuffer.remove('\n');

                /// file
                if ( (!qbuffer.startsWith("@")) &&
                            (!qbuffer.startsWith("%D")) )
                    filesL.append(prefix + "/" + qbuffer);
                /// dir change (usually == prefix)
                else if (qbuffer.startsWith("@cwd"))// && prefix=="")
                    prefix = qbuffer.remove("@cwd ");
                /// requirement in PKGNAME-form
//                else if (qbuffer.startsWith("@pkgdep"))
//                    reqsL.append(qbuffer.section(' ', -1,-1));
                /// requirement in ORIGIN-form
                else if (qbuffer.startsWith("@comment DEPORIGIN:"))
                {
                    if (p = QBsdPort::global->portWithOrigin(
                                qbuffer.remove("@comment DEPORIGIN:")) )
                        reqsPtrListL.append(p);
                    else
                        QBsdPort::global->emitWarning("readContentsFile() : " +
                                     parent()->origin() + " requires port" +
                                     "that doesnt exist.");
                }
            }
        }
    }
    contentsFile.close();

//    m_reqs = reqsL;
    m_reqsPtrList = reqsPtrListL;
    m_files = filesL;
}
#endif //SPONTANEOUS

QStringList QBsdPortInstPkg::deps()
{
  #ifndef SPONTANEOUS
    if (m_deps.count() == 0)
	return m_deps;
    if (m_deps.at(0) != "bpk_no_init")
        return m_deps;
  #endif

    QStringList ret;
    if (QFile::exists(QBsdPort::global->pkgDbDir().absolutePath() + "/" +
                      name() + "-" + version() + "/+REQUIRED_BY"))
        ret = QBsdPort::global->runCmd("/bin/cat " + QBsdPort::global->pkgDbDir().absolutePath()
                     + "/" + name() + "-" + version() + "/+REQUIRED_BY");

  #ifndef SPONTANEOUS
    m_deps = ret;
  #endif
    return ret;
}

QString QBsdPortInstPkg::message()
{
/*
  #ifndef SPONTANEOUS
    if (m_message != "bpk_no_init")
        return m_message;
  #endif
*/
    QString ret;
    if (QFile::exists(QBsdPort::global->pkgDbDir().absolutePath() + "/" +
                      name() + "-" + version() + "/+DISPLAY"))
        ret = QBsdPort::global->runCmd(
                     "/bin/cat " + QBsdPort::global->pkgDbDir().absolutePath()
                     + "/" + nv() + "/+DISPLAY").join("\n");
/*
  #ifndef SPONTANEOUS
    m_message = ret;
  #endif
*/
    return ret;
}

QString QBsdPortInstPkg::maint()
{
    /// n.a.
    return QString();
}

QString QBsdPortInstPkg::updWarning()
{
    QString ret;
    if (QFile::exists("/usr/sbin/pkg_updating"))
        ret = QBsdPort::global->runCmd( "/usr/sbin/pkg_updating -d " +
                                        QBsdPort::global->updDate().toString(
                                            "yyyyMMdd") + " " +
                                        parent()->origin()
                                       ).join("\n");
    return ret;
}

QStringList QBsdPortInstPkg::falseFiles()
{
  #ifndef SPONTANEOUS
    if (m_falseFiles.count() == 0)
	return m_falseFiles;
    if (m_falseFiles.at(0) != "bpk_no_init")
        return m_falseFiles;
  #endif
    QStringList ret = QStringList();
    /// protection against packages too large
    if ( (size() < QBsdPort::global->bigPkgSize())
        || (m_ignoreSizeForChecksums ) )
        ret = QBsdPort::global->runCmd(
                        "/usr/sbin/pkg_info -qg " + name() + "-" + version() );
  #ifndef SPONTANEOUS
    m_falseFiles = ret;
  #endif

    return ret;
}
void QBsdPortInstPkg::exportPkg(QString dest_file, bool recursive)
{
    if (proc())
    {
        QBsdPort::global->emitError("exportPkg() : " + name()+ "-" + version() + " already has" +
                   " process running. Export not started.");
        emit exportPkgFailed(0, "");
        return;
    }

    QDir tmpdir(QBsdPort::global->sessionDir().absolutePath() + "/" +
                parent()->origin());
    if (!tmpdir.exists())
        tmpdir.mkpath(tmpdir.absolutePath());

    m_proc = new QProcess(this);

    proc()->setProcessChannelMode(QProcess::MergedChannels);
    proc()->setStandardOutputFile(tmpdir.absolutePath() + "/" + name() + "-" +
                                version() + "_pkgcreate.log");

    QString cmd = "/usr/sbin/pkg_create";
    QStringList args;
    args.append("-v");
    if (recursive)
        args.append("-R");
    args.append("-b");
    args.append(name()+"-"+version());
    args.append(dest_file);

    connect(proc(), SIGNAL(finished(int,QProcess::ExitStatus)),
            this, SLOT(exportPkgFinished(int,QProcess::ExitStatus)));

    proc()->start(cmd, args, QIODevice::ReadOnly);

}

void QBsdPortInstPkg::exportPkgFinished(int exitCode,
                                                QProcess::ExitStatus exitStatus)
{
    if ( (exitCode == 0) && (exitStatus == 0) )
        emit exportPkgSuccessful();
    else
        emit exportPkgFailed(3,
                                QBsdPort::global->sessionDir().absolutePath() +
                                "/" + parent()->origin() + "/" + name() +
                                "-" + version() + "_pkgcreate.log");

    QBsdPort::global->cleanDelete(proc());
}

void QBsdPortInstPkg::backup()
{
    exportPkg(QBsdPort::global->sessionDir().absolutePath() + "/" +
                parent()->origin() + "/backup.tbz");
}

void QBsdPortInstPkg::remove(bool force)
{
    if (proc())
    {
        QBsdPort::global->emitError("remove() : " + name() + "-" + version() + " already has" +
                   " process running. Removal not started.");
        emit removeFailed(0,"");
        return;
    }

    QDir tmpdir(QBsdPort::global->sessionDir().absolutePath() + "/" +
                parent()->origin());
    if (!tmpdir.exists())
        tmpdir.mkpath(tmpdir.absolutePath());

    m_proc = new QProcess(this);

    proc()->setProcessChannelMode(QProcess::MergedChannels);
    proc()->setStandardOutputFile(tmpdir.absolutePath() + "/" + name() + "-" +
                                version() + "_remove.log");

    /// backup the +REQUIRED_BY file
    if (QFile::exists(QBsdPort::global->pkgDbDir().absolutePath() + "/" +
                      name() + "-" + version() + "/+REQUIRED_BY"))
        QFile::copy(QBsdPort::global->pkgDbDir().absolutePath() + "/" +
                    name() + "-" + version() + "/+REQUIRED_BY",
                    tmpdir.absolutePath() + "/+REQUIRED_BY");

    QString cmd = "/usr/sbin/pkg_delete";
    QStringList args;
    args.append("-v");
    if (force)
        args.append("-f");
    args.append(name()+"-"+version());

    connect(proc(), SIGNAL(finished(int,QProcess::ExitStatus)),
            this, SLOT(removeFinished(int,QProcess::ExitStatus)));

    proc()->start(cmd, args, QIODevice::ReadOnly);
}

void QBsdPortInstPkg::removeFinished(int exitCode,
                                                QProcess::ExitStatus exitStatus)
{
    if ( (exitCode == 0) && (exitStatus == 0) )
        emit removeSuccessful();
    else
        emit removeFailed(3, QBsdPort::global->sessionDir().absolutePath() +
                            "/" + name()+"-"+version() + "_removal.log");

    QBsdPort::global->cleanDelete(proc());
    parent()->findInst();
}

void QBsdPortInstPkg::reinplaceRequiredBy()
{
    QDir tmpdir(QBsdPort::global->sessionDir().absolutePath() + "/" +
                parent()->origin());
    if (!tmpdir.exists())
        return;

    if (QFile::exists(tmpdir.absolutePath() + "/+REQUIRED_BY"))
        QFile::copy(tmpdir.absolutePath() + "/+REQUIRED_BY",
                    QBsdPort::global->pkgDbDir().absolutePath() + "/" +
                    name() + "-" + version() + "/+REQUIRED_BY");
    else
        return;
    //TODO iterate through these packages and update their +CONTENTS
    // BEFORE RELEASE !!!
}


/************* QBsdPortAvBinPkg ***********************/
// if not spontaneuos we do not need to reimplement the get calls
#ifdef SPONTANEOUS
QString QBsdPortAvBinPkg::name()
{
    QStringList lbuffer = QBsdPort::global->runCmd("/usr/bin/grep /" +
                                 parent()->origin() + "\\| " +
                                 indexPath().absolutePath(),1).at(0).split('|');
    return lbuffer.at(0).section('-',0,-2);
}

QString QBsdPortAvBinPkg::version()
{
    QStringList lbuffer = QBsdPort::global->runCmd("/usr/bin/grep /" +
                                 parent()->origin() + "\\| " +
                                 indexPath().absolutePath(),1).at(0).split('|');
    return lbuffer.at(0).section('-',-1,-1);
}

QString QBsdPortAvBinPkg::comment()
{
    QStringList lbuffer = QBsdPort::global->runCmd("/usr/bin/grep /" +
                                 parent()->origin() + "\\| " +
                                 indexPath().absolutePath(),1).at(0).split('|');
    return lbuffer.at(3);
}

QString QBsdPortAvBinPkg::maint()
{
    QStringList lbuffer = QBsdPort::global->runCmd("/usr/bin/grep /" +
                                 parent()->origin() + "\\| " +
                                 indexPath().absolutePath(),1).at(0).split('|');
    return lbuffer.at(5);
}

QString QBsdPortAvBinPkg::categories()
{
    QStringList lbuffer = QBsdPort::global->runCmd("/usr/bin/grep /" +
                                 parent()->origin() + "\\| " +
                                 indexPath().absolutePath(),1).at(0).split('|');
    return lbuffer.at(6);
}

QString QBsdPortAvBinPkg::www()
{
    QStringList lbuffer = QBsdPort::global->runCmd("/usr/bin/grep /" +
                                 parent()->origin() + "\\| " +
                                 indexPath().absolutePath(),1).at(0).split('|');
    return lbuffer.at(9);
}

QStringList QBsdPortAvBinPkg::reqs()
{
    QStringList lbuffer = QBsdPort::global->runCmd("/usr/bin/grep /" +
                                 parent()->origin() + "\\| " +
                                 indexPath().absolutePath(),1).at(0).split('|');
    return lbuffer.at(8).split(' ');
    //TODO: find out what .at(7) is good for
}

QPortList QBsdPortAvBinPkg::reqsPtrList()/// dependencies as qbsdport-pointers
{
    QPortList ret;
    QPortList l;
    for (int i = 0; i < reqs().count(); ++i)
    {
        /// package kind matters -> 2
        l = QBsdPort::global->portWithPkgName(reqs.at(i),2);
        if (l.count()==1)
            ret.append(l.at(0));
        else if (l.count()>1)
        {
            ret.append(l.at(0));
            QBsdPort::global->emitWarning("reqsPtrList() : " + name() + "-" + version() +
                            " has more than one match for " + reqs.at(i) ".");
        } else
        {
            QBsdPort::global->emitWarning("reqsPtrList() : " + name() + "-" + version() +
                            " has no match for " + reqs.at(i) ".");
    }
    return ret;
}

#endif //N SPONTANEOUS

QBsdPort::QPortList QBsdPortAvBinPkg::reqsPtrList()
{

  #ifndef SPONTANEOUS
    if (m_reqsPtrList.isEmpty())
        return m_reqsPtrList;
    if (m_reqsPtrList.at(0) != 0)
        return m_reqsPtrList;
  #endif

    QPortList ret;

    QString str = QBsdPort::global->runCmd("/usr/bin/grep -e \""
    "|/usr/ports/" + parent()->origin() + "|\" /usr/ports/FTPINDEX",1).at(0);

    QStringList reqs =  str.section('|', 8,8).split(' ');

    QBsdPort* p = 0;
    QPortList searchResults;
    foreach (QString req, reqs)
    {
        searchResults = QBsdPort::global->portWithPkgName(req,
                                                      QBsdPortGlobal::AVBINPKG);

        if ((searchResults.count() > 0) && searchResults.at(0))
            ret.append(searchResults.at(0));
        else
            QBsdPort::global->emitWarning(
                "reqsPtrList(): no port found to pkg " +req.toLatin1());
    }

  #ifndef SPONTANEOUS
    m_reqsPtrList = ret;
  #endif

    return ret;
}

void    QBsdPortAvBinPkg::fetch()
{
    QDir tmpdir(QBsdPort::global->sessionDir().absolutePath() + "/" +
                parent()->origin());
    if (!tmpdir.exists())
        tmpdir.mkpath(tmpdir.absolutePath());

    for (int i = 0; i < QBsdPort::global->pkgDirs().count(); ++i)
    {
        if (QFile::exists(QBsdPort::global->pkgDirs().at(i) + "/" + name() +
                                                      "-" + version() + ".tbz"))
        {
            if (QFile::copy(QBsdPort::global->pkgDirs().at(i) + "/" + name() +
                             "-" +version() + ".tbz", tmpdir.absolutePath() +
                             "/" + name() + "-" + version() + ".tbz"))
            {
                emit fetchNoNeedTo();
                return;
            }
        }
    }

    if (fetchFtp)
    {
        QBsdPort::global->emitError("fetch() : " + name() + "-" + version() + " already " +
                   "fetching.");
        emit fetchFailed("already fetching.");
        return;
    }

    fetchFtp       = new QFtp(this);
    outfile        = new QFile(tmpdir.absolutePath() + "/" + name() + "-" +
                                version() + ".tbz");

    QString host = QBsdPort::global->pkgSite().section('/', 1, 1,
                                                     QString::SectionSkipEmpty);
    /// dir is part after host without "/Latest"
    QString dir = QBsdPort::global->pkgSite().section('/', 2, -2,
                                            QString::SectionSkipEmpty) + "/All";
    connect(fetchFtp, SIGNAL(dataTransferProgress(qint64,qint64)),
            this, SIGNAL(fetchProgress(qint64,qint64)));
    connect(fetchFtp, SIGNAL(done(bool)), this, SLOT(fetchFinished(bool)));
    fetchFtp->connectToHost(host);
    fetchFtp->login("anonymous");
    fetchFtp->cd(dir);
    fetchFtp->get(name()+"-"+version()+".tbz", outfile);
    fetchFtp->close();

    //TODO add possibility to abort transfer
}

void    QBsdPortAvBinPkg::fetchFinished(bool error)
{
    if (error)
        emit fetchFailed(fetchFtp->errorString());
    else
        emit fetchSuccessful();

    QBsdPort::global->cleanDelete(fetchFtp);
}

void    QBsdPortAvBinPkg::install()
{
    if (parent()->hasInst())
    {
        emit installFailed(1,"");
        return;
    }

    if (proc())
    {
        QBsdPort::global->emitError("install() : " + name() + "-" + version() + " already has" +
                   " process running. Install not started.");
        emit installFailed(0,"");
        return;
    }

    QDir tmpdir(QBsdPort::global->sessionDir().absolutePath() + "/" +
                parent()->origin());
    if (!tmpdir.exists())
        tmpdir.mkpath(tmpdir.absolutePath());

    m_proc = new QProcess(this);

    proc()->setProcessChannelMode(QProcess::MergedChannels);
    proc()->setStandardOutputFile(tmpdir.absolutePath() + "/" + name() + "-" +
                                version() + "_remove.log");


    QString cmd = "/usr/sbin/pkg_add";
    QStringList args;
    args.append("-v");
    args.append(name()+"-"+version());

    connect(proc(), SIGNAL(finished(int,QProcess::ExitStatus)),
            this, SLOT(installFinished(int,QProcess::ExitStatus)));

    proc()->start(cmd, args, QIODevice::ReadOnly);
}

void    QBsdPortAvBinPkg::installFinished(int exitCode,
                                                QProcess::ExitStatus exitStatus)
{
    if ( (exitCode == 0) && (exitStatus == 0) )
        emit installSuccessful();
    else
        emit installFailed(3,QBsdPort::global->sessionDir().absolutePath() +
                            "/" + name()+"-"+version() + "_install.log");

    QBsdPort::global->cleanDelete(proc());
    parent()->findInst();
    if (parent()->hasInst())
        parent()->inst()->reinplaceRequiredBy();
}

/*********** QBsdPortAvSrcPkg ******************/


QString    QBsdPortAvSrcPkg::name()
{
  #ifndef SPONTANEOUS
    if (m_name != "bpk_no_init")
        return m_name;
  #endif

    QString str = QBsdPort::global->getMakeEnv(
                                 QBsdPort::global->portsDir().absolutePath() +
                                 "/" + parent()->origin(),"PKGNAME",totalEnv());
    QString ret = str.section('-',0,-2);

  #ifndef SPONTANEOUS
    m_name = ret;
    m_version = str.section('-',-1,-1); /// set anyway since we got it now
  #endif

    return ret;
}

QString    QBsdPortAvSrcPkg::version()
{
  #ifndef SPONTANEOUS
    if (m_version != "bpk_no_init")
        return m_version;
  #endif

    QString str = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                             "/" + parent()->origin(),
                                            "PKGNAME", totalEnv());
    QString ret = str.section('-',-1,-1);

  #ifndef SPONTANEOUS
    m_version = ret;
    m_name = str.section('-',0,-2); /// set anyway since we got it now
  #endif

    return ret;
}

QString    QBsdPortAvSrcPkg::comment()
{
  #ifndef SPONTANEOUS
    if (m_comment != "bpk_no_init")
        return m_comment;
  #endif

    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                             "/" + parent()->origin(),
                                            "COMMENT", totalEnv());
  #ifndef SPONTANEOUS
    m_comment = ret;
  #endif

    return ret;
}

QString    QBsdPortAvSrcPkg::maint()
{
  #ifndef SPONTANEOUS
    if (m_maint != "bpk_no_init")
        return m_maint;
  #endif

    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                             "/" + parent()->origin(),
                                            "MAINTAINER", totalEnv());
  #ifndef SPONTANEOUS
    m_maint = ret;
  #endif

    return ret;
}

QString    QBsdPortAvSrcPkg::categories()
{
  #ifndef SPONTANEOUS
    if (m_categories != "bpk_no_init")
        return m_categories;
  #endif

    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                             "/" + parent()->origin(),
                                             "CATEGORIES", totalEnv());
  #ifndef SPONTANEOUS
    m_categories = ret;
  #endif

    return ret;
}
/*
QStringList   QBsdPortAvSrcPkg::reqs()
{
  #ifndef SPONTANEOUS
    if (m_reqs.isEmpty())
        return m_reqs;
    if (m_reqs.at(0) != "bpk_no_init")
        return m_reqs;
  #endif

    QStringList ret;
    for (int i = 0; i < reqsPtrList().count(); ++i)
        if (reqsPtrList().at(i)->hasAvSrc())
            ret.append(reqsPtrList().at(i)->avSrc()->name() + "-" +
                       reqsPtrList().at(i)->avSrc()->version());

  #ifndef SPONTANEOUS
    m_reqs = ret;
  #endif

    return ret;
}*/

QString QBsdPortAvSrcPkg::descr()
{
  #ifndef SPONTANEOUS
    if (m_descr != "bpk_no_init")
        return m_descr;
  #endif
    //TODO find out if checking for existance of origin/pkg-descr is much
    // faster and use it instead if it is the case
    QString file = QBsdPort::global->getMakeEnv(
                                    QBsdPort::global->portsDir().absolutePath()+
                                    "/" + parent()->origin(),
                                    "DESCR", totalEnv());
    QString ret = QBsdPort::global->runCmd("/usr/bin/grep -v \"WWW:\" \"" +
                                           file + "\"").join("\n");
  #ifndef SPONTANEOUS
    m_descr = ret;
  #endif

    return ret;
}

QString QBsdPortAvSrcPkg::www()
{
  #ifndef SPONTANEOUS
    if (m_www != "bpk_no_init")
        return m_www;
  #endif

    QString file = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                             "/" + parent()->origin(),
                                            "DESCR", totalEnv());
    QString ret = QBsdPort::global->runCmd("/usr/bin/grep \"WWW:\" \"" +
                                           file + "\"").at(0);
  #ifndef SPONTANEOUS
    m_www = ret;
  #endif

    return ret;
}

qint64  QBsdPortAvSrcPkg::size()
{
    /// n.a.
    return 0;

}

QBsdPort::QPortList   QBsdPortAvSrcPkg::reqsPtrList()
{
  #ifndef SPONTANEOUS
    if (m_reqsPtrList.isEmpty())
        return m_reqsPtrList;
    if (m_reqsPtrList.at(0) != 0)
        return m_reqsPtrList;
  #endif

    QPortList   ret;
    QStringList l;
    QString     tmp;
    QBsdPort*   p;

/*    /// LIB_DEPENDS
    tmp = QBsdPort::global->getMakeEnv(
                            QBsdPort::global->portsDir().absolutePath() +
                            "/" + parent()->origin(), "LIB_DEPENDS",totalEnv());
    l = tmp.split(' ',QString::SkipEmptyParts);
    for (int i = 0; i < l.count(); ++i)
        if (p = QBsdPort::global->portWithOrigin(
                                   l.at(i).section(':',1,1).section('/',-2,-1)))
            ret.append(p);
    /// RUN_DEPENDS
    tmp = QBsdPort::global->getMakeEnv(
                             QBsdPort::global->portsDir().absolutePath()+ "/" +
                             parent()->origin(), "RUN_DEPENDS",
                             totalEnv());
    l = tmp.split(' ',QString::SkipEmptyParts);
    for (int i = 0; i < l.count(); ++i)
        if (p = QBsdPort::global->portWithOrigin(
                                   l.at(i).section(':',1,1).section('/',-2,-1)))
//            if (!ret.contains(p))
                ret.append(p);
    /// BUILD_DEPENDS
    tmp = QBsdPort::global->getMakeEnv(
                             QBsdPort::global->portsDir().absolutePath()+ "/" +
                             parent()->origin(), "BUILD_DEPENDS",
                             totalEnv());
    l = tmp.split(' ',QString::SkipEmptyParts);
    for (int i = 0; i < l.count(); ++i)
        if (p = QBsdPort::global->portWithOrigin(
                                   l.at(i).section(':',1,1).section('/',-2,-1)))
//            if (!ret.contains(p))
                ret.append(p);
    /// FETCH_DEPENDS
    tmp = QBsdPort::global->getMakeEnv(
                             QBsdPort::global->portsDir().absolutePath() + "/" +
                             parent()->origin(), "FETCH_DEPENDS",
                             totalEnv());
    l = tmp.split(' ',QString::SkipEmptyParts);
    for (int i = 0; i < l.count(); ++i)
        if (p = QBsdPort::global->portWithOrigin(
                                   l.at(i).section(':',1,1).section('/',-2,-1)))
//            if (!ret.contains(p))
                ret.append(p);
    /// EXTRACT_DEPENDS
    tmp = QBsdPort::global->getMakeEnv(
                             QBsdPort::global->portsDir().absolutePath()+ "/" +
                             parent()->origin(), "EXTRACT_DEPENDS",
                             totalEnv());
    l = tmp.split(' ',QString::SkipEmptyParts);
    for (int i = 0; i < l.count(); ++i)
        if (p = QBsdPort::global->portWithOrigin(
                                   l.at(i).section(':',1,1).section('/',-2,-1)))
//            if (!ret.contains(p))
                ret.append(p);
    /// PATCH_DEPENDS
    tmp = QBsdPort::global->getMakeEnv(
                             QBsdPort::global->portsDir().absolutePath()+ "/" +
                             parent()->origin(), "PATCH_DEPENDS",
                             totalEnv());
    l = tmp.split(' ',QString::SkipEmptyParts);
    for (int i = 0; i < l.count(); ++i)
        if (p = QBsdPort::global->portWithOrigin(
                                   l.at(i).section(':',1,1).section('/',-2,-1)))
//            if (!ret.contains(p))
                ret.append(p);*/

    /// ALL_DEPENDS
    QStringList depends_types;
    depends_types << "LIB_DEPENDS" << "RUN_DEPENDS" << "BUILD_DEPENDS"
                  << "FETCH_DEPENDS" << "EXTRACT_DEPENDS" << "PATCH_DEPENDS"
                  << "DEPENDS";
    tmp = QBsdPort::global->getMakeEnv(
                            QBsdPort::global->portsDir().absolutePath() +
                            "/" + parent()->origin(), depends_types,totalEnv());
    l = tmp.split(' ',QString::SkipEmptyParts);
    for (int i = 0; i < l.count(); ++i)
        if (p = QBsdPort::global->portWithOrigin(
                                   l.at(i).section(':',1,1).section('/',-2,-1)))
            ret.append(p);
    ret = QBsdPort::global->removeDuplicates((QList<QBsdPort*>)ret);

    /* get recursive requirements
    foreach(QBsdPort* req, ret)
    {
        if (req->avSrc())
            ret+=req->avSrc()->reqsPtrList();
    }

    ret = QBsdPort::global->removeDuplicates((QList<QBsdPort*>)ret);*/
  #ifndef SPONTANEOUS
    m_reqsPtrList = ret;
  #endif

    return ret;
}

QStringList   QBsdPortAvSrcPkg::files()
{
/*
  #ifndef SPONTANEOUS
    if (m_files.isEmpty())
        return m_files;
    if (m_files.at(0) != "bpk_no_init")
        return m_files;
  #endif
*/
//    QStringList ret;

/*    QString pref = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                                "/" + parent()->origin(),
                                                "PREFIX", totalEnv(), 1);
*/
/*    if (QFile::exists(QBsdPort::global->portsDir().absolutePath()+ "/" +
                      parent()->origin() + "/pkg-plist") )
        ret = QBsdPort::global->runCmd("/usr/bin/grep -v \"@\" " +
                                  QBsdPort::global->portsDir().absolutePath() +
                                  "/" + parent()->origin() + "/pkg-plist");
    else
        ret = QBsdPort::global->getMakeEnv(
                                   QBsdPort::global->portsDir().absolutePath() +
                                   "/" + parent()->origin(), "PLIST_FILES",
                                   totalEnv()).split(' ');
*/
    if (QFile::exists(QBsdPort::global->portsDir().absolutePath()+ "/" +
                      parent()->origin() + "/pkg-plist") )
        return QBsdPort::global->runCmd("/usr/bin/grep -v \"@\" " +
                                  QBsdPort::global->portsDir().absolutePath() +
                                  "/" + parent()->origin() + "/pkg-plist");
    else
        return QBsdPort::global->getMakeEnv(
                                   QBsdPort::global->portsDir().absolutePath() +
                                   "/" + parent()->origin(), "PLIST_FILES",
                                   totalEnv()).split(' ');
    //TODO: process the list to replace %%VAR%% with contents
/*
  #ifndef SPONTANEOUS
    m_files = ret;
  #endif

    return ret;*/
}

QString     QBsdPortAvSrcPkg::broken()
{
  #ifndef SPONTANEOUS
    if (m_broken != "bpk_no_init")
        return m_broken;
  #endif
    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                               "/" + parent()->origin(),
                                               "BROKEN", totalEnv());
  #ifndef SPONTANEOUS
    m_broken = ret;
  #endif

    return ret;
}

QString     QBsdPortAvSrcPkg::forbidden()
{
  #ifndef SPONTANEOUS
    if (m_forbidden != "bpk_no_init")
        return m_forbidden;
  #endif

    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                               "/" + parent()->origin(),
                                               "FORBIDDEN", totalEnv());
  #ifndef SPONTANEOUS
    m_forbidden = ret;
  #endif

    return ret;
}

QString     QBsdPortAvSrcPkg::ignore()
{
  #ifndef SPONTANEOUS
    if (m_ignore != "bpk_no_init")
        return m_ignore;
  #endif

    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                               "/" + parent()->origin(),
                                               "IGNORE", totalEnv());
  #ifndef SPONTANEOUS
    m_ignore = ret;
  #endif

    return ret;
}

QString     QBsdPortAvSrcPkg::restricted()
{
  #ifndef SPONTANEOUS
    if (m_restricted != "bpk_no_init")
        return m_restricted;
  #endif

    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                               "/" + parent()->origin(),
                                               "RESTRICTED", totalEnv());
  #ifndef SPONTANEOUS
    m_restricted = ret;
  #endif

    return ret;
}

QString     QBsdPortAvSrcPkg::deprecated()
{
  #ifndef SPONTANEOUS
    if (m_deprecated != "bpk_no_init")
        return m_deprecated;
  #endif

    QString ret = QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                               "/" + parent()->origin(),
                                               "DEPRECATED", totalEnv());
  #ifndef SPONTANEOUS
    m_deprecated = ret;
  #endif

    return ret;
}

QList <QBsdPortAvSrcPkg::_option> QBsdPortAvSrcPkg::options()
{
    //DEBUG
    return QList <QBsdPortAvSrcPkg::_option>();
  #ifndef SPONTANEOUS
    if (m_options.isEmpty())
        return m_options;
    if (m_options.at(0).var != "bpk_no_init")
        return m_options;
  #endif

    /// THIS IS FUCKED UP AND DOESNT WORK

    QList   <_option> ret;

    QStringList localEnv; ///without OPTIONS
    localEnv = QBsdPort::global->defaultBuildEnv();
    for (int i = 0; i < otherEnv().count(); ++i)
        if ( (otherEnv().at(i).count('=') == 1) &&
                (otherEnv().at(i).count(' ') == 0) ) /// check syntax
            localEnv.append(otherEnv().at(i));

    QString tmp =
     QBsdPort::global->getMakeEnv(QBsdPort::global->portsDir().absolutePath()+
                                               "/" + parent()->origin(),
                                               "OPTIONS", localEnv);
    tmp.replace('\t',' ');
    QStringList l   = tmp.split(' ', QString::SkipEmptyParts);
    int     c       = -1;
    _option *o      = 0;
    bool    with    = false;
    bool    without = false;

    for (int i = 0; i < l.count(); ++i)
    {
        if (c == 3) /// reached a new VAR
            c = 0;
        else
            ++c;
        if (!l.at(i).isEmpty())
        {
            if (c==0)           /// VAR
            {
                o       = new _option;
                ret.append(*o);
                o->var  = l.at(i);
            } else if (c==1)    /// DESCR
            {
                o = &ret.last();
                o->descr = l.at(i);
            } else if (c==2)      /// Default Value from Makefile
            {
                o = &ret.last();
                o->defaultVal = (l.at(i) == "on");

                /// get current value
                with = (QBsdPort::global->getMakeEnv(
                        QBsdPort::global->portsDir().absolutePath() + "/" +
                        parent()->origin(), "WITH_" + o->var, localEnv) != "");
                without = (QBsdPort::global->getMakeEnv(
                        QBsdPort::global->portsDir().absolutePath() + "/" +
                        parent()->origin(), "WITHOUT_" + o->var, localEnv)!= "");
                if (!with && !without)
                    o->currentVal = o->defaultVal;
                else if (with)
                    o->currentVal = true;
                else
                    o->currentVal = false;
                /// and set toBeVal
                o->toBeVal = o->currentVal;
            }
        }
    }

  #ifndef SPONTANEOUS
    m_options = ret;
  #endif

    return ret;
}

QStringList QBsdPortAvSrcPkg::totalEnv()
{
    QStringList ret;
    for (int i = 0; i < options().count(); ++i)
    {
        if (options().at(i).currentVal != options().at(i).toBeVal)
        {
            if (options().at(i).toBeVal)
            {
                ret.append("WITH_" + options().at(i).var + "=True");
                ret.append("WITHOUT_" + options().at(i).var + "=");
            } else
            {
                ret.append("WITH_" + options().at(i).var + "=");
                ret.append("WITHOUT_" + options().at(i).var + "=True");
            }
        }
    }

    ret+= QBsdPort::global->defaultBuildEnv();

    for (int i =0; i < otherEnv().count(); ++i)
        if ( (otherEnv().at(i).count('=') == 1) &&
                (otherEnv().at(i).count(' ') == 0) ) /// check syntax
            ret.append(otherEnv().at(i));
    return ret;
}


