/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __BSDPORT_H__
#define __BSDPORT_H__

#include <QString>
#include <QStringList>
#include <QFile>
//#include <Qt/qftp.h>
#include <QFtp>
#include <QDir>
#include <QThread>
#include <QProcess>
#include <QTime>
#include <QUrlInfo>
#include <QMetaType>
//#include "bsdportglobal.h"

class QBsdPort;
class QBsdPortGlobal;
class QBsdPortPkg;
class QBsdPortInstPkg;
class QBsdPortAvSrcPkg;
class QBsdPortAvBinPkg;
typedef QList <QBsdPort*> QPortList;
typedef QList <QBsdPortPkg*> QPkgList;

//DEBUG
#define PORTAUDIT

/** concerning structure of this file and classes:

    QBsdPort global stores all data that is static and global to the other
    classes. it is necessary because i want signals and slots for the
    global functions which would not be available if the were static members
    of QBsdPort.

    QBsdPort is the main object and stores all data concerning one port,
    i.e. PORT REFERS TO A PIECE OF SOFTWARE HAVING A UNIQUE LOCATION
    (ORIGIN) IN THE FREEBSD PORTSTREE.

    a QBsdPort object has three possible sub-objects, one each of
    QBsdPortInstPkg, QBsdPortAvSrcPkg + QBsdPortAvBinPkg - who all
    inherit from QBsdPortPkg (who is never used directly).

    QBsdPortInstPkg stores all relevant data of the installed package
        of the port if one is installed.

    QBsdPortAvSrcPkg represents what is usually referred to as "port",
        it stores all data available for the source install.

    QBsdPortAvBinPkg is the same for the binary tarball available on ftp

**/


class QBsdPort : public QObject
{

Q_OBJECT

private:
    QString m_origin;
    QBsdPortInstPkg	*m_inst;
    QBsdPortAvSrcPkg    *m_avSrc;
    QBsdPortAvBinPkg    *m_avBin;

public:
typedef QList <QBsdPort*> QPortList;
typedef QList <QBsdPortPkg*> QPkgList;

    static QBsdPortGlobal *global;

    QBsdPort(QString origin = "bpk_no_init");

    QBsdPort(const QBsdPort&);

    ~QBsdPort()
    {
        setInst(0);
        setAvSrc(0);
        setAvBin(0);
    }

    /** find calls **/
    void                findInst();
    void                findAvSrc();
    void                findAvBin();

    /** "check" calls **/
     bool     hasInst()   const { return (m_inst != 0); }
     bool     hasAvSrc()  const { return (m_avSrc != 0); }
     bool     hasAvBin()  const { return (m_avBin != 0); }

    /** get calls **/
    QString       const     origin() const { return m_origin; }
    QBsdPortInstPkg 	*inst() const;
    QBsdPortAvSrcPkg	*avSrc() const;
    QBsdPortAvBinPkg	*avBin() const;

    /// convenient for sorting
//    operator QString() { return origin(); }

/*    bool operator<(const QBsdPort& p)
    {
        return this->origin() < p.origin();
    }
*/
public slots:
    /** set calls **/
    void    setOrigin(const QString& newValue)		{ m_origin = newValue; }
    void    setInst(QBsdPortInstPkg* newValue);
    // TODO longterm: remove the following and register the types
    void    setInstQObject(QObject* newValue);
    void    setAvSrc(QBsdPortAvSrcPkg *newValue);
    void    setAvBin(QBsdPortAvBinPkg* newValue);
    // TODO longterm: remove the following and register the types
    void    setAvBinQObject(QObject* newValue);


//    void    emitStatusChanged();

    ///changed
     void    emitPortState();

    // 3 original single
//     void    emitInstState();
//     void    emitAvSrcState();
//     void    emitAvBinState();
// 
//     // 2 new single
//     void    emitPkgState();
    static inline bool lessThan(QBsdPort* p1, QBsdPort* p2)
    {
        return p1->origin() < p2->origin();
    }
signals:
    void    statusChanged(QBsdPort*, const QString&,
                                     const QString&, const QString&);
    /// name-version of new pkg or '''' for no pkg
//     void    instState(const QString&);
//     /// name-version of new pkg or '''' for no pkg
//     void    avSrcState(const QString&);
//     /// name-version of new pkg or '''' for no pkg
//     void    avBinState(const QString&);
// 
//     void	pkgState(const QString&,const QString&, const QString&);
};


class QBsdPortPkg: public QObject
{
Q_OBJECT
public:
    typedef QList <QBsdPort*> QPortList;
    typedef QList <QBsdPortPkg*> QPkgList;

protected:
#ifndef SPONTANEOUS
    QString m_version,	/// version
            m_name,	/// normally same on pkgs of one port but not always
//            m_prefix,	/// prefix this package was/will be installed to
            m_comment,	/// one-line comment
            m_www,	/// webpage
            m_descr,	/// long description
            m_categories, /// categories for this port
            m_maint;	 /// port-maintainers email-address
//    QStringList m_reqs,	 /// requirements as "name-version"-list
//                m_files; /// contained/installed files
    QPortList m_reqsPtrList;

  #ifdef PORTAUDIT
    QStringList m_vulna; /// known vulnarabilities, stored as "description|reference"
  #endif
    qint64	m_size;		/// size occupied by files in package
#endif

    QBsdPort *m_parent;	/// pointer to parent
    QProcess *m_proc;

public:
    QBsdPortPkg(QBsdPort* parent = 0);
    QBsdPortPkg(const QBsdPortPkg&);

    ~QBsdPortPkg()
    {
        stopProc();

    }
 #ifndef SPONTANEOUS
    virtual  void reset()
    {
        m_version   = "bpk_no_init";
        m_version.squeeze();
        m_name      = "bpk_no_init";
        m_name.squeeze();
//        m_files.clear();
//        m_files.append("bpk_no_init");
//        m_prefix    = "bpk_no_init";
        m_comment   = "bpk_no_init";
        m_comment.squeeze();
        m_www       = "bpk_no_init";
        m_www.squeeze();
        m_descr     = "bpk_no_init";
        m_descr.squeeze();
        m_categories = "bpk_no_init";
        m_categories.squeeze();
        m_maint     = "bpk_no_init";
        m_maint.squeeze();
        #ifdef PORTAUDIT
        m_vulna.clear();
        m_vulna.append("bpk_no_init");
        #endif
        m_size      = -1;
        m_reqsPtrList.clear();
        m_reqsPtrList.append((QBsdPort*)0);
//        m_reqs.clear();
//        m_reqs.append("bpk_no_init");
    }
  #endif




    /** get calls **/
    QBsdPort* parent()    { return m_parent; }
    QProcess* proc()      { return m_proc; }
    /** these get calls should be reimplemented by children IMPORTANT: the
     get-calls actually retrieve the info if it isnt there yet **/
    virtual QString version()     { return m_version; }
    virtual QString name()        { return m_name; }
	QString nv()				  { return name() + "-" + version(); }
    virtual QStringList files();
//    virtual QStringList reqs()    { return m_reqs; }
//    virtual QString prefix();
    virtual QString comment()     { return m_comment; }
    virtual QString descr()       { return m_descr; }
    virtual QString www()         { return m_www; }
    virtual QString categories()  { return m_categories; }
    virtual QString maint()       { return m_maint; }
    virtual qint64  size()        { return m_size; }
  #ifdef PORTAUDIT
    QStringList vulna();
  #endif
    virtual QPortList reqsPtrList();

    virtual int reqsMet();

    /** set calls **/
    void    setParent(QBsdPort* newValue) { m_parent = newValue; }
  #ifndef SPONTANEOUS
    /// just needed for creation of errorpkgs
    void	setVersion(QString newValue)	{ m_version = newValue; }
    void	setName(QString newValue)	{ m_name = newValue; }
//    void	setFiles(QStringList newValue)	{ m_files = newValue; }
//    void	setReqs(QStringList newValue)	{ m_reqs = newValue; }
  #ifdef PORTAUDIT
    void	setVulna(QStringList newValue)	{ m_vulna = newValue; }
  #endif
//   void	setPrefix(QString newValue)	{ m_prefix = newValue; }
    void	setComment(QString newValue)	{ m_comment = newValue; }
    void	setDescr(QString newValue)	{ m_descr = newValue; }
    void	setWWW(QString newValue)	{ m_www = newValue; }
    void	setSize(qint64 newValue)	{ m_size = newValue; }
    void	setCategories(QString newValue){ m_categories = newValue; }
    void	setMaint(QString newValue)	{ m_maint = newValue; }
  #endif


signals:
  #ifndef SPONTANEOUS
    void    nameSignal(QString);
    void    versionSignal(QString);
    void    commentSignal(QString);
    void    wwwSignal(QString);
    void    descrSignal(QString);
    void    categoriesSignal(QString);
    void    maintSignal(QString);
    void    sizeSignal(qint64);
 //   void    reqsSignal(QStringList);
    void    filesSignal(QStringList);
  #ifdef PORTAUDIT
    void    vulnaSignal(QStringList);
  #endif
//    void    reqsPtrListSignal(QPortList);
    void    reqsPtrListSignal(const QPortList&);
  #endif


public slots:
  #ifndef SPONTANEOUS
    void nameSlot()          { emit nameSignal(name()); }
    void versionSlot()       { emit versionSignal(version()); }
    void commentSlot()       { emit commentSignal(comment()); }
    void wwwSlot()           { emit wwwSignal(www()); }
    void descrSlot()         { emit descrSignal(descr()); }
    void categoriesSlot()    { emit categoriesSignal(categories()); }
    void maintSlot()         { emit maintSignal(maint()); }
    void sizeSlot()          { emit sizeSignal(size()); }
//    void reqsSlot()          { emit reqsSignal(reqs()); }
    void filesSlot()         { emit filesSignal(files()); }
  #ifdef PORTAUDIT
    void vulnaSlot()         { emit vulnaSignal(vulna()); }
  #endif
    void reqsPtrListSlot()   { emit reqsPtrListSignal(reqsPtrList()); }

    virtual void allSlot() /// emit all contents as signals
    {
        nameSlot();
        versionSlot();
        commentSlot();
        wwwSlot();
        descrSlot();
        categoriesSlot();
        maintSlot();
        sizeSlot();
        reqsPtrListSlot();
        filesSlot();
    #ifdef PORTAUDIT
        vulnaSlot();
    #endif
    }
  #endif

    void stopProc();
};

class QBsdPortInstPkg : public QBsdPortPkg
{
Q_OBJECT
private:

  #ifndef SPONTANEOUS
    /** additional members for installed versions **/
/*    QString m_message,	/// post installation message
            m_updWarning; /// warnings from the updating file
*/    QStringList m_deps,	/// dependencies as "name-version"-list
                m_falseFiles;
    //TODO remove the below
    QStringList m_files;
    /// files with mismatching md5s, an ! after the filepath marks missing files
    bool m_ignoreSizeForChecksums;
    /// checksum calculation is deactivated for pkgs > 10MB
    /// this knob overwrites that bahviouf


    void    readContentsFile();
  #endif


public:
    //TODO: check if this actually calls this function's reset or bsdportpkg's
    QBsdPortInstPkg() : QBsdPortPkg() { reset(1); }
    QBsdPortInstPkg(QBsdPort* parent) : QBsdPortPkg(parent) { reset(1); }

 #ifndef SPONTANEOUS
    void reset(bool init=0)
    {
        if (!init) /// on creation those members have been set by QBsdPortPkg()
            this->QBsdPortPkg::reset();

        /// instPkg specific:
        m_deps.clear();
        m_deps.append("bpk_no_init");
/*        m_message       = "bpk_no_init";
        m_message.squeeze();
        m_updWarning    = "bpk_no_init";
        m_updWarning.squeeze();
*/        m_falseFiles.clear();
        m_falseFiles.append("bpk_no_init");
        m_ignoreSizeForChecksums = false;

    }
  #endif


    QString version()  ;
    QString name()     ;
    QStringList files();
//    virtual QStringList reqs() ;
//    virtual QString prefix();
    QString comment()  ;
    QString descr()    ;
    QString www()      ;
    QString categories();
    QString maint()    ;
    qint64  size()     ;

    /** get calls, note that these also retrieve info when it isnt there **/
    QStringList     deps();
    QString         message();
    QString         updWarning();
    QStringList     falseFiles();
    QPortList       reqsPtrList();
    QPortList       depsPtrList();

    void    setDeps(QStringList newValue)   { m_deps = newValue; }
//    void    setMessage(QString newValue)    { m_message = newValue; }
//    void    setUpdWarning(QString newValue) { m_updWarning = newValue; }
    void    setFalseFiles(QStringList newValue) { m_falseFiles = newValue; }
//    void    setDepsPtrList(QPortList newValue) { m_depsPtrList = newValue; }

   void reinplaceRequiredBy();


signals:
  #ifndef SPONTANEOUS
    void depsSignal(const QStringList&);
    void depsPtrListSignal(const QPortList&);
    void messageSignal(const QString&);
    void updWarningSignal(const QString&);
    void falseFilesSignal(const QStringList&);
  #endif

    void removeSuccessful();
    void removeFailed(int,QString);
    void exportPkgSuccessful();
    void exportPkgFailed(int,QString);
    /** *Failed int reason, QString is path to log
        0 = already a process running for this pkg
        3 = process returned code != 0 **/

public slots:

    /** do something with the pkg **/
    void    backup();			/// frame around exportPkg
    void    exportPkg(QString, bool=false);
    /// create a tarball from this package
    void    remove(bool force);		/// pkg_delete (-f)

  #ifndef SPONTANEOUS
    void depsSlot()	 { emit depsSignal(deps()); }
    void depsPtrListSlot(){ emit depsPtrListSignal(depsPtrList()); }
    void messageSlot()	 { emit messageSignal(message()); }
    void updWarningSlot() { emit updWarningSignal(updWarning()); }
    void falseFilesSlot() { emit falseFilesSignal(falseFiles()); }


    void setIgnoreSizeForChecksums(bool newValue)
    {
        m_ignoreSizeForChecksums = newValue;
    }

    virtual void allSlot() /// emit all contents as signals
    {
        this->QBsdPortPkg::allSlot();

        depsSlot();
        depsPtrListSlot();
        messageSlot();
        updWarningSlot();
        falseFilesSlot();
    }
  #endif

    void removeFinished(int, QProcess::ExitStatus);

    void exportPkgFinished(int, QProcess::ExitStatus);
};

class QBsdPortAvBinPkg : public QBsdPortPkg //right now avbin doesnt differ from parent
{
Q_OBJECT

private:
    QFtp *fetchFtp;
    QFile *outfile;

public:
    //TODO: check if this actually calls this function's reset or bsdportpkg's
    QBsdPortAvBinPkg() : QBsdPortPkg() { init(); }
    QBsdPortAvBinPkg(QBsdPort* parent) : QBsdPortPkg(parent) { init(); }

    void init()
    {
        connect(this, SIGNAL(fetchNoNeedTo()), this, SIGNAL(fetchSuccessful()));
        fetchFtp = 0;
        outfile = 0;
    }

    ~QBsdPortAvBinPkg()
    {
        delete fetchFtp;
        delete outfile;
    }

    QBsdPort::QPortList reqsPtrList();

public slots:
    /** do something with the pkg **/
    void    fetch();	/// fetch the package from specified mirrors
    void    install();	/// pkg_add it

    void    fetchFinished(bool);
    void    installFinished(int,QProcess::ExitStatus);

signals:
    void    fetchNoNeedTo();  /// found pkg in pkgDir, no need to fetch
    void    fetchProgress(qint64 progress, qint64 total);
    void    fetchSuccessful();
    void    fetchFailed(QString);
    /** QString is reason for failure **/

    void    installStarted();
    void    installSuccessful();
    void    installFailed(int, QString);
    /** *Failed int reason
        0 = already a process running for this pkg
        1 = port already has an instPkg
        3 = process returned code != 0 **/

/*  #ifndef SPONTANEOUS
    /** threaded get calls **
     void bg_complete() /// get all information that isnt already loaded
    {
        if (bg_mutex.tryLock())
            new QBgFetcher(this, false);
    }
     void bg_reloadAll() /// reload all members
    {
        if (bg_mutex.tryLock())
            new QBgFetcher(this, true);
    }
  #endif*/
};

class QBsdPortAvSrcPkg : public QBsdPortPkg
{
Q_OBJECT
private:
    /** additional members for installed versions **/
    QString	/// certain flags for ports
            m_broken,
            m_forbidden,
            m_ignore,
            m_deprecated,
            m_restricted;

    struct _option
    {
        QString var;    /// name of var e.g. MYSQL
        QString descr;  /// description
        bool    defaultVal; /** defaults from makefile without overrides from
                                         make.conf or previously set options **/
        bool    currentVal; /** = defaultVal if none of  WITH_ and WITHOUT_ are
                            set;
                            = 1  if both or only WITH_ is set;
                            = 0  if only WITHOUT_ is.**/
        bool    toBeVal; /// desired value for build
    };

    QList <_option> m_options;
    QStringList m_otherEnv; /// additional user-specified make environment

    /*TODO-LONGTERM replace with single LIB_* RUN_* ...requirements **/

    /// determine the overall environmentvars that are to be set
    QStringList totalEnv();

public:
    //TODO: check if this actually calls this function's reset or bsdportpkg's
    QBsdPortAvSrcPkg() : QBsdPortPkg() { reset(1); }
    QBsdPortAvSrcPkg(QBsdPort* parent) : QBsdPortPkg(parent) { reset(1); }

 #ifndef SPONTANEOUS
     void reset(bool init=0)
    {
        if (!init) /// on creation those members have been set by QBsdPortPkg()
            this->QBsdPortPkg::reset();

        /// avSrcPkg specific:
        m_broken        = "bpk_no_init";
        m_broken.squeeze();
        m_forbidden     = "bpk_no_init";
        m_broken.squeeze();
        m_ignore        = "bpk_no_init";
        m_ignore.squeeze();
        m_restricted    = "bpk_no_init";
        m_restricted.squeeze();
        m_deprecated    = "bpk_no_init";
        m_deprecated.squeeze();
        m_otherEnv.clear();
        m_otherEnv.append("bpk_no_init");
        m_options.clear();
        m_options.append(_option());
        m_options[0].var = "bpk_no_init";

    }
  #endif

//    bool    reqsMet();  /// check whether all requirements are met
    bool    portDirIsClean();   /// returns true if the portsdir is clean

    QString version()  ;
    QString name()     ;
    QStringList files();
//    virtual QStringList reqs() ;
//    virtual QString prefix();
    QString comment()  ;
    QString descr()    ;
    QString www()      ;
    QString categories();
    QString maint()    ;
    qint64  size()     ;

    /** extra get calls **/
    QList <_option> options();
//    QStringList options2Env();  ///convert options to list of vars for env
    QString forbidden();
    QString ignore();
    QString restricted();
    QString broken();
    QString deprecated();
    QStringList otherEnv()      { return m_otherEnv; }
    QPortList reqsPtrList();

signals:

  #ifndef SPONTANEOUS
    void optionsSignal(QList <_option>);
    void forbiddenSignal(QString);
    void ignoreSignal(QString);
    void restrictedSignal(QString);
    void brokenSignal(QString);
    void deprecatedSignal(QString);
//    void reqsPtrListSignal(const QPortList&);
    void otherEnvSignal(QStringList);
  #endif
public slots:
    /** additional set calls for new members **/
     void setOptions(QList <_option> newValue) { m_options = newValue; }
     void setForbidden(QString newValue)	{ m_forbidden = newValue; }
     void setIgnore(QString newValue)           { m_ignore = newValue; }
     void setRestricted(QString newValue)	{ m_restricted = newValue; }
     void setBroken(QString newValue)           { m_broken = newValue; }
     void setDeprecated(QString newValue)	{ m_deprecated = newValue; }
//     void setReqsPtrList(QPortList newValue)    { m_reqsPtrList = newValue; }
     void setOtherEnv(QStringList newValue)     { m_otherEnv = newValue; }
  #ifndef SPONTANEOUS

    void optionsSlot()       { emit optionsSignal(options()); }
    void forbiddenSlot()     { emit forbiddenSignal(forbidden()); }
    void ignoreSlot()        { emit ignoreSignal(ignore()); }
    void restrictedSlot()    { emit restrictedSignal(restricted()); }
    void brokenSlot()        { emit brokenSignal(broken()); }
    void deprecatedSlot()    { emit deprecatedSignal(deprecated()); }
//    void reqsPtrListSlot()   { emit reqsPtrListSignal(reqsPtrList()); }
    void otherEnvSlot()      { emit otherEnvSignal(otherEnv()); }

    virtual void allSlot() /// emit all contents as signals
    {

        this->QBsdPortPkg::allSlot();

        optionsSlot();
        forbiddenSlot();
        ignoreSlot();
        restrictedSlot();
        brokenSlot();
        deprecatedSlot();
//        reqsPtrListSlot();
    }
  #endif

    /** do somethig with the pkg **/
/*    void    fetch();    /// fetch the tarball (make fetch or manually)
    void    extract();  /// make sanity-check, make checksum, make extract
    void    build();    /// make patch, make configure, make build
    void    install();  /// make install
    void    clean();    /// make clean
    void    quickclean(); /// just rm -rf the work dir */
};


Q_DECLARE_METATYPE(QBsdPort)
Q_DECLARE_METATYPE(QBsdPortPkg)
Q_DECLARE_METATYPE(QPortList)


#endif // __BSDPORT_H__
