/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "bsdportglobal.h"

#include <QString>
#include <QSettings>

//#include <sys/param.h>
#include <sys/utsname.h>

#include <math.h>

/******** QBsdPortGlobal ****************/

/** private **/
QString QBsdPortGlobal::randomString(int length)
{
/* TODO fix this
    QString ret("");
    qsrand(QTime::currentTime().second());

    for (int i = 0; i < length; ++i)
        ret.append(QChar(qrand()%26)+65);

    return ret;*/
    return "random";
}

QString QBsdPortGlobal::getMakeEnv( const QString& path,
                                    const QString& var,
                                    const QStringList& env )
{
    QStringList list;
    return getMakeEnv(path, list+=var, env);
}

QString QBsdPortGlobal::getMakeEnv( const QString& path,
                                    const QStringList& vars,
                                    const QStringList& env )
{
    if (!QFile::exists(path+"/Makefile"))
    {
        emit warning("getMakeEnv() : Directory \"" + path +
                     "\" contains no Makefile, cant get Var." );
        return "";
    }

//    QString cmd = "cd \"" + path + "\" && " + env.join(" ") + " make -V " + var;
//    return QBsdPort::global->runCmd(cmd,1).at(0);
    QString cmd = "/usr/bin/make -C " + path + " ";
    foreach (QString var, vars)
        cmd += "-V " + var + " ";
    cmd += env.join(" ");

    return QBsdPort::global->runCmd(cmd).join(" ");
}

QStringList QBsdPortGlobal::runCmd(const QString& cmd, int linesOfOutput)
{
    /** this call runs a command in foreground and reads #linesOufOutput,
        where 0 means as much output as there is **/

//    qDebug(cmd.toLatin1());
    FILE*   pipe;
    char    buffer[4001] = "";
    QString qbuffer("");
    QStringList lbuffer;
    //TODO remove frame, not neccessary
//    QString realcmd = "/bin/sh -c \'" + cmd + " 2>&1\'";
    QString realcmd = cmd + " 2>&1";

    if( (pipe = popen( realcmd.toLatin1(), "r" )) == NULL )
    {
           emit error("runCmd() : could not open pipe for command:\n" + cmd);
           return QStringList();
    }

    while (!feof(pipe) &&
            ((lbuffer.count() < linesOfOutput) || (!linesOfOutput)) )
    {
        fgets( buffer, 4000, pipe );
        qbuffer = buffer;
        qbuffer.remove(qbuffer.length()-1,1);
        qbuffer.remove('\n');
        lbuffer.append(qbuffer);
    }


    /// remove trailing empty lines
    while ( (!lbuffer.isEmpty()) &&
            ( (lbuffer.last() == "") || (lbuffer.last() == " ") ) )
        lbuffer.removeLast();

    /// work around the double last line
    if (lbuffer.count() >1 )
        if (lbuffer[lbuffer.count()-1] == lbuffer[lbuffer.count()-2])
            lbuffer.removeLast();

    if (lbuffer.isEmpty())
        lbuffer.append("");

    pclose(pipe);

    return lbuffer;
}

void QBsdPortGlobal::readSettings()
{
    QSettings settings("/usr/local/etc/bpk.conf", QSettings::NativeFormat);

    utsname* un = new utsname();
    if (uname(un) != 0) 
        emit critical("Could not get uname-Information from sys/utsname.h");

    QString freeBSDVersion = un->release;
    freeBSDVersion = freeBSDVersion.toLower();

    //BUG: detecting release version doesnt seem to work
    qDebug("UNAME: " + freeBSDVersion.toLatin1());
    int majorVersion = freeBSDVersion.contains('.') ?
                        freeBSDVersion.section('.', 0, 0).toInt() :
                        freeBSDVersion.section('-', 0, 0).toInt();
    int minorVersion = freeBSDVersion.contains('.') ?
                freeBSDVersion.section('-', 0, 0).section('.', 1, 1).toInt() :
                        -1;
    QString release = freeBSDVersion.section('-', 1, 1);
    if ((release == "stable") || (release == "current") )
        minorVersion = -1;
    else if (release != "release")
        emit warning("You seem to be running neither -RELEASE nor -STABLE nor"
                     " -CURRENT...");
     
    
    QString defVal;

    if (!m_portsDir.cd(settings.value("portsDir", "/usr/ports/").toString()))
        emit error("PortsDir does not exist! Change the Path in the Settings"
                   " or refetch Ports");


    if (!m_pkgDbDir.cd(settings.value("pkgDbDir", "/var/db/pkg/").toString()))
        emit error("PkgDb-Directory does not exist! "

              "Change the Path in the Settings.");


    if (!QFile::exists(portsDir().path() +
                      "/INDEX-" + QString::number(majorVersion)))
        emit error("INDEX-File not found. Change the Path in the Settings"
                   " or refetch Portstree");
    else
        defVal = portsDir().path()+"/INDEX-" + QString::number(majorVersion);

    indexFile().setFileName(settings.value("indexFile", defVal).toString());


    QString defaultPkgMirror = "ftp://ftp.freebsd.org/pub/FreeBSD/ports/";

    QString arch = un->machine;

    defVal = getenv("PACKAGESITE") ? getenv("PACKAGESITE")
                                   : defaultPkgMirror + arch + "/packages-" +
                                     QString::number(majorVersion) +
                                     (minorVersion == -1 ? "" :
                                       "." + QString::number(minorVersion)) +
                                     "-" + release + "/Latest/";
    m_pkgSite = settings.value("pkgSite", defVal).toString();

    //TODO connect this to settings as well
    m_pkgDirs.clear();
    m_pkgDirs.append("/usr/ports/Packages/All");


    m_bigPkgSize = settings.value("bigPkgSize", 1024*1024*50 /*50MiB*/).toInt();

    m_updDate = QDate::fromString( settings.value("updDate",
                                                  "2008-01-01").toString(),
                                  Qt::ISODate);

    //TODO connect this to settings as well
    m_defaultBuildEnv.clear();
    m_defaultBuildEnv.append("BATCH=1");
    m_defaultBuildEnv.append("DISABLE_VULNERABILITIES=1");


}

void QBsdPortGlobal::writeSettings()
{
//     QMutexLocker locker(&settingsLock);
    QSettings settings("/usr/local/etc/bpk.conf", QSettings::NativeFormat);

    settings.setValue("portsDir", portsDir().absolutePath());
    settings.setValue("pkgDbDir", pkgDbDir().absolutePath());
    settings.setValue("pkgSite", pkgSite());
    settings.setValue("updDate", updDate().toString("yyyy-MM-dd"));
    settings.setValue("indexFile", indexFile().fileName());
    settings.setValue("bigPkgSize", bigPkgSize());


    settings.sync();
    //TODO connect defaultbuildenv,pkgDirs 

}

/*
void QBsdPortGlobal::defaultValues()
{
    //TODO: check environment vars and os-specific consts for these vars

    if (!m_portsDir.cd("/usr/ports/"))
        m_portsDir.cd("illegal");

//    if (!m_portsDbDir.cd("/var/db/ports/"))
//        m_portsDbDir.cd("illegal");

    if (!m_pkgDbDir.cd("/var/db/pkg/"))
        m_pkgDbDir.cd("illegal");


    if (QFile::exists("/usr/ports/INDEX-7"))
        m_indexFile.setFileName("/usr/ports/INDEX-7");
    else
        m_indexFile.setFileName("illegal");

//    if (QFile::exists("/var/db/bpk/FTPINDEX"))

//    else
//        m_ftpIndexFile.setFileName("illegal");

    m_pkgSite = "ftp://ftp.freebsd.org/pub/FreeBSD/ports/i386/"
                "packages-7-stable/Latest/";
    m_pkgDirs.clear();
    m_pkgDirs.append("/usr/ports/Packages/All");
//    m_updDate = 0;

    m_defaultBuildEnv.clear();
    m_defaultBuildEnv.append("BATCH=1");
    m_defaultBuildEnv.append("DISABLE_VULNERABILITIES=1");



    m_bigPkgSize = 1024*1024*50; // 50MB

    m_portaudit = 0; // no portaudit support, not activated

}*/
#ifndef SPONTANEOUS
void QBsdPortGlobal::createErrPkgs()
{
    /// initialize err pkgs here, THESE ARE NEVER CHANGED
    m_errorPort = new QBsdPort("illegal/illegal");
    QBsdPortAvSrcPkg *avsrc = new QBsdPortAvSrcPkg();
    QBsdPortAvBinPkg *avbin = new QBsdPortAvBinPkg();
    QBsdPortInstPkg *inst   = new QBsdPortInstPkg();


    avsrc->setVersion("illegal");
    avsrc->setName("illegal");
//    avsrc->setPrefix("illegal");
    avsrc->setComment("illegal");
    avsrc->setMaint("illegal");
    avsrc->setCategories("illegal");
    avsrc->setWWW("illegal");
//    avsrc->setReqs(QStringList());
    avsrc->setDescr("illegal");
//    avsrc->setFiles(QStringList());
    avsrc->setBroken("illegal");
    avsrc->setIgnore("illegal");
    avsrc->setRestricted("illegal");
    avsrc->setDeprecated("illegal");
    avsrc->setForbidden("illegal");

    avbin->setVersion("illegal");
    avbin->setName("illegal");
//    avbin->setPrefix("illegal");
    avbin->setComment("illegal");
    avbin->setMaint("illegal");
    avbin->setCategories("illegal");
    avbin->setWWW("illegal");
//    avbin->setReqs(QStringList());
    avbin->setDescr("illegal");
//    avbin->setFiles("illegal");

    inst->setVersion("illegal");
    inst->setName("illegal");
//    inst->setPrefix("illegal");
    inst->setComment("illegal");
    inst->setMaint("illegal");
    inst->setCategories("illegal");
    inst->setWWW("illegal");
//    inst->setReqs(QStringList());
    inst->setDescr("illegal");
//    inst->setFiles(QStringList());

    inst->setDeps(QStringList());
//    inst->setMessage("illegal");
//     inst->setUpdWarning("illegal");
    inst->setFalseFiles(QStringList());
#ifdef PORTAUDIT
/*    inst->setVulna("illegal");
    avbin->setVulna("illegal");
    avsrc->setVulna("illegal");*/
#endif
    errorPort()->setAvSrc(avsrc);
    errorPort()->setAvBin(avbin);
    errorPort()->setInst(inst);
}
#endif //NSPONTANEOUS

QBsdPortGlobal::QBsdPortGlobal()
{
#ifdef SPONTANEOUS
    init();
#endif
}

QBsdPortGlobal::~QBsdPortGlobal()
{
/*    if (avBinThread())
    {
        QMetaObject::invokeMethod(m_avBinThread,
                                "quit",
                                Qt::BlockingQueuedConnection);
        if (!m_avBinThread->wait(100))
            QMetaObject::invokeMethod(m_avBinThread,
                                    "terminate",
                                    Qt::BlockingQueuedConnection);
        m_avBinThread->deleteLater();
    }*/

    writeSettings();

    delete m_errorPort;
    delete m_instPkgDbReader;


    qDeleteAll(m_collection);
}

#ifndef SPONTANEOUS
void    QBsdPortGlobal::getGoing()
{
    start();
    m_instPkgDbReader = new QInstPkgDbReader();
    m_instPkgDbReader->moveToThread(this);
    this->moveToThread(this);
//     QTimer::singleShot(0, this, SLOT(init()));
    QMetaObject::invokeMethod(this, "init", Qt::QueuedConnection);
}
#endif

void QBsdPortGlobal::init()
{
    /// register metadatatypes
    qRegisterMetaType<QBsdPort>();
    qRegisterMetaType<QBsdPort*>("QBsdPort*");
    qRegisterMetaType<QPortList>("QPortList");
    qRegisterMetaType<QBsdPortPkg>();
    qRegisterMetaType<QBsdPortPkg*>("QBsdPortPkg*");

    /// set vars to default
//    defaultValues();
    readSettings();

    /// check for config file
/*    if (QFile::exists("/usr/local/etc/bpk.conf"))
        m_bpkConf.setFileName("/usr/local/etc/bpk.conf");
    else if (QFile::exists("/var/db/bpk/bpk.conf"))
        m_bpkConf.setFileName("/var/db/bpk/bpk.conf");
    else
        m_bpkConf.setFileName("illegal");
*/
    /// if config file exists read it and set vars accordingly
    //TODO: implement

    /// create temporary dirctory
    QString r = randomString(6);
    //TODO check existance of QDir::tempPath+"/bpk-"+randomString(6) and if
    // exists remove dir recursively
    m_sessionDir.mkpath(QDir::tempPath() + "/bpk-" + r);
    m_sessionDir.cd(QDir::tempPath() + "/bpk-" + r);


    createErrPkgs();

    /// create collection
    findAllAvSrc();
  #ifndef SPONTANEOUS

    m_avBinThread = new QFtpIndexThread();

    connect(avBinThread(), SIGNAL(critical(const QString&)),
            this,           SIGNAL(critical(const QString&)),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(error(const QString&)),
            this,           SIGNAL(error(const QString&)),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(warning(const QString&)),
            this,           SIGNAL(warning(const QString&)),
                            Qt::QueuedConnection);

    connect(avBinThread(), SIGNAL(fetchProgress(qint64, qint64)),
            this,           SIGNAL(ftpIndexFetchProgress(qint64, qint64)),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(fail(const QString&)),
            this,           SIGNAL(ftpIndexFail(const QString&)),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(loggingIn()),
            this,           SIGNAL(pkgSiteLoggingIn()),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(foundIndexOnFtp()),
            this,           SIGNAL(pkgSiteFoundIndex()),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(aboutToFetch()),
            this,           SIGNAL(ftpIndexAboutToFetch()),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(localFtpIndexUpToDate()),
            this,           SIGNAL(ftpIndexNowUpToDate()),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(reading()),
            this,           SIGNAL(findAllAvBinRunning()),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(readingDone(int)),
            this,           SLOT(sortCollection()));
    connect(avBinThread(), SIGNAL(readingDone(int)),
            this,           SIGNAL(findAllAvBinFinished(int)),
                            Qt::QueuedConnection);
    connect(avBinThread(), SIGNAL(finished()),
            avBinThread(), SLOT(deleteLater()));

    avBinThread()->getGoing();

//     // TODO replace with a thread
//     findAllInst();

//     setInstPkgDbReader(new QInstPkgDbReader(this));
    m_instPkgDbReader->go();

  #ifdef PORTAUDIT
    QStringList log = runCmd("portaudit THISPORTDOESNTEXIST");
    if ( (log.count() > 0) &&
          (log.at(0).startsWith("portaudit: Database too old.")  ||
           log.at(0).startsWith("portaudit: Database missing") ) )
    //TODO implement option to always update vulnadb on start
    {
        updateVulnaDb();
        log = runCmd("portaudit THISPORTDOESNTEXIST");
    }

  #endif //portaudit


  #endif //n spontaneous


}

#ifndef SPONTANEOUS
void QBsdPortGlobal::run()
{
    exec();
    /// catch all other threads
    if (avBinThread())
    {
        QMetaObject::invokeMethod(m_avBinThread,
                                "quit",
                                Qt::BlockingQueuedConnection);
        if (!m_avBinThread->wait(100))
            QMetaObject::invokeMethod(m_avBinThread,
                                    "terminate",
                                    Qt::BlockingQueuedConnection);
        m_avBinThread->deleteLater();
    }
}
#endif


void QBsdPortGlobal::findAllAvSrc()
{
    emit findAllAvSrcRunning();

  #ifdef SPONTANEOUS
    /** in spontaneous mode we iterate through the portsdir and create ports **/
    QStringList cats = getMakeEnv( portsDir().absolutePath(),
                                                          "SUBDIR" ).split(' ');
    QStringList children;
    QString catname(""), childname(""); /// first and second half of origin
    QBsdPort    *p;

    for (int i = 0; i < cats.count(); ++i)
    {
        catname = cats.at(i);
        children = getMakeEnv( portsDir().absolutePath() +"/"+catname,
                                                          "SUBDIR" ).split(' ');

        for (int ii = 0; ii < children.count(); ++ii)
        {
            qDebug("loop:" + (catname+"/"+children.at(ii)).toLatin1());
            p = new QBsdPort(catname+"/"+children.at(ii));
            p->setAvSrc(new QBsdPortAvSrcPkg(p));
            p->avSrc()->setParent(p);

            addPortToCollection(p, false);
        }
    }
  #else
    /** in normal mode we read the index file **/
    if (!indexFile().exists())
    {
        emit error("findAllAvailableSrc() : INDEX not found");
        emit findAllAvSrcFinished(0);
        return;
    }

    if (!indexFile().open(QFile::ReadOnly))
    {
        emit error("findAllAvailableSrc() : INDEX found, but not readable");
        emit findAllAvSrcFinished(0);
        return;
    }

    char                buffer[15000]="";
    QString             qbuffer("");
    QStringList         lbuffer, reqs;
    qint64              lineLength=0;
    QBsdPort            *port;
    QBsdPortAvSrcPkg    *pkg;


    uint counter = 0;
    while (!indexFile().atEnd())
    {
        lineLength = indexFile().readLine(buffer, sizeof(buffer));

        if (lineLength == -1) /// error reading from INDEX
        {
            emit critical("findAllAvailableSrc() : unknown read error in INDEX");
            indexFile().close();
            return;
        }

        qbuffer = buffer;
        if (!qbuffer.endsWith('\n'))
            emit warning("findAllAvailableSrc() : a line not completely read.");
        else
            qbuffer.remove('\n');

        lbuffer = qbuffer.split('|');

        port    = new QBsdPort(lbuffer.at(1).section('/',3,4));
        pkg     = new QBsdPortAvSrcPkg();

        pkg->setVersion(lbuffer.at(0).section('-',-1,-1));
        pkg->setName(lbuffer.at(0).section('-',0,-2));
        pkg->setComment(lbuffer.at(3));
        pkg->setMaint(lbuffer.at(5));
        pkg->setCategories(lbuffer.at(6));
        pkg->setWWW(lbuffer.at(9));


//        reqs.clear();
//        reqs = lbuffer.at(7).split(' ') + lbuffer.at(8).split(' ');
//        pkg->setReqs(removeDuplicates(reqs));
        addPortToCollection(port, false);

        ++counter;

        port->setAvSrc(pkg);

    }
    indexFile().close();

    //TODO:implement sorting
//    qSort(m_collection);
    qSort(m_collection.begin(), m_collection.end(), QBsdPort::lessThan);

  #endif //not SPONTANEOUS

    emit findAllAvSrcFinished(counter);

}

#ifndef SPONTANEOUS
void    QBsdPortGlobal::findAllInst()
{
    /** we find out which ports are installed and preset / prefetch some info**/
/*
    //TODO: find out if processing "pkg_info -aQo" isnt faster

    emit findAllInstRunning();

    QStringList list = pkgDbDir().entryList(QDir::NoDotAndDotDot|QDir::Dirs);
    QFile       contentsFile;
    QString     origin;
    char        buffer[1024]="";
    QString     qbuffer("");
    QBsdPort    *port;
    QBsdPortInstPkg *pkg;
    qint64 lineLength = 0;
    uint        counter=0;

    for (int i=0; i < list.count(); ++i)
    {
        origin.clear();
        port    = 0;
        pkg     = 0;

        contentsFile.setFileName(pkgDbDir().absolutePath() + "/" + list.at(i) +
                                 "/" + "+CONTENTS");

        if ( (!contentsFile.exists()) || (!contentsFile.open(QFile::ReadOnly)) )
        {
            emit error("findAllInstalled() : corrupt pkg-info for " +
                        list.at(i) + " or not readable .");
        } else
        {
            while ( (origin.isEmpty()) && (!contentsFile.atEnd()) )
            {
                lineLength = contentsFile.readLine(buffer, sizeof(buffer));

                if (lineLength == -1) /// error reading from INDEX
                {
                    emit critical("findAllInstalled() : unknown read error in" +
                                  list.at(i) + "'s pkg_info.");
                    contentsFile.close();
                    return;

                } else
                {
                    qbuffer = buffer;
                    if (!qbuffer.endsWith('\n'))
                        emit warning("findAllInstalled() : "
                                                "a line not completely read.");
                    else
                        qbuffer.remove('\n');

                    if (qbuffer.startsWith("@comment PKG_FORMAT_REVISION:") &&
                        !qbuffer.endsWith("1.1"))
                    {
                        emit critical("findAllInstalled() : unknown"
                                        " PKG_FORMAT_REVISION");
                        contentsFile.close();
                        return;
                    }
                    if (qbuffer.startsWith("@comment ORIGIN:"))
                        origin = qbuffer.remove("@comment ORIGIN:");
                }
            }

            contentsFile.close();

            if (origin.isEmpty()) /// no origin recorded
                origin = "NO_ORIGIN/" + list.at(i).section('-',0,-2);
            if (!(port = portWithOrigin(origin))) /// stale port
            {
                port = new QBsdPort(origin);
                addPortToCollection(port);
            }

            pkg = new QBsdPortInstPkg();
            /// things we "pre-set"
            pkg->setName(list.at(i).section('-',0,-2));
            pkg->setVersion(list.at(i).section('-',-1,-1));
            pkg->setCategories(origin.section('/',0,0));
            /// things we "pre-fetch"
//             pkg->reqs(); /// also gets reqsPtrList and files
//             pkg->deps();
// 
//             pkg->comment();

           port->setInst(pkg);
           ++counter; 
        }
    }

    emit findAllInstFinished(counter);
*/
}

/*
void    QBsdPortGlobal::readUpdFile()
{
    emit readingUpdFile();

    //TODO: implement

    emit readUpdFileFinished();
}*/

#endif // not SPONTANEOUS

QBsdPort* QBsdPortGlobal::portWithOrigin(const  QString& origin,
                                         uint   startPos,
                                         int    *returnPos)
{
//    QMutexLocker locker(&collectionLOCK);

    if (collection().size() == 0)
        return 0;


    QBsdPort *p = 0;

    unsigned int max_i = collection().size()-1;

    // Binaersuche nach origin
    unsigned int links = startPos;
    unsigned int mitte = startPos+1;//collection().size() / 2;
    unsigned int rechts = max_i;

    forever
    {
        mitte = links + ((rechts - links) / 2);

        if ( (mitte > max_i) ||
             (rechts < links) )
             return 0;

        p = collection().at(mitte);
        
        if (p->origin()==origin)
        {
            if (returnPos != 0)
                *returnPos = mitte;
            return p;
        }

        if (p->origin() > origin)
            rechts = mitte - 1;
        else
            links = mitte + 1;
    } 
/*

    if (*i)
        qDebug("iterator shows:" + i->origin() +" for " + origin);

    //TODO replace with a O = log n function
    for (uint i=0; i < collection().size(); ++i)
        if (collection().at(i)->origin()==origin)
            return collection().at(i);
//    emit warning(origin.toLatin1() + " not found");

*/
    return 0;
}

const QBsdPort::QPortList QBsdPortGlobal::portWithPkgName(const QString& name,
                                                          PKGTYPE    pkgtype,
                                                          bool withVersion)
{
    QString n,v;
    QBsdPort *p;
    QPortList ret;
    bool add;

    if (withVersion)
    {
        n = name.section('-',0,-2);
        v = name.section('-',-1,-1);
    } else
        n = name;

//    QMutexLocker locker(&collectionLOCK);

    for (int i = 0; i < collection().size(); ++i)
    {
        p = collection().at(i);
        add = false;
        if (p->hasAvSrc() && (pkgtype &  AVSRCPKG) )
            if ( (p->avSrc()->name()==n) && ( (!withVersion) ||
                  ( (withVersion) && (p->avSrc()->version()==v)) ) )
                add = true;
        if (!add)
            if (p->hasInst() && (pkgtype & INSTPKG) )
                if ( (p->inst()->name()==n) && ( (!withVersion) ||
                        ( (withVersion) && (p->inst()->version()==v)) ) )
                    add = true;;
        if (!add)
            if (p->hasAvBin() && (pkgtype & AVBINPKG) )
                if ( (p->avBin()->name()==n) && ( (!withVersion) ||
                    ( (withVersion) && (p->avBin()->version()==v)) ) )
                    add = true;

        if (add)
            ret += p;
    }

    return ret;
}

QBsdPort::QPortList QBsdPortGlobal::search(const QString& term,
                                           qint64 mask)
{

    QPortList ret;
    QBsdPort *p;
    QString compTerm;
    int percent = 0,
        percentbuf = 0;
    bool add = false;
    bool hasAvSrcPkg = false,
         hasAvBinPkg = false,
         hasInstPkg  = false;

    emit searching();


    int min = 20,
        max = 0;
    for (int j = 0; j < 20; ++j)
    {
        if ((mask & (qint64)pow(2,j)) && (min > j))
            min = j;
        if ((mask & (qint64)pow(2,j)) && (max < j))
            max = j;
    }

    int j = min;

    for (int i = 0; i < collection().size(); ++i)
    {

        percentbuf = i*100/collection().size();
        if ( percentbuf != percent)
        {
            percent = percentbuf;
            emit searchProgress(percent);
        }

        p = collection().at(i);
        hasAvSrcPkg = p->hasAvSrc();
        hasAvBinPkg = p->hasAvBin();
        hasInstPkg  = p->hasInst();

        j = min;
        add = false;        
        while ((j <= max) && (!add))
        {            
            if (mask & (qint64)pow(2,j))
                switch(j)
                {
                    case  0: if (hasAvSrcPkg)
                                add = p->avSrc()->name().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case  1: if (hasAvSrcPkg)
                                add = p->avSrc()->comment().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case  2: if (hasAvSrcPkg)
                                add = (p->avSrc()->files().filter(term,
                                               Qt::CaseInsensitive).size() > 0 );
                             break;
                    case  3: if (hasAvSrcPkg)
                                add = p->avSrc()->descr().contains(term,
                                                           Qt::CaseInsensitive);
                             break;                
    
                    case  8: if (hasAvBinPkg)
                                add = p->avBin()->name().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case  9: if (hasAvBinPkg)
                                add = p->avBin()->comment().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                             
                    case 16: if (hasInstPkg)
                                add = p->inst()->name().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case 17: if (hasInstPkg)
                                add = p->inst()->comment().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    case 18: if (hasInstPkg)
                                add = (p->inst()->files().filter(term,
                                               Qt::CaseInsensitive).size() > 0 );
                             break;
                    case 19: if (hasInstPkg)
                                add = p->inst()->descr().contains(term,
                                                            Qt::CaseInsensitive);
                             break;
                    default:
                            add = false;
                }

            ++j;
        }
        
        if (add)
        {
            emit searchResult(p);
            ret += p;
        }
    }

   emit searchFinished();

   return ret;
        

}


QBsdPort::QPortList QBsdPortGlobal::search(const QString term,
                                           int criteria,
                                           int    pkgtype,
                                           bool exact)
{
    /** there are two ways to use this function: either wait for it to
        return a portlist or watch the signals it emits. **/

    QPortList ret;
    QBsdPort *p;
    QString compTerm;
    int percent = 0,
        percentbuf = 0;
    bool add = false;
    bool hasAvSrcPkg = false,
         hasAvBinPkg = false,
         hasInstPkg  = false;


    emit searching();

//    QMutexLocker locker(&collectionLOCK);

    for (int i = 0; i < collection().size(); ++i)
    {
        add = false;
        percentbuf = i*100/collection().size();
        if ( percentbuf != percent)
        {
            percent = percentbuf;
            emit searchProgress(percent);
        }

        p = collection().at(i);

        hasAvSrcPkg = p->hasAvSrc() && (pkgtype & AVSRCPKG);
        hasAvBinPkg = p->hasAvBin() && (pkgtype & AVBINPKG);
        hasInstPkg  = p->hasInst() && (pkgtype & INSTPKG);
        if (criteria & PKGNAME)
        {
            if (hasAvSrcPkg && p->avSrc()->name().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasAvBinPkg && p->avBin()->name().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasInstPkg && p->inst()->name().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }
        if (!add && (criteria & CATEGORIES))
        {
            if (hasAvSrcPkg && p->avSrc()->categories().contains(
                                         " " + term + " ", Qt::CaseInsensitive))
                add = true;
        }
        if (!add && (criteria & MAINT))
        {
            if (hasAvSrcPkg && p->avSrc()->maint().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasAvBinPkg && p->avBin()->maint().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }
        if (!add && (criteria & COMMENT))
        {
            if (hasAvSrcPkg && p->avSrc()->comment().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasAvBinPkg && p->avBin()->comment().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasInstPkg && p->inst()->comment().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }

        if (!add && (criteria & DESCR))
        {
            if (hasAvSrcPkg && p->avSrc()->descr().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
            else if (hasInstPkg && p->inst()->descr().contains(term,
                                                           Qt::CaseInsensitive))
                add = true;
        }

        if (!add && (criteria & FILES))
        {
            if (hasAvSrcPkg && (p->avSrc()->files().filter(term,
                                    Qt::CaseInsensitive).size() > 0 ) )
                add = true;
            else if (hasInstPkg && (p->inst()->files().filter(term,
                                    Qt::CaseInsensitive).size() > 0 ) )
                add = true;
        }

        if (add)
        {
            emit searchResult(p);
            ret += p;
        }
    }

   emit searchFinished();

   return ret;
}
#ifdef PORTAUDIT
void QBsdPortGlobal::updateVulnaDb()
{
    bool success = false;

    emit updatingVulnaDb();

    QStringList log = runCmd("portaudit -F");

    foreach(QString line, log)
    {
        if (line.startsWith("New database installed.") && !success)
        {
            m_portaudit = 1;
            success = true;
            emit vulnaDbUpdateFinished();
        }
    }

    if (!success)
    {
        m_portaudit = 0;
        emit vulnaDbUpdateFailed(log.join("\n"));
    }
}
#endif //PORTAUDIT

void QBsdPortGlobal::updatePorts(bool force)
{
    /** will run "make update" in /usr/ports/
        if it fails because portsnap is default, but ports werent fetched by
        portsnap in the first place, it will remove the ports dir and fetch it
        anew with portsnap IF FORCE IS SET (otherwise it will simply fail) **/
//TODO implement


}


void QBsdPortGlobal::receiveBinaryPkgs(QList<QBsdPortAvBinPkg*> list)
{
    uint pos = -1;
    foreach(QBsdPortAvBinPkg* pkg, list)
    {

        pos = collection().indexOf(pkg->parent());
        if (pos == -1)
        {
            addPortToCollection(pkg->parent());
//            pos = collection().indexOf(pkg->parent());
        }
        pkg->parent()->setAvBin(pkg);
    }

}

/***** class QFtpIndexThread *****/
void QFtpIndexThread::getGoing()
{
    if (isRunning())
    {
        emit error("QFtpIndexThread is already running!");
        return;
    }

    start();
    this->moveToThread(this);
    QMetaObject::invokeMethod(this, "checkFtpIndex", Qt::QueuedConnection);
//     QTimer::singleShot(0, this, SLOT(checkFtpIndex()));
    

}
void QFtpIndexThread::checkFtpIndex()
{
    QString host = QBsdPort::global->pkgSite().section('/', 1, 1,
                                                     QString::SectionSkipEmpty);
    /// dir is part after host without "/Latest"
    QString dir = QBsdPort::global->pkgSite().section('/', 2, -2,
                                                     QString::SectionSkipEmpty);

    if (pkgSiteFtp())
    {
        emit error ("checkPkgSiteIndex() : "
                                    "FTP already in use. COULDNT check Index!");
        return;
    }

    setState(1);
    m_pkgSiteFtp = new QFtp(this);
    connect(pkgSiteFtp(), SIGNAL(listInfo(QUrlInfo)),
            this, SLOT(parseUrlInfo(QUrlInfo)) );
    connect(pkgSiteFtp(), SIGNAL(commandFinished(int,bool)),
            this, SLOT(cmdFinished(int,bool)) );
    connect(pkgSiteFtp(), SIGNAL(dataTransferProgress(qint64,qint64)),
            this, SIGNAL(fetchProgress(qint64,qint64)));
    pkgSiteFtp()->connectToHost(host);
    pkgSiteFtp()->login("anonymous");
    pkgSiteFtp()->cd(dir);
    pkgSiteFtp()->list();

}

void QFtpIndexThread::parseUrlInfo(const QUrlInfo i)
{
    if (!pkgSiteFtp())
    {
        setState(-1);
        emit error("parseUrlInfo() : pkgSiteFtp dissapeared");
        return;
    }
    if (pkgSiteFtp()->error())
    {
        setState(-1);
        return;
    }

    if ((i.name() == "INDEX") && (state()<2))
    {
        setState(2);
        QFileInfo localFileI(m_ftpIndexFile);
//         qDebug("Date on localFile: " + localFileI.lastModified().toString().toLatin1());
//         qDebug("Date on remoteFile: " + i.lastModified().toString().toLatin1());

        if ( (localFileI.lastModified() < i.lastModified() ) ||
             (localFileI.size() != i.size()) || (!localFileI.exists()) )
        {
//            QFile *localFile = new QFile(m_ftpIndexFile);
            if (ftpIndexFile().open(QIODevice::WriteOnly))
            {
                setState(3);
                pkgSiteFtp()->get("INDEX", &ftpIndexFile());
            } else
            {
                setState(-4);
            }

        } else /// fle is up2date already
            setState(4);
     }
}

void    QFtpIndexThread::cmdFinished(int id,bool error)
{
//     qDebug(QString::number(id).toLatin1() + " finished." +
//             (error ? "error":"noerror"));
    if (id==6) /// list command finished
    {
        if (state() <= 1) /// contents was listed but no INDEX there
            setState(-2);
    } else if ((id==8) || (id==5) && (state()==3) ) /// get command finished
    {
        if (!error)
            setState(4); /// now up2date
        else
            setState(-3); /// error fetching
    } else if ( (error) && (state() >= 0) )
    {
        setState(-1); /// error
//        sleep(1);
    }

}

void QFtpIndexThread::readIndexFile()
{
/*    if (m_finalStateReported)
        return;
    if (state() < 0)
    {
        return;
    } else if (state()==0)
        checkFtpIndex();
    else if (state()==4) /// ftpindex is up2date
    {*/
        setState(5);
        if (!ftpIndexFile().exists())
        {
            emit error("findAllAvBin() : FTPINDEX not found");
            setState(-5);
            return;
        }
        if (ftpIndexFile().isOpen())
            ftpIndexFile().close();

//         qDebug("PATH: " + ftpIndexFile().fileName().toLatin1());

        if (!ftpIndexFile().open(QFile::ReadOnly))
        {
            emit error("findAllAvBin() : FTPINDEX found,"
                          " but not readable");
            setState(-5);
            return;
        }

        char                buffer[15000]="";
        QString             qbuffer(""),
                            origin("");
        QStringList         lbuffer,
                            reqs;
        qint64              lineLength=0;
        QBsdPort            *port;
        QBsdPortAvBinPkg    *pkg;
        int                pos = 0;
        counter             = 0;

//         QList<QBsdPortAvBinPkg*> ret;

//         qDebug("start reading FTPINDEX " + QTime::currentTime().toString().toLatin1());
        qRegisterMetaType<QBsdPort>();
        qRegisterMetaType<QBsdPortPkg>();

        while (!ftpIndexFile().atEnd())
        {
            origin.clear();
            lineLength = ftpIndexFile().readLine(buffer, sizeof(buffer)-1);

            if (lineLength == -1) /// error reading from INDEX
            {
                emit error("findAllAvBin() : unknown read error"
                              " in FTPINDEX");
                ftpIndexFile().close();
                setState(-5);
                return;
            }

            qbuffer = buffer;
            if (!qbuffer.endsWith('\n'))
                emit error("findAllAvBin() : a line not "
                             " completely read.");
            else
                qbuffer.remove('\n');

            lbuffer = qbuffer.split('|');

            origin =  lbuffer.at(1).section('/',3,4);
            if (!origin.contains('/'))
                origin = "NO_ORIGIN/" + lbuffer.at(0).section('-',0,-2);

            pkg     = new QBsdPortAvBinPkg();

            pkg->setVersion(lbuffer.at(0).section('-',-1,-1));
            pkg->setName(lbuffer.at(0).section('-',0,-2));
            pkg->setComment(lbuffer.at(3));
            pkg->setMaint(lbuffer.at(5));
            pkg->setCategories(lbuffer.at(6));
            pkg->setWWW(lbuffer.at(9));

            pkg->moveToThread(QBsdPort::global);

            if (! QMetaObject::invokeMethod(
                        QBsdPort::global,
                        "addPkgToPort",
//                         Qt::BlockingQueuedConnection,
                       Qt::QueuedConnection,
                        Q_ARG(const QString, origin),
                        Q_ARG(QBsdPortPkg*, qobject_cast<QBsdPortPkg*>(pkg)) ) )
                emit error("could not invoke method port");

            ++counter;

            /// assume that pkgs are already sorted alphabetically by origin
            /// and skip those already past
//             port    = QBsdPort::global->portWithOrigin(QString(origin),
//                                                        pos,
//                                                        &pos);
//             /// try again without assuming the former
//             if (!port)
//                 port    = QBsdPort::global->portWithOrigin(QString(origin));
// 
//             if (!port)
//             {
//                 if (!origin.contains('/'))
//                     port = new QBsdPort("NO_ORIGIN/" +
//                                          lbuffer.at(0).section('-',0,-2));
//                 else
//                     port = new QBsdPort(origin);
// 
//                 port->moveToThread(QBsdPort::global);
// 
//                 /// == addPortToCollection(port);
//                 if (! QMetaObject::invokeMethod(QBsdPort::global,
//                                           "addPortToCollection",
//                                           Qt::BlockingQueuedConnection,
// //                                          Qt::QueuedConnection,
//                                           Q_ARG(QBsdPort*, port),
//                                           Q_ARG(bool, true) ) )
//                 emit error("could not invoke method port");
//             }
// 
//             pkg->setParent(port);
//             pkg->moveToThread(QBsdPort::global);
//             ret.append(pkg);
// 
//             ++counter;
// 
//             /// == port->setAvBin(pkg);
//             if (!QMetaObject::invokeMethod(port, "setAvBinQObject",
//                                            Qt::BlockingQueuedConnection,
// //                                          Qt::QueuedConnection,
//                                            Q_ARG(QObject*, (pkg)) ) )
//             emit error("could not invoke method pkg");
        }
//         qRegisterMetaType<QList<QBsdPortAvBinPkg*> >("QBsdPortAvBinPkgList");
//         if (!QMetaObject::invokeMethod(QBsdPort::global, "receiveBinaryPkgs",
//                                         Qt::BlockingQueuedConnection,
// //                                           Qt::QueuedConnection,
//                                         Q_ARG(QList<QBsdPortAvBinPkg*>, ret) ) )
//         emit error("could not invoke method pkg");

        ftpIndexFile().close();
//         qDebug("finished reading FTPINDEX " + QTime::currentTime().toString().toLatin1());
        setState(6);

//    }

}
