/***************************************************************************
 *   Copyright (C) 2005, 2006 by Hannes Hauswedell                         *
 *   hannes.hauswedell@gmail.com                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "bsdportinstall.h"

/// constructor
QBsdPortInstall::QBsdPortInstall(QPkgList list, bool install=true, bool createBackups=true,
				 bool cleanBeforeBuild=true, bool cleanAfterBuild=true,
				 bool waitForDeps=true)
{
	m_install		= install;
	m_createBackups 	= createBackups;
	m_cleanBeforeBuild 	= cleanBeforeBuild;
	m_cleanAfterBuild	= cleanAfterBuild;
	m_waitForDeps		= waitForDeps;

	/// setup connections
	
	connect(this, SIGNAL(taskFinished()), this, SLOT(checkWaitingTasks()));
	connect(this, SIGNAL(readyTasksChanged()), this, SLOT(processReadyTasks()));
	
	
	if (install)
		install(list);
	else
		remove(list);
}

/// private

int QBsdPortInstall::checkReqs(QBsdPortPkg* pkg)
{
    if (!pkg->reqsMet()) /// requirements arent met
        if (pkgFromListInTaskList(pkg->reqs(), unsucTasks)
            return -1;
        else
            return 0;
        
    if (m_waitForReqs) 
    {
        //TODO check ready, waiting and active tasks for reqs of this one
        // if they contain some 
        if (	(pkgFromListInTaskList(pkg->reqs(), readyTasks) ||
                (pkgFromListInTaskList(pkg->reqs(), waitingTasks) ||
                (pkgFromListInTaskList(pkg->reqs(), activeTasks) )
                /// requirements are met but we want to wait for them to be upgraded
            return 0;

        if (pkgFromListInTaskList(pkg->reqs(), unsucTasks)
            return -1;
    }

    return 1; /// all requirements are met (and up-to-date)

}

bool QBsdPortInstall::pkgFromListInTaskList(QString pkgList, QTaskList taskList)
{
    QStringList pkgListL = QStringList::split(' ', pkgList);
	
    for (QPkgTask* t = taskList.first(); t; t = taskList.next())
        for (QString s = pkgListL.first(); s; s = pkgListL.next())
            if (t->pkg->name() == s.section('-', 0,-2)
                return true;
    return false;
}

bool QBsdPortInstall::installTaskActive()
{
    QPkgTask* t;
    
    for (t = activeTasks.first(); t; t=activeTasks.next())
        if ((t->action == 5) || (t->action == 8))
            return 1;

    for (t = readyTasks.first(); t; t=readyTasks.next())
        if ((t->action == 5) || (t->action == 8))
            return 1;
 
    return 0;

}


/// static public 
QPkgList  QBsdPortInstall::createInstList(QPortList list, bool reqs, bool deps,
		int mode, bool reinst, int ident=0)
{
    QPkgList ret;
    bool	go;
    QBsdPortPkg *pkg;
	
    for (QBsdPort *p = list.first(); p != list.end(); p = list.next())
    {
        go = false;
        pkg = 0;
        
        if ((reinst) || (!p->installed()))
            go = true;
        else if ((ident==0) || (ident==1 && reqs) || (ident==2 && deps) )
        {
            if (p->avSource() )
            {
                if (p->avSource()->version() > p->installed()->version())
                    go=true;
            } else if (p->avBinary() )
            {
                if (p->avBinary()->version() > p->installed()->version())
                    go=true;
            } else
                emit noPkg(p);
         }
	

	if (go)		
            if (mode==0) /// binary only
            {
                if (p->avBinary())
                    pkg = p->avBinary();
                else
                    emit noBinPkg(p);	
            } else if (mode==1) /// source only if there is no binary
            {		 
                if (p->avBinary())
        	    pkg = p->avBinary();
                else if (p->avSource()
                    pkg = p->avSource();
                else
                    emit noPkg(p);

            } else if (mode==2) /// source if source is newer than binary
            {
                if (p->avBinary())
                {
                    if (!p->avSource())
                        pkg = p->avBinary();
                    else if ( p->avBinary()->version()
				>= p->avSource()->version() )
	      		pkg = p->avBinary();
		
		} else if (p->avSource())
		    pkg = p->avSource();
		else
		    emit noPkg(p);		 		
	    
	    } else if (mode==3) /// only source
	    {
		if (p->avSource())
		    pkg = p->avSource();
		else
		    emit noPkg(p);	
	    }
	    
            if (pkg)
            {
                ret += pkg + createInstList( pkg->reqsPtrList(), reqs, 0,
                    mode, reinst, 1);
                
                if (p->installed() && deps)
                    ret += createInstList(p->installed()->depsPtrList(), 0,
                            0, mode, reinst, 2);
            }
    }
	
    /// remove duplicates in list
    if (ident==0) /// only need to do this once in the end
        for (int i = 0; i < ret.count(); ++i)
        {
            pkg = ret.at(i);
        
            while (ret.containsRef(pkg)>1)
            {
                findRef(pkg); /// goto first occurrence
                ret.remove(findNextRef(pkg)); /// remove second occurence
            }
        }
        
    return ret;
    
}

QPkgList QBsdPortInstall::createRemoveList(QPortList list, bool reqs, bool deps, int ident=0)
{
    QPkgList ret;
    QBsdPortPkg *pkg;
    
    for (QBsdPort *p = list.first(); p != list.end(); p = list.next())
    {
        pkg = 0;
        
        if ((ident==0) || (ident==1 && reqs) || (ident==2 && deps) )
            if (p->installed())
                pkg = p->installed();
	
        if (pkg)
        {
            ret += pkg;
            if (deps)
                ret += createRemoveList(pkg->depsPtrList(), 0, deps, 2);
            
            if (reqs)
                ret += createRemoveList(pkg->reqsPtrList(), reqs, deps, 1);
        }
    }
	
    /// remove duplicates in list
    if (ident==0) /// only need to do this once in the end
        for (int i = 0; i < ret.count(); ++i)
        {
            pkg = ret.at(i);
            
            while (ret.containsRef(pkg)>1)
            {
                findRef(pkg); /// goto first occurrence
                ret.remove(findNextRef(pkg)); /// remove second occurence
            }
	}
    
    return ret;
}

/// local public
void	QBsdPortInstall::install(QPkgList list)
{

}

void	QBsdPortInstall::remove(QPkgList list)
{
    for ( QBsdPortInstPkg* pkg = (QBsdPortInstPkg*)list.first(); 
			  		pkg !=(QBsdPortInstPkg*)list.last(); 
			  		pkg = (QBsdPortInstPkg*)list.next() )
	waitingTasks += new QPkgTask(pkg,
	 		    (force() ? QPkgTask::remove1 : QPkgTask::remove2));	
		
		
    checkWaitingTasks();

}

void	QBsdPortInstall::checkWaitingTasks()
{
    QBsdPortPkg 	*p1;
    QPkgTask	*t;
    bool		pkgAlreadyInActiveQ = false;
    int		conditionsMet = 0; /// 0 = not met, 1 = met, -1 = cannot be met
    for (int i=0; (i < waitingTasks.count()); ++i)
    {
        t = waitingTasks.at(i);
        p1 = task->pkg;
        pkgAlreadyInActiveQ = false;
        conditionsMet	= 0;

        if (m_install)
        {
            for (int ii=0; (ii < activeTasks.count()) && !pkgAlreadyInActiveQ;
									++ii)
                if (activeTasks.at(ii)->pkg == p1)	
                    pkgAlreadyInActiveQ = true;

            /// if something else is being done with this pkg, we dont
            if (!pkgAlreadyInActiveQ)  
            {
                switch (t->action)
                {
		    case    QPkgTask::backup:
                    case    QPkgTask::fetch_src:
                    case    QPkgTask::fetch_bin: /// have no preconditions
                        if (checkReqs(t->pkg) == 1) 
                        {
                            readyTasks.append(t);
                            waitingTasks.remove(t);
                        } else if (checkReqs(t->pkg) == -1)
 			{
			    /// if a req cannot be met we neednt work onthis pkg
                            t->error = QPkgTask::reqfail;
                            unsucFinTasks.append(t);
                            waitingTasks.remove(t);
                        }
                        break;
                    case    QPkgTask::check: /** check if reqs met before
			extracting (then we neednt check before building...)**/
                        if (checkReqs(t->pkg) == 1) 
                        {
                                readyTasks.append(t);
                                waitingTasks.remove(t);
                        } else if (checkReqs(t->pkg) == -1)
			{
			    /// if a req cannot be met we neednt work onthis pkg
			    t->error = QPkgTask::reqfail;
                            unsucFinTasks.append(t);
                            waitingTasks.remove(t);
                            break;
                        } else 
                  //TODO: test if these break statement acutally exit the switch
                            break;
                    default:	
			/** check if the last(prerequisite) task of the pkg
							finished succesfully**/
                        for (int iii=0; (iii < sucFinTasks.count()); ++iii)
                            if ((sucFinTasks.at(iii)->pkg == p1) &&
				(sucFinTasks.at(iii)->action == t->action-1))	
				conditionsMet = 1;
	
                        /// if not check if the last(prerequisite) task of the pkg failed
                        if (conditionsMet == 0)
                            for (int iii=0;(iii < unsucFinTasks.count()); ++iii)
                                if ((unsucFinTasks.at(iii)->pkg == p1) 
                                    && (unsucFinTasks.at(iii)->action == 
                                                                t->action-1))	
                                    conditionsMet = -1;

                        if ((conditionsMet == 1)
                        {
                            if (t->action == QPkgTask::install_bin) /// for bin inst reqs need to be checked
                            {
                                if (checkReqs(t->pkg) == 1)
                                {
                                    readyTasks.append(t);
                                    waitingTasks.remove(t);
                                } else if ((checkReqs(t->pkg) == -1) 
                                {
                                    /// if a requirement cannot be met we neednt work on this pkg
                                    t->error = QPkgTask::reqfail;
                                    unsucFinTasks.append(t);
                                    waitingTasks.remove(t);
                                }
                            } else
                            {
                                readyTasks.append(t);
                                waitingTasks.remove(t);
                            }
                        } else if (conditionsMet == -1) /// prerequisite task failed -> mark this as undoable
                        {
                            t->error = QPkgTask::parentfail;
                            unsucFinTasks.append(t);
                            waitingTasks.remove(t);
                        }
                        /// if the prerequisite task hasnt been processed yet, do nothing (wait)
                        break;
                }
            }		
    	} else /// this is a pkg-removal
        {
            if (t->action != QpkgTask::remove2 && t->action != QPkgTask::remove1)
                emit error();
                
            //TODO: implement
		
        }
		
    }

    emit readyTasksChanged();
}

void	QBsdPortInstall::processReadyTasks()
{
	QPkgTask *t = 0;
	while ((!readyTasks.isEmpty()) && (activeTasks.count() < numMaxTasks))
	{
		t = readyTasks.first();
		readyTasks.remove(t);
		activeTasks.append(t);
		t->run();
		emit activeTasksChanged();
	}
}

void	QBsdPortInstall::installProcExited(KProcess *proc)
{

	/// find the task this process belonged to
	QPkgTask *t=0;
	for (int i=0; (i < activeTasks.count()); ++i)
		if (activeTasks.at(i)->pkg->proc() == proc)
			t = activeTasks.at(i);

	if (!t)
	{
		qDebug("task that owns this proc could not be found.");
		return;
	}


	if (proc->normalExit())
	{
		if (proc->exitStatus()==0) /// everything went well
			sucFinTasks.append(t);
		else
		{
			t->error = Queue::selffail;
			unsucFinTasks.append(t);
		}
	} else if (proc->signalled())	/// KILLED
	{
		t->error = Queue::killed;
		unsucFinTasks.append(t);
	} else 
	{
		qDebug("something wicked happend. reached line " + QString::number(__LINE__) + " in bsdport.cpp");
		return;
	}

	activeTasks.remove(t);	/// not active any longer ;)
	
	emit taskFinished();
	emit activeTasksChanged();

	if (readyTasks.isEmpty() && activeTasks.isEmpty() && waitingTasks.isEmpty())
		emit allDone();

}
