/***************************************************************************
 *   Copyright (C) 2005, 2006 by Hannes Hauswedell                         *
 *   hannes.hauswedell@gmail.com                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "bsdport.h"

typedef QList <QPkgTask*> QTaskList;

class QPkgTask : public QObject
{
Q_OBJECT
    /** an item that represents a certain action during install
            or uninstall **/
private:

public:
    enum _action /// i for instPkg, s for srcpkg, b for binpkg
    {
        backup,		/// i	0
        remove1,	/// i	1
        fetch_src,	/// s	2
        check,		/// s	3
        clean1,		/// s	4
        build,		/// s	5
        install_src,	/// s	6
        clean2,		/// s	7
        fetch_bin,	/// b	8
        install_bin,	/// b	9
        remove2		/// i	10
    };
    
    enum _error
    {
        selffail,	/// task failed, e.g. build error...
        parentfail,	/// a prerequisite task failed, e.g. fetch before build
        killed,		/// the task was killed
        reqfail		/// another pkg that is required couldnt be installed
    };
            
    QBsdPortPkg	*pkg;
    
    int	action;
    int	error;		

    QPkgTask(QBsdPortInstPkg* Pkg, _action Action) 
    { 
            pkg = (QBsdPortPkg*)Pkg; 
            action = Action;
            error = -1;
    }
    QPkgTask(QBsdPortAvSrcPkg* Pkg, _action Action) 
    { 
            pkg = (QBsdPortPkg*)Pkg; 
            action = Action;
            error = -1;
    }
    QPkgTask(QBsdPortAvBinPkg* Pkg, _action Action) 
    { 
        pkg = (QBsdPortPkg*)Pkg; 
        action = Action;
        error = -1;
    }

    void	run()
    {
        switch (action)
        {
            case backup:    ((QBsdPortInstPkg*)pkg)->backup();
			break;
            case remove1:   ((QBsdPortInstPkg*)pkg)->remove(true);
            		 /// forced removal during upgrade
                        break;
            case fetch_src: ((QBsdPortAvSrcPkg*)pkg)->fetch();
                        break;
            case check:	    ((QBsdPortAvSrcPkg*)pkg)->check();
                        break;
            case clean1:    ((QBsdPortAvBinPkg*)pkg)->clean();
                        break;
            case build:	    ((QBsdPortAvSrcPkg*)pkg)->build();
                        break;
            case install_src:((QBsdPortAvSrcPkg*)pkg)->install();
                        break;
            case clean2      ((QBsdPortAvBinPkg*)pkg)->clean();
                        break;
            case fetch_bin:  ((QBsdPortAvBinPkg*)pkg)->fetch();
                        break;
            case install_bin:((QBsdPortAvBinPkg*)pkg)->install();
                        break;
            case remove2:    ((QBsdPortInstPkg*)pkg)->remove(false); 
            		/// non-forced removal
                        break;
        }
    }

}; 

class QBsdPortInstall : public QObject
{
Q_OBJECT
	/**  starts a session in which ports are installed/upgraded
		or removed... **/
private:

    bool    m_cleanBeforeBuild;
    bool    m_cleanAfterBuild;
    bool    m_createBackups;
    bool    m_install;
    bool    m_waitForReqs;
    
    bool    m_force;
	
	bool	m_nstall_renstall;// renstall man ports f they are up2date
	nt		m_nstallMode; /*0 = only binary, 1 = src if there is no binary, 
						2 = src if source is newer than binary (d), 3 = src */
	nt		m_nstall_reqs; /* 0 = only install new reqs, 1 = upgrade existing (d)
								2 = also renstall reqs that are up2date */
	nt		m_nstall_deps; /* 0 = do nothing with deps (d), 1 = upgrade deps too
							2 = also renstall deps that are up2date */
    
    /** check if currently or about to install something to prevent simultaneous
     writes to /var/db/pkg **/
    bool    installTaskActive();
    /** check if requirements are met (1) and if not (0) if they can be met 
     (if not -1) **/
    int	checkReqs(); 
	
	
    QString session;
public:

    QBsdPortInstall(QPkgList list, bool install, bool createBackups, 
    		bool cleanBeforeBuild, bool cleanAfterBuild, bool waitForDeps);
    /** list should always be created with createInstList or 
        createRemoveList, install says if install or remove**/


    QTaskList	readyTasks,	/// tasks ready to be processed
                waitingTasks,	/// tasks waiting for other tasks to finish
                activeTasks,	/// tasks currently being processed
                sucFinTasks,	/// tasks successfully finished
                unsucFinTasks;	/// tasks finished unsuccessfully
	
	//TODO remove the below ts deprecated
    /* gets all ports (in)directly connected to given one, and gets the appropriate Pkg
    static QPkgList	createInstList(QPortList list,bool reqs, bool deps, 
    					    int mode, bool reinst, int ident);
    /** reqs:	0 = only install new reqs, 1 = upgrade existing (d)
        deps:	0 = do nothing with deps (d), 1 = upgrade deps too
        mode;	0 = only binary, 1 = src if there is no binary, 
                2 = src if source is newer than binary (d), 3 = src 
        reinst:	reinstall if already up-todate
        ident:	0 = originally requested ports, 1 = reqs, 2 = deps *

    static QPkgList	createRemoveList(QPortList list, bool reqs, bool deps);
    /** reqs:	0 = do nothing with reqs (d), 1 = remove reqs
        deps:	0 = do nothing with deps, 1 = remove deps (d) 
            removing deps is essentiel and not doing this will break pkgdb **/	


	QPkgLst portLst2PkgLst(QPortLst lst);
	QBsdPortPkg* port2Pkg(QBsdPort*);
	
	//TODO reqs and deps
	
    void install(QPkgList list);
    void remove(QPkgList list);
    
    
    /// calls for member vars

    const bool cleanBeforeBuild()   { return m_cleanBeforeBuild; }
    const bool cleanAfterBuild()    { return m_cleanAfterBuild; }
    const bool createBackups()      { return m_createBackups; }
    const bool force() 		    { return m_force; }
				
public slots:
	void	installProcExited(KProcess *);

	void 	processReadyTasks();	/// prccesses the list
	void 	checkWaitingTasks();
	void	setForce(bool newValue)	{ m_force = newValue; } 

public signals:

	void	activeTasksChanged();
	void	readyTasksChanged();
	void	taskFinished();

	void	allDone();
	
	void	noPkg(QBsdPort *p);
	void 	noBinPkg(QBsdPort *p);
	void	noSrcPkg(QBsdPort *p);
};