/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "infopagewidget.h"


/** Fade-Out-Object **/

QInfoPageWidgetFader::QInfoPageWidgetFader(QInfoPageWidget *w)
{
    widget = w;
    fadeTimer = new QTimer(this);
    fadeTimer->setInterval(40);
    widget->detailsFrame->hide();
    widget->line->hide();
    minHSize = widget->sizeHint().height();
    hideWidget();
}


void QInfoPageWidgetFader::setWidgetVisible(bool show)
{
    fadeTimer->disconnect(SIGNAL(timeout()));
    if (!fadeTimer->isActive())
    {
        fadeTimer->start();
        emit beginningToFade();
    }
    connect(fadeTimer, SIGNAL(timeout()),
            this,      (show ? SLOT(toSizeHint()) : SLOT(toMinHeight())) );
}

void QInfoPageWidgetFader::toMinHeight()
{
    if (widget->size().height() > minHSize + 12)
        widget->setFixedHeight(widget->size().height() - 12);
    else
    {
        fadeTimer->stop();
        hideWidget();
    }
}


void QInfoPageWidgetFader::toSizeHint()
{
    if (!widget->line->isVisible())
    {
        widget->line->show();
        widget->detailsFrame->show();
    }
    if (widget->size().height() < widget->sizeHint().height() - 10)
        widget->setFixedHeight(widget->size().height() + 10);
    else if (widget->size().height() > widget->minimumSizeHint().height() + 10)
        widget->setFixedHeight(widget->size().height() - 10);
    else
    {
        fadeTimer->stop();
        showWidget();
    }
}

void QInfoPageWidgetFader::hideWidget()
{
    widget->line->hide();
    widget->detailsFrame->hide();
    widget->setFixedHeight(minHSize);
    emit fadeCompleted(false);
}

void QInfoPageWidgetFader::showWidget()
{
    widget->detailsFrame->show();
    widget->line->show();
    widget->setFixedHeight(widget->sizeHint().height());
    emit fadeCompleted(true);
}

/** Base class **/
void QInfoPageWidget::init()
{
    pkg = 0;

    setupUi();

    detailsFader = new QInfoPageWidgetFader(this);

    
    connect(detailsButton, SIGNAL(toggled(bool)),
            detailsFader, SLOT(setWidgetVisible(bool)) );
//     connect(detailsFader, SIGNAL(fadeCompleted(bool)),
//             line,              SLOT(setVisible(bool)));
    /* DIRTY WORKAROUND AGAINST FLICKERING */
//     connect(detailsFrameFader, SIGNAL(fadeCompleted(bool)),
//             this,              SLOT(enableUpdatesOnHeaderFrame()));
//     connect(detailsFrameFader, SIGNAL(beginningToFade()),
//             this,              SLOT(disableUpdatesOnHeaderFrame()));
//     connect(detailsFrameFader->fadeTimer, SIGNAL(timeout()),
//             this,              SLOT(enableUpdatesOnHeaderFrame()));
//     connect(detailsFrameFader->fadeTimer, SIGNAL(timeout()),
//             this,              SLOT(disableUpdatesOnHeaderFrame()));
            
    
    //TODO make a check for style when configsystem is there
    styleOne();

    /// (re)set all variabel fields
    pkgChanged(0);


}
void QInfoPageWidget::setupUi()
{
    if (this->objectName().isEmpty())
        this->setObjectName(QString::fromUtf8("this"));


    topLayout = new QGridLayout(this);
    topLayout->setObjectName(QString::fromUtf8("topLayout"));

    mainFrame = new QFrame(this);
    mainFrame->setObjectName(QString::fromUtf8("mainFrame"));
    mainFrame->setAutoFillBackground(true);
    mainLayout = new QVBoxLayout(mainFrame);
    mainLayout->setObjectName(QString::fromUtf8("mainLayout"));

    headerFrame = new QFrame(mainFrame);
    headerFrame->setObjectName(QString::fromUtf8("headerFrame"));

    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
//     QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(headerFrame->sizePolicy().hasHeightForWidth());

//     this->setSizePolicy(sizePolicy);


    headerFrame->setSizePolicy(sizePolicy);

    headerLayout = new QHBoxLayout(headerFrame);
    headerLayout->setObjectName(QString::fromUtf8("headerLayout"));

    iconLabel = new QLabel(headerFrame);
    iconLabel->setObjectName(QString::fromUtf8("iconLabel"));


    headerLayout->addWidget(iconLabel);

    descrLabel = new QLabel(headerFrame);
    descrLabel->setObjectName(QString::fromUtf8("descrLabel"));

    headerLayout->addWidget(descrLabel);

    spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

    headerLayout->addItem(spacerItem);

    contentsLabel = new QLabel(headerFrame);
    contentsLabel->setObjectName(QString::fromUtf8("contentsLabel"));

    headerLayout->addWidget(contentsLabel);

    detailsButton = new QToolButton(headerFrame);
    detailsButton->setObjectName(QString::fromUtf8("detailsButton"));
    detailsButton->setCheckable(true);
    //TODO replace with BoTTuM icon
//    detailsButton->setIcon(QIcon(QString::fromUtf8(":/images/resources/hd.png")));
    detailsButton->setArrowType(Qt::DownArrow);


    headerLayout->addWidget(detailsButton);


    mainLayout->addWidget(headerFrame);

//     spacerItem2 = new QSpacerItem(1,0, QSizePolicy::Minimum,
//                                       QSizePolicy::Preferred);
//     
//     mainLayout->addItem(spacerItem2);

    line = new QFrame(mainFrame);
    line->setObjectName(QString::fromUtf8("line"));
    line->setFrameShape(QFrame::HLine);
    line->setFrameShadow(QFrame::Sunken);
     line->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
    
     mainLayout->addWidget(line);

    detailsFrame = new QFrame(mainFrame);
    detailsFrame->setObjectName(QString::fromUtf8("detailsFrame"));
    QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Minimum);
    sizePolicy1.setHorizontalStretch(0);
    sizePolicy1.setVerticalStretch(0);
    sizePolicy1.setHeightForWidth(detailsFrame->sizePolicy().hasHeightForWidth());
    detailsFrame->setSizePolicy(sizePolicy1);
    detailsFrame->setFrameShape(QFrame::NoFrame);
    
    /// function implemented by children
    addDetails();
//        detailsLayout = new QVBoxLayout(detailsFrame);

//     if (detailsFrame->layout())
//         detailsFrame->layout()->addWidget(line);


    mainLayout->addWidget(detailsFrame);


    topLayout->addWidget(mainFrame, 0, 0, 1, 1);


    retranslateUi();

    QMetaObject::connectSlotsByName(this);
} 

void QInfoPageWidget::styleOne()
{
    mainFrame->setFrameShape(QFrame::StyledPanel);
    mainFrame->setFrameShadow(QFrame::Raised);
    mainFrame->setBackgroundRole(QPalette::Window);
//        mainLayout->setContentsMargins(0,0,0,0);

//     connect(detailsButton, SIGNAL(toggled(bool)),
//             line, SLOT(setVisible(bool)) );
// 
//     line->setVisible(detailsButton->isDown());


    headerFrame->setFrameShape(QFrame::NoFrame);
    headerFrame->setBackgroundRole(QPalette::Window);
    headerLayout->setContentsMargins(0,0,0,0);

    detailsFrame->setBackgroundRole(QPalette::Window);
//        detailsLayout->setContentsMargins(0,0,0,0);
}

void QInfoPageWidget::styleTwo()
{
    mainFrame->setFrameShape(QFrame::NoFrame);
    mainFrame->setBackgroundRole(QPalette::Base);
    
    //TODO add mainframes default margins again

    disconnect(detailsButton, SIGNAL(toggled(bool)),
            line, SLOT(setVisible(bool)) );
    line->hide();

    headerFrame->setFrameShape(QFrame::StyledPanel);
    headerFrame->setFrameShadow(QFrame::Raised);
    headerFrame->setBackgroundRole(QPalette::Button);

    detailsFrame->setBackgroundRole(QPalette::Base);
}

void QInfoPageWidget::retranslateUi()
{
    //TODO rework this
    this->setWindowTitle(QApplication::translate("infoPageWidget", "", 0, QApplication::UnicodeUTF8));
    iconLabel->setText(QApplication::translate("infoPageWidget", "", 0, QApplication::UnicodeUTF8));
    descrLabel->setText(QApplication::translate("infoPageWidget", "", 0, QApplication::UnicodeUTF8));
    contentsLabel->setText(QApplication::translate("infoPageWidget", "", 0, QApplication::UnicodeUTF8));
    detailsButton->setText(QApplication::translate("infoPageWidget", "", 0, QApplication::UnicodeUTF8));
}

/** nameWidget **/
void QInfoPageNameItem::retranslateUi()
{
    this->QInfoPageWidget::retranslateUi();

    //TODO replace with some cool button
    iconLabel->setPixmap(
                            KPortsIcons::iconLoader.loadIcon(
                            "application-x-compressed-tar").pixmap(22,22) );

    descrLabel->setText(tr("<b>Package</b>"));
    realCatHdr->setText(tr("<b>Categories</b>:"));
    virtCatHdr->setText(tr("<b>Categories(virtual)</b>:"));
}

void QInfoPageNameItem::addDetails()
{
    detailsLayout = new QGridLayout(detailsFrame);
    detailsLayout->setObjectName(QString::fromUtf8("detailsLayout"));
    realCatHdr = new QLabel(detailsFrame);
    realCatHdr->setObjectName(QString::fromUtf8("realCatHdr"));
    realCatHdr->setTextFormat(Qt::RichText);


    detailsLayout->addWidget(realCatHdr, 0, 0, 1, 1);

    realCats = new QLabel(detailsFrame);
    realCats->setObjectName(QString::fromUtf8("realCats"));
    realCats->setText(QString());


    detailsLayout->addWidget(realCats, 0, 1, 1, 1);

    virtCatHdr = new QLabel(detailsFrame);
    virtCatHdr->setObjectName(QString::fromUtf8("virtCatHdr"));


    detailsLayout->addWidget(virtCatHdr, 1, 0, 1, 1);

    virtCats = new QLabel(detailsFrame);
    virtCats->setObjectName(QString::fromUtf8("virtCats"));
    virtCats->setText(QString());

    detailsLayout->addWidget(virtCats, 1, 1, 1, 1);
}

void QInfoPageNameItem::pkgChanged(QBsdPortPkg *newPkg)
{
    if (pkg)
        disconnect(pkg, 0, this, 0);

    contentsLabel->clear();
    realCats->clear();
    virtCats->hide();
    virtCatHdr->hide();

    pkg = newPkg;

    if (!pkg)
        return;

    connect(pkg,    SIGNAL(nameSignal(QString)), 
            this,   SLOT(nameReceived(const QString&)));
    connect(pkg,    SIGNAL(versionSignal(const QString&)), 
            this,   SLOT(versionReceived(const QString&)));
    connect(pkg,    SIGNAL(categoriesSignal(const QString&)), 
            realCats, SLOT(setText(const QString&)));
    //TODO implement differences between real and virtual cats

    if (detailsFrame->isVisible())
    {
        this->updateGeometry();
        detailsFrame->updateGeometry();
        detailsFader->setWidgetVisible(true);
    }
}

void QInfoPageNameItem::nameReceived(const QString& newValue)
{
    name = newValue;

    if (!name.isEmpty() && !version.isEmpty())
        contentsLabel->setText(name+"-"+version);
}
void QInfoPageNameItem::versionReceived(const QString& newValue)
{
    version = newValue;

    if (!name.isEmpty() && !version.isEmpty())
        contentsLabel->setText("<i>"+name+"-"+version+"</i>");
}

/** Web Widget **/

void QInfoPageWebItem::addDetails()
{
    detailsLayout = new QGridLayout(detailsFrame);
    detailsLayout->setObjectName(QString::fromUtf8("detailsLayout"));
    
    maintHdr    = new QLabel(detailsFrame);
    maintHdr->setTextFormat(Qt::RichText);

    detailsLayout->addWidget(maintHdr, 0, 0);

    maint       = new QLinkLabel(detailsFrame);
    maint->setTextFormat(Qt::RichText);
    maint->setText(QString());


    detailsLayout->addWidget(maint, 0, 1);

    wwwHdr    = new QLabel(detailsFrame);
    wwwHdr->setTextFormat(Qt::RichText);

    detailsLayout->addWidget(wwwHdr, 1, 0);

    www       = new QLinkLabel(detailsFrame);
    www->setTextFormat(Qt::RichText);
    www->setText(QString());

    detailsLayout->addWidget(www, 1, 1);

    freshPortsHdr    = new QLabel(detailsFrame);
    freshPortsHdr->setTextFormat(Qt::RichText);

    detailsLayout->addWidget(freshPortsHdr, 2, 0);

    freshPorts      = new QLinkLabel(detailsFrame);
    freshPorts->setTextFormat(Qt::RichText);
    freshPorts->setText(QString());

    detailsLayout->addWidget(freshPorts, 2, 1);

    cvsWebHdr    = new QLabel(detailsFrame);
    cvsWebHdr->setTextFormat(Qt::RichText);

    detailsLayout->addWidget(cvsWebHdr, 3, 0);

    cvsWeb       = new QLinkLabel(detailsFrame);
    cvsWeb->setTextFormat(Qt::RichText);
    cvsWeb->setText(QString());

    detailsLayout->addWidget(cvsWeb, 3, 1);

    contentsLabel->clear();

}

void QInfoPageWebItem::retranslateUi()
{
    this->QInfoPageWidget::retranslateUi();

    iconLabel->setPixmap( KPortsIcons::iconLoader.loadIcon(
                          "applications-internet").pixmap(22,22) );

    descrLabel->setText(tr("<b>Web</b>"));
    maintHdr->setText(tr("<b>Maintainer:</b>"));
    wwwHdr->setText(tr("<b>Homepage:</b>"));
    freshPortsHdr->setText(tr("<b>Freshports:</b>"));
    cvsWebHdr->setText(tr("<b>CVSWeb:</b>"));
}

void QInfoPageWebItem::pkgChanged(QBsdPortPkg *newPkg)
{
    if (pkg)
        disconnect(pkg, 0, this, 0);

    maintHdr->hide();
    maint->hide();
    maint->clear();
    www->clear();
    freshPorts->clear();
    cvsWeb->clear();

    pkg = newPkg;

    if (!pkg)
        return;

    if (QString(pkg->metaObject()->className())!="QBsdPortInstPkg")
    {
        maintHdr->show();
        maint->show();
    }

    freshPorts->setLink("http://www.freshports.org/" 
                                    + pkg->parent()->origin());
    cvsWeb->setLink("http://www.freebsd.org/cgi/cvsweb.cgi/ports/" +
                                pkg->parent()->origin());


    connect(pkg,    SIGNAL(wwwSignal(QString)), 
            www,   SLOT(setLink(const QString&)));
    connect(pkg,    SIGNAL(maintSignal(const QString&)), 
            maint,   SLOT(setLink(const QString&)));

    if (detailsFrame->isVisible())
    {
        this->updateGeometry();
        detailsFrame->updateGeometry();
        detailsFader->setWidgetVisible(true);
    }

}


/** descrWidget **/

void QInfoPageDescrItem::addDetails()
{
    detailsLayout = new QHBoxLayout(detailsFrame);
    detailsLayout->setObjectName(QString::fromUtf8("detailsLayout"));

    description = new QTextEdit(detailsFrame);
    description->setReadOnly(true);

    detailsLayout->addWidget(description);

}

void QInfoPageDescrItem::retranslateUi()
{
    this->QInfoPageWidget::retranslateUi();

    iconLabel->setPixmap(
            KPortsIcons::iconLoader.loadIcon("feed-subscribe").pixmap(22,22) );

    descrLabel->setText(tr("<b>Description</b>"));

}


void QInfoPageDescrItem::pkgChanged(QBsdPortPkg *newPkg)
{
    if (pkg)
        disconnect(pkg, 0, this, 0);

    contentsLabel->clear();
    description->clear();

    pkg = newPkg;

    if (!pkg)
    {
        detailsFader->setWidgetVisible(false);
        return;
    }

    connect(pkg,            SIGNAL(commentSignal(QString)), 
            this,           SLOT(commentReceived(const QString&)));
    connect(pkg,            SIGNAL(descrSignal(const QString&)), 
            this,           SLOT(setDescrText(const QString&)));


    if (QString(pkg->metaObject()->className())=="QBsdPortAvBinPkg")
    {
        /// AvBinPkg dont have long Descrition
        if (detailsButton->isChecked())
            QTimer::singleShot(0, detailsButton, SLOT(toggle()));
        detailsButton->setEnabled(false);
//         detailsFader->setWidgetVisible(false);
    } else
        detailsButton->setEnabled(true);

    if (detailsFrame->isVisible())
    {
        this->updateGeometry();
        detailsFrame->updateGeometry();
        detailsFader->setWidgetVisible(true);
    }

}

void QInfoPageDescrItem::commentReceived(const QString& newValue)
{
    contentsLabel->setText("<i>"+newValue+"</i>");
}

/** Warning Widget **/



void QInfoPageWarningItem::addDetails()
{
    detailsButton->hide();

    bFlags      = new QPushButton(this);
    bVulna      = new QPushButton(this);
    bUpdating   = new QPushButton(this);

    headerLayout->addWidget(bFlags);
    headerLayout->addWidget(bVulna);
    headerLayout->addWidget(bUpdating);

    bFlags->setFlat(true);
    bVulna->setFlat(true);
    bUpdating->setFlat(true);

    bFlags->setCheckable(true);
    bVulna->setCheckable(true);
    bUpdating->setCheckable(true);

    connect(bFlags,     SIGNAL(toggled(bool)),
            this,       SLOT(bFlagsToggled(bool)));
    connect(bVulna,     SIGNAL(toggled(bool)),
            this,       SLOT(bVulnaToggled(bool)));
    connect(bUpdating,  SIGNAL(toggled(bool)),
            this,       SLOT(bUpdatingToggled(bool)));


    detailsLayout = new QHBoxLayout(detailsFrame);
    detailsLayout->setObjectName(QString::fromUtf8("detailsLayout"));
    
    
    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);


    ws          = new QMyStackedWidget(detailsFrame);
    ws->setSizePolicy(sizePolicy);

    flagsPage   = new QWidget(ws);
    vulnaPage   = new QWidget(ws);
    updatingPage= new QWidget(ws);

    flagsPage->setSizePolicy(sizePolicy);
    vulnaPage->setSizePolicy(sizePolicy);
    updatingPage->setSizePolicy(sizePolicy);

    flagsLayout = new QVBoxLayout(flagsPage);
    vulnaLayout = new QVBoxLayout(vulnaPage);
    updatingLayout = new QVBoxLayout(updatingPage);

    ws->addWidget(flagsPage);
    ws->addWidget(vulnaPage);
    ws->addWidget(updatingPage);

    updatingText = new QTextEdit(updatingPage);
    updatingLayout->addWidget(updatingText);
    updatingText->setReadOnly(true);

    detailsLayout->addWidget(ws);

}

void QInfoPageWarningItem::retranslateUi()
{
    this->QInfoPageWidget::retranslateUi();

    iconLabel->setPixmap(
            KPortsIcons::iconLoader.loadIcon("dialog-warning").pixmap(22,22) );

    descrLabel->setText(tr("<b>Warnings</b>"));
    contentsLabel->setText(tr("<i>none</i>"));

    bFlags->setIcon(KPortsIcons::iconLoader.loadIcon("flag-red") );
    bFlags->setText(tr("Flags"));
    bVulna->setIcon(KPortsIcons::iconLoader.loadIcon("security-low") );
    bVulna->setText(tr("Vulnerabilities"));
    bUpdating->setIcon(QPixmap(QString::fromUtf8(
                            ":/custom_artwork/custom_artwork/up_red.png")));
    bUpdating->setText(tr("UPDATING"));
    bUpdating->setToolTip(tr("UPDATING-File warnings describe problems you may"
                            " encounter when upgrading this Port. Read them!"));


}


void QInfoPageWarningItem::pkgChanged(QBsdPortPkg *newPkg)
{
    if (pkg)
        disconnect(pkg, 0, this, 0);

    contentsLabel->show();
    bFlags->hide();
    bVulna->hide();
    bUpdating->hide();

    qDeleteAll(flagsPage->children());
    flagsLayout = new QVBoxLayout(flagsPage);
    qDeleteAll(vulnaPage->children());
    vulnaLayout = new QVBoxLayout(vulnaPage);
    updatingText->clear();

/*        detailsFrame->hide();
    line->hide();*/
    detailsFader->setWidgetVisible(false);
    

    pkg = newPkg;

    if (!pkg)
        return;

    if (QString(pkg->metaObject()->className())=="QBsdPortAvSrcPkg")
    {
        connect((QBsdPortAvSrcPkg*)pkg, SIGNAL(ignoreSignal(QString)), 
                this,           SLOT(ignoreReceived(const QString&)));
        connect((QBsdPortAvSrcPkg*)pkg, SIGNAL(forbiddenSignal(QString)), 
                this,           SLOT(forbiddenReceived(const QString&)));
        connect((QBsdPortAvSrcPkg*)pkg, SIGNAL(brokenSignal(QString)), 
                this,           SLOT(brokenReceived(const QString&)));
        connect((QBsdPortAvSrcPkg*)pkg, SIGNAL(restrictedSignal(QString)), 
                this,           SLOT(restrictedReceived(const QString&)));
        connect((QBsdPortAvSrcPkg*)pkg, SIGNAL(deprecatedSignal(QString)), 
                this,           SLOT(deprecatedReceived(const QString&)));
    }

    if (QBsdPort::global->portaudit() == 1)
    {
    #ifdef PORTAUDIT
        connect(pkg,            SIGNAL(vulnaSignal(QStringList)), 
                this,           SLOT(vulnaReceived(const QStringList&)));
    #endif //PORTAUDIT
    } else
        vulnaPlaceholder();


    if (QString(pkg->metaObject()->className())=="QBsdPortInstPkg")
    {
        connect(pkg,            SIGNAL(updWarningSignal(QString)), 
                this,           SLOT(updWarningReceived(const QString&)));
    }



}

void    QInfoPageWarningItem::vulnaPlaceholder()
{
    contentsLabel->hide();
    bVulna->show();
    bFlags->setChecked(false);
    bVulna->setChecked(false);
    bUpdating->setChecked(false);

    QLabel* label = new QLabel(vulnaPage);
    label->setText(tr("Support for Vulnerabilities Database deactivated or not "
                      "useable. Package might include known Vulnerabilites."));
    label->setWordWrap(true);
    vulnaLayout->addWidget(label);

}


void    QInfoPageWarningItem::vulnaReceived(const QStringList& list)
{
    if (list.isEmpty())
        return;

    contentsLabel->hide();
    bVulna->show();
    bFlags->setChecked(false);
    bVulna->setChecked(false);
    bUpdating->setChecked(false);


    QLabel *label;
    for (int i = 0; i < list.count(); ++i)
    {
        label = new QLabel(vulnaPage);
        label->setOpenExternalLinks(true);
        label->setTextInteractionFlags( Qt::LinksAccessibleByKeyboard 
                                      | Qt::LinksAccessibleByMouse);
        label->setText(list.at(i).section('|',0,0) + " <a href=" +
                       list.at(i).section('|',1,1) + ">more</a>");
        vulnaLayout->addWidget(label);
    }

}

void    QInfoPageWarningItem::ignoreReceived(const QString& str)
{
    if (str.isEmpty())
        return;

    contentsLabel->hide();
    bFlags->show();

    QLabel *label = new QLabel(flagsPage);
    label->setText("<b>IGNORE:</b> " + str);
    flagsLayout->addWidget(label);
}

void    QInfoPageWarningItem::forbiddenReceived(const QString& str)
{
    if (str.isEmpty())
        return;

    contentsLabel->hide();
    bFlags->show();

    QLabel *label = new QLabel(flagsPage);
    label->setText("<b>FORBIDDEN:</b> " + str);
    flagsLayout->addWidget(label);
}

void    QInfoPageWarningItem::brokenReceived(const QString& str)
{
    if (str.isEmpty())
        return;

    contentsLabel->hide();
    bFlags->show();

    QLabel *label = new QLabel(flagsPage);
    label->setText("<b>BROKEN:</b> " + str);
    flagsLayout->addWidget(label);
}

void    QInfoPageWarningItem::restrictedReceived(const QString& str)
{
    if (str.isEmpty())
        return;

    contentsLabel->hide();
    bFlags->show();

    QLabel *label = new QLabel(flagsPage);
    label->setText("<b>RESTRICTED:</b> " + str);
    flagsLayout->addWidget(label);
}

void    QInfoPageWarningItem::deprecatedReceived(const QString& str)
{
    if (str.isEmpty())
        return;

    contentsLabel->hide();
    bFlags->show();

    QLabel *label = new QLabel(flagsPage);
    label->setText("<b>DEPRECATED:</b> " + str);
    flagsLayout->addWidget(label);
}

void    QInfoPageWarningItem::updWarningReceived(const QString& str)
{
    if (str.isEmpty())
        return;

    contentsLabel->hide();
    bUpdating->show();

    updatingText->setText(str);
}

void    QInfoPageWarningItem::bFlagsToggled(bool checked)
{
    if (checked)
    {
        bVulna->setChecked(false);
        bUpdating->setChecked(false);
        ws->setCurrentIndex(0);
//         detailsFrame->show();
//         line->show();

        detailsFader->setWidgetVisible(true);
        
//        ws->setFixedSize(flagsPage->sizeHint());
 //       detailsFrame->update();
    } else if (!(bVulna->isChecked() || bUpdating->isChecked()))
    {
/*        detailsFrame->hide();
        line->hide();*/
        detailsFader->setWidgetVisible(false);
    }
}

void    QInfoPageWarningItem::bVulnaToggled(bool checked)
{
    if (checked)
    {
        bFlags->setChecked(false);
        bUpdating->setChecked(false);
        ws->setCurrentIndex(1);
//         detailsFrame->show();
//         line->show();

        detailsFader->setWidgetVisible(true);
//        ws->setFixedSize(vulnaPage->sizeHint());
    } else if (!(bFlags->isChecked() || bUpdating->isChecked()))
    {
/*        detailsFrame->hide();
        line->hide();*/
        detailsFader->setWidgetVisible(false);
    }
}

void    QInfoPageWarningItem::bUpdatingToggled(bool checked)
{
    if (checked)
    {
        bVulna->setChecked(false);
        bFlags->setChecked(false);
        ws->setCurrentIndex(2);
//         detailsFrame->show();
//         line->show();

        detailsFader->setWidgetVisible(true);
//        ws->setFixedSize(updatingPage->sizeHint());
    } else if (!(bVulna->isChecked() || bFlags->isChecked()))
    {
/*        detailsFrame->hide();
        line->hide();*/
        detailsFader->setWidgetVisible(false);
    }
}

/********* KPortsItem - a subsitution for an about dialog *********************/


void QInfoPageKPortsItem::pkgChanged(QBsdPortPkg *newPkg)
{
    if ((newPkg) && (newPkg->name() == "kports"))
        this->show();
    else
    {
        if (bCredits->isChecked())
            bCredits->toggle();
        this->hide();
    }
        

}


void QInfoPageKPortsItem::retranslateUi()
{
    this->QInfoPageWidget::retranslateUi();

    iconLabel->setPixmap(
            KPortsIcons::iconLoader.loadIcon("favorites").pixmap(22,22) );

    descrLabel->setText(tr("<b>About KPorts</b>"));
    contentsLabel->setText("");
    contentsLabel->hide();

//     bCredits->setIcon(KPortsIcons::iconLoader.loadIcon("flag-red") );
    bCredits->setText(tr("Credits"));
//     bLicense->setIcon(KPortsIcons::iconLoader.loadIcon("security-low") );

    creditsText->setText(tr("<b>Lone Developer:</b><br>"
                         "<a href=\"mailto:kports@soulrebel.in-berlin.de\">"
                         "Hannes Hauswedell</a><br><br>"
/*                         "<b>Thanks to:</b>\n<ul>"
                         "<li>Kamikaze for the new Logo!</li>"
                         "<li>Map7 for his Support during the development of "
                         "the KDE3-Version of KPorts.</li>\n</ul>"));     */
                         "<b>Thanks to:</b><br>"
                         "* <a href=\"mailto:kamikaze@bsdforen.de\">Kamikaze</a>"
                         " for the new Logo!<br>"
                         "* Map7 for his Support during the development of "
                         "the KDE3-Version of KPorts."));

    bLicense->setText(tr("License"));
//     bUpdating->setIcon(QPixmap(QString::fromUtf8(
//                             ":/custom_artwork/custom_artwork/up_red.png")));
    licenseText->setText(tr("<b>All Code:</b><br>"
    "Copyright (c) 2005-2009 <a href=\"mailto:kports@soulrebel.in-berlin.de\">"
    "Hannes Hauswedell</a> <br>"
    "This program is free software: you can redistribute it and/or modify it "
    "under the terms of the <a href=\"http://www.gnu.org/licences/gpl.html\">"
    "GNU General Public License</a> as published by the Free Software Foundation, "
    "either version 3 of the License, or (at your option) any later version.<br>"
    "<br>"
    "<b>KPorts Icon:</b><br>"
    "Copyright (c) 2009 <a href=\"mailto:kamikaze@bsdforen.de\">Dominic "
    "Fandrey</a>, licensed under a free Creative-Commons-License: \""
    "<a href=\"http://creativecommons.org/licenses/by-sa/"
    "3.0/de/\">cc-by-sa-3.0-de</a>\""));
    bOtherWorks->setText(tr("other Works"));
//     bUpdating->setToolTip(tr("UPDATING-File warnings describe problems you may"
//                             " encounter when upgrading this Port. Read them!"));
    otherWorksText->setText(tr("<b>Qt:</b><br>"
    "KPorts uses qt4-corelib, qt4-gui and qt4-network<br>"
    "Copyright (c)  by <a href=\"http://www.qtsoftware.com\">QtSoftware</a> "
    "/ <a href=\"http://www.nokia.com\">Nokia</a><br><br>"
    "<b>Oxygen:</b><br>"
    "If you haven't changed the defaults, KPorts uses the <a href=\"http://www."
    "oxygen-icons.org\">Oxygen-Icon-Theme</a> brought with kdebase4-runtime"));

}

void QInfoPageKPortsItem::addDetails()
{
    detailsButton->hide();

    bCredits      = new QPushButton(this);
    bLicense      = new QPushButton(this);
    bOtherWorks   = new QPushButton(this);

    headerLayout->addWidget(bCredits);
    headerLayout->addWidget(bLicense);
    headerLayout->addWidget(bOtherWorks);

    bCredits->setFlat(true);
    bLicense->setFlat(true);
    bOtherWorks->setFlat(true);

    bCredits->setCheckable(true);
    bLicense->setCheckable(true);
    bOtherWorks->setCheckable(true);

    connect(bCredits,     SIGNAL(toggled(bool)),
            this,       SLOT(bCreditsToggled(bool)));
    connect(bLicense,     SIGNAL(toggled(bool)),
            this,       SLOT(bLicenseToggled(bool)));
    connect(bOtherWorks,  SIGNAL(toggled(bool)),
            this,       SLOT(bOtherWorksToggled(bool)));


    detailsLayout = new QHBoxLayout(detailsFrame);
    detailsLayout->setObjectName(QString::fromUtf8("detailsLayout"));
    
    
    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);


    ws          = new QMyStackedWidget(detailsFrame);
    ws->setSizePolicy(sizePolicy);

    creditsPage   = new QWidget(ws);
    licensePage   = new QWidget(ws);
    otherWorksPage= new QWidget(ws);

    creditsPage->setSizePolicy(sizePolicy);
    licensePage->setSizePolicy(sizePolicy);
    otherWorksPage->setSizePolicy(sizePolicy);

    creditsLayout = new QHBoxLayout(creditsPage);
    licenseLayout = new QVBoxLayout(licensePage);
    otherWorksLayout = new QVBoxLayout(otherWorksPage);

    ws->addWidget(creditsPage);
    ws->addWidget(licensePage);
    ws->addWidget(otherWorksPage);

    //TODO change before release
//     QPixmap logo("/usr/home/hannes/devel/KPORTS/SVN/kports/kports-qt4/kports/data/hicolor/256x256/kports.png");
    QPixmap logo(PREFIX "/share/icons/hicolor/256x256/apps/kports.png");
    creditsLogo = new QLabel(this);
    creditsLogo->setPixmap(logo.scaledToWidth(128));
    creditsLogo->setAlignment(Qt::AlignHCenter);
    creditsLayout->addWidget(creditsLogo);
    
//     creditsLayout->addItem(new QSpacerItem(40, 20,
//                            QSizePolicy::Expanding, QSizePolicy::Minimum));

    creditsText = new QLabel(this);
    creditsText->setTextFormat(Qt::RichText);
    creditsText->setOpenExternalLinks(true);
    creditsText->setWordWrap(true);

    creditsLayout->addWidget(creditsText);
//      creditsLayout->addItem(new QSpacerItem(40, 20,
//                             QSizePolicy::Expanding, QSizePolicy::Minimum));

    licenseText = new QLabel(this);
    licenseText->setTextFormat(Qt::RichText);
    licenseText->setOpenExternalLinks(true);
    licenseText->setWordWrap(true);

    licenseLayout->addWidget(licenseText);

    otherWorksText = new QLabel(this);
    otherWorksText->setTextFormat(Qt::RichText);
    otherWorksText->setOpenExternalLinks(true);
    otherWorksText->setWordWrap(true);

    otherWorksLayout->addWidget(otherWorksText);

//     otherWorksText = new QTextEdit(otherWorksPage);
//     otherWorksLayout->addWidget(otherWorksText);
//     otherWorksText->setReadOnly(true);

    detailsLayout->addWidget(ws);

}

void    QInfoPageKPortsItem::bCreditsToggled(bool checked)
{
    if (checked)
    {
        bLicense->setChecked(false);
        bOtherWorks->setChecked(false);
        ws->setCurrentIndex(0);

        detailsFader->setWidgetVisible(true);

    } else if (!(bLicense->isChecked() || bOtherWorks->isChecked()))
    {

        detailsFader->setWidgetVisible(false);
    }
}

void    QInfoPageKPortsItem::bLicenseToggled(bool checked)
{
    if (checked)
    {
        bCredits->setChecked(false);
        bOtherWorks->setChecked(false);
        ws->setCurrentIndex(1);


        detailsFader->setWidgetVisible(true);

    } else if (!(bCredits->isChecked() || bOtherWorks->isChecked()))
    {

        detailsFader->setWidgetVisible(false);
    }
}

void    QInfoPageKPortsItem::bOtherWorksToggled(bool checked)
{
    if (checked)
    {
        bLicense->setChecked(false);
        bCredits->setChecked(false);
        ws->setCurrentIndex(2);
        detailsFader->setWidgetVisible(true);

    } else if (!(bLicense->isChecked() || bCredits->isChecked()))
    {
        detailsFader->setWidgetVisible(false);
    }
}