/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __INFOPAGEWIDGET_H__
#define __INFOPAGEWIDGET_H__

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QSpacerItem>
#include <QtGui/QToolButton>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include <QtGui/QTextEdit>
#include <QtGui/QStackedWidget>
#include <QtGui/QPushButton>


#include "bsdport.h"
#include "extrawidgets.h"
#include "kportsicons.h"

class QBsdPortPkg;
class QInfoPageWidget;



class QInfoPageWidgetFader : public QObject
{
Q_OBJECT
private:
    QInfoPageWidget *widget;
    QTimer          *fadeTimer;
    int             minHSize;
    int             maxHSize;


public:

    QInfoPageWidgetFader(QInfoPageWidget *w);

public slots:
    void setWidgetVisible(bool show);
    void toMinHeight();
    void toSizeHint();
    void hideWidget();
    void showWidget();
signals:
    void beginningToFade();
    void fadeCompleted(bool isVisible);

};


class QInfoPageWidget: public QWidget
{
Q_OBJECT
public:
    QGridLayout *topLayout;
    QFrame      *mainFrame;
    QVBoxLayout *mainLayout;
    QFrame      *headerFrame;
    QHBoxLayout *headerLayout;
    QLabel      *iconLabel;
    QLabel      *descrLabel;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem2;
    QLabel      *contentsLabel;
    QToolButton *detailsButton;
    QFrame      *line;
    QFrame      *detailsFrame;
//    QVBoxLayout *detailsLayout;


    QInfoPageWidgetFader *detailsFader;

    QBsdPortPkg *pkg;
public:
    QInfoPageWidget(QWidget *parent=0, Qt::WindowFlags f = 0) 
        : QWidget(parent,f)
    {
    }

    virtual void init();
    
    virtual void setupUi();
    
    virtual void styleOne();

    virtual void styleTwo();

    virtual void retranslateUi();

    virtual void    addDetails() = 0;
public slots:
    virtual void    pkgChanged(QBsdPortPkg* newPkg) = 0;

    virtual void    enableUpdatesOnHeaderFrame()
    {
        headerFrame->setUpdatesEnabled(true);
    }
    virtual void    disableUpdatesOnHeaderFrame()
    {
        headerFrame->setUpdatesEnabled(false);
    }
    

};


class QInfoPageNameItem : public QInfoPageWidget
{
Q_OBJECT
private:
    QString name;
    QString version;
public:
    QGridLayout *detailsLayout;
    QLabel *realCatHdr;
    QLabel *realCats;
    QLabel *virtCatHdr;
    QLabel *virtCats;



    QInfoPageNameItem(QWidget* parent=0)
        : QInfoPageWidget(parent)
    {
        init();
    }

    void    retranslateUi();
    void    addDetails();

public slots:
    void    nameReceived(const QString&);
    void    versionReceived(const QString&);
    void    pkgChanged(QBsdPortPkg* newPkg);
};

class QInfoPageWebItem : public QInfoPageWidget
{
Q_OBJECT
public:
    QGridLayout *detailsLayout;
    QLabel *maintHdr;
    QLabel *wwwHdr;
    QLabel *freshPortsHdr;
    QLabel *cvsWebHdr;

    QLinkLabel *maint;
    QLinkLabel *www;
    QLinkLabel *freshPorts;
    QLinkLabel *cvsWeb;


    QInfoPageWebItem(QWidget* parent=0)
        : QInfoPageWidget(parent)
    {
        init();
    }

    void    retranslateUi();
    void    addDetails();


public slots:
    void    pkgChanged(QBsdPortPkg* newPkg);
};

class QInfoPageDescrItem : public QInfoPageWidget
{
Q_OBJECT
public:
    QHBoxLayout *detailsLayout;
    QTextEdit   *description;

    QInfoPageDescrItem(QWidget* parent=0)
        : QInfoPageWidget(parent)
    {
        init();
    }

    void    retranslateUi();
    void    addDetails();

public slots:
    void    commentReceived(const QString&);
    void    pkgChanged(QBsdPortPkg* newPkg);
    inline void    setDescrText(const QString& s)
    {
        description->setPlainText((s == "bpk_no_init") ? "" : s);
    }
};


class QInfoPageWarningItem : public QInfoPageWidget
{
Q_OBJECT
public:
    QHBoxLayout     *detailsLayout;

    QMyStackedWidget  *ws;
    QWidget         *flagsPage;
    QWidget         *vulnaPage;
    QWidget         *updatingPage;
    QPushButton     *bFlags;
    QPushButton     *bVulna;
    QPushButton     *bUpdating;
    QVBoxLayout     *flagsLayout;
    QVBoxLayout     *vulnaLayout;
    QVBoxLayout     *updatingLayout;
    QTextEdit       *updatingText;

    bool        noVulnaDb;
    

    QInfoPageWarningItem(QWidget* parent=0)
        : QInfoPageWidget(parent)
    {
        init();
        noVulnaDb = true;
    }

    void    retranslateUi();
    void    addDetails();

    void    vulnaPlaceholder();

public slots:
    void    pkgChanged(QBsdPortPkg* newPkg);

    void    vulnaReceived(const QStringList&);
    void    ignoreReceived(const QString&);
    void    forbiddenReceived(const QString&);
    void    brokenReceived(const QString&);
    void    restrictedReceived(const QString&);
    void    deprecatedReceived(const QString&);
    void    updWarningReceived(const QString&);

    void    bFlagsToggled(bool);
    void    bVulnaToggled(bool);
    void    bUpdatingToggled(bool);
};

class QInfoPageKPortsItem : public QInfoPageWidget
{
Q_OBJECT
public:
    QHBoxLayout     *detailsLayout;

    QMyStackedWidget  *ws;
    QWidget         *creditsPage;
    QWidget         *licensePage;
    QWidget         *otherWorksPage;
    QPushButton     *bCredits;
    QPushButton     *bLicense;
    QPushButton     *bOtherWorks;
    QHBoxLayout     *creditsLayout;
    QVBoxLayout     *licenseLayout;
    QVBoxLayout     *otherWorksLayout;

    QLabel          *creditsLogo;
    QLabel          *creditsText;

    QLabel          *licenseText;
    QLabel          *otherWorksText;

    

    QInfoPageKPortsItem(QWidget* parent=0, Qt::WindowFlags f = 0)
        : QInfoPageWidget(parent,f)
    {
        init();
    }

    void    retranslateUi();
    void    addDetails();

    void    vulnaPlaceholder();

public slots:
    void    pkgChanged(QBsdPortPkg* newPkg);


    void    bCreditsToggled(bool);
    void    bLicenseToggled(bool);
    void    bOtherWorksToggled(bool);
};

#endif // __INFOPAGEWIDGET_H__
