/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include "instpkgdbreader.h"
#include "bsdportglobal.h"


QInstPkgDbReader::QInstPkgDbReader(QObject* parent) : QObject(parent)
{
    proc    = new QProcess(this);
    counter = 0;
    connect(proc,   SIGNAL(readyReadStandardOutput()),
            this,   SLOT(readStdOut()));
    connect(proc,   SIGNAL(readyReadStandardError()),
            this,   SLOT(readStdErr()));
    connect(proc,   SIGNAL(finished(int, QProcess::ExitStatus)),
            this,   SLOT(procFinished(int, QProcess::ExitStatus)));
}

// just for the sake of it
QInstPkgDbReader::QInstPkgDbReader(const QInstPkgDbReader& n)
{
//     n.proc = proc;
//     n.counter = counter;
//     connect(proc,   SIGNAL(readyReadStandardOutput()),
//             n,   SLOT(readStdOut()));
//     connect(proc,   SIGNAL(readyReadStandardError()),
//             n,   SLOT(readStdErr()));
//     connect(proc,   SIGNAL(finished(int, QProcess::ExitStatus)),
//             n,   SLOT(procFinished(int, QProcess::ExitStatus)));
}

void QInstPkgDbReader::go()
{
    proc->start("/usr/sbin/pkg_info -aQo", QIODevice::ReadOnly);
    /* find /var/db/pkg/ -name "+CONTENTS" | 
        xargs awk '$1=="@comment" && substr($2,1,6)=="ORIGIN" 
            { split(FILENAME,a,"/"); print a[5] ":" substr($2,8) }'
       does the trick aswell and a little faster. Probably not worth losing
       flexibility though */
    emit started();
}

void QInstPkgDbReader::readStdOut()
{
    foreach(QString line, 
            QString(proc->readAllStandardOutput()).split('\n'))
    {
        if (!line.isEmpty())
        {
            origin = line.section(':',-1,-1);
            if (origin.isEmpty())
                origin = "NO_ORIGIN/" + 
                            line.section(':',-2,-2).section('-',0,-2);
            pkg = new QBsdPortInstPkg();
            pkg->setName(line.section(':',-2,-2).section('-',0,-2));
            pkg->setVersion(line.section(':',-2,-2).section('-',-1,-1));
            pkg->setCategories(origin.section('/',0,0));
            QBsdPort::global->addPkgToPort(origin, pkg);
            counter++;
        }
    }
}

void QInstPkgDbReader::readStdErr()
{
    foreach(QString line, 
            QString(proc->readAllStandardError()).split('\n'))
    {
        qWarning(proc->readAllStandardError());
    }
}

void QInstPkgDbReader::procFinished(int exitCode, 
                                    QProcess::ExitStatus exitStatus)
{
    readStdOut();
    if (exitCode==0 && exitStatus==0)
        emit finishedSuccessfully(counter);
    else if (exitStatus)
        emit failed(tr("pkg_info crashed. That's bad."));
    else
        emit failed(tr("pkg_info returned %1 != 0. "
                        "Don't know what that means.").arg(exitCode));
    QBsdPort::global->sortCollection();
}