/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "kportsconfigview.h"
#include "kportsicons.h"
// #include "extrawidgets.h"

#include <QWidget>
#include <QDialog>
#include <QMessageBox>
#include <QColorDialog>



KPortsConfigView::KPortsConfigView(QWidget *parent)
    : QDialog(parent)
{
    setupUi(this);
    indexFileSelecter->caption = tr("Select INDEX-File");



    retranslateUi(this);

    

    this->setWindowIcon(KPortsIcons::iconLoader.loadIcon("configure"));

    m_settings = new QSettings("kports", "kports");


    appearanceButton->setIcon(
                    KPortsIcons::iconLoader.loadIcon("applications-graphics"));
    pathsButton->setIcon(KPortsIcons::iconLoader.loadIcon("edit-rename"));
    miscButton->setIcon(KPortsIcons::iconLoader.loadIcon(
                                                  "applications-accessories"));

    pkgSiteDescriptionIcon->setPixmap(
      KPortsIcons::iconLoader.loadIcon("dialog-information").pixmap(32,32));
    pathsWarningIcon->setPixmap(
        KPortsIcons::iconLoader.loadIcon("dialog-warning").pixmap(32,32));


    updDateEdit->setMinimumDate(QDate(2000,1,1));
    updDateEdit->setMaximumDate(QDate::currentDate());

//    colorButtonFileModified->setFixedWidth(colorButtonFileModified->height());
    

    connect(colorButtonFileModified,    SIGNAL(clicked()),
            this,                       SLOT(colorChoserFileModified()));
    connect(colorButtonPortImprove,     SIGNAL(clicked()),
            this,                       SLOT(colorChoserPortImprove()));
    connect(colorButtonPortRemove,      SIGNAL(clicked()),
            this,                       SLOT(colorChoserPortRemove()));


    connect(appearanceButton,   SIGNAL(clicked()),
            this,               SLOT(showAppearance()));
    connect(pathsButton,          SIGNAL(clicked()),
            this,               SLOT(showPaths()));
    connect(miscButton,          SIGNAL(clicked()),
            this,               SLOT(showMisc()));


    connect(buttonBox,          SIGNAL(accepted()),
            this,               SLOT(accepted()));
    connect(buttonBox,          SIGNAL(clicked(QAbstractButton*)),
            this,               SLOT(buttonInBoxClicked(QAbstractButton*)));

    readKPortsSettings();
    readBpkSettings();
}

void KPortsConfigView::readKPortsSettings()
{
    /** get kports-options **/


    m_fileModified.setNamedColor( settings()->value("colors/filemodified",
                         QColor(255,55,55).name()).toString() );

    m_ports2BeImproved.setNamedColor( settings()->value("colors/portsimprove",
                         QColor(55,255,55).name()).toString() );

    m_ports2BeRemoved.setNamedColor( settings()->value("colors/portsremove",
                         QColor(255,55,55).name()).toString() );

   QPixmap pixmap(12, 12);
/*    QPixmap pixmap(colorButtonFileModified->width()- 5,
                   colorButtonFileModified->height()-5 );*/
    pixmap.fill(m_fileModified);
    colorButtonFileModified->setIcon(QIcon(pixmap));

    pixmap.fill(m_ports2BeImproved);
    colorButtonPortImprove->setIcon(QIcon(pixmap));

    pixmap.fill(m_ports2BeRemoved);
    colorButtonPortRemove->setIcon(QIcon(pixmap));

    switch (settings()->value("icons/toolbariconsize", 32).toInt())
    {
        case 16:    iconSizeToolbar->setCurrentIndex(0); break;
        case 22:    iconSizeToolbar->setCurrentIndex(1); break;
        case 32:    iconSizeToolbar->setCurrentIndex(2); break;
        case 48:    iconSizeToolbar->setCurrentIndex(3); break;
    }

    switch (settings()->value("icons/infoboxiconsize", 48).toInt())
    {
        case 16:    iconSizeInfoBoxes->setCurrentIndex(0); break;
        case 22:    iconSizeInfoBoxes->setCurrentIndex(1); break;
        case 32:    iconSizeInfoBoxes->setCurrentIndex(2); break;
        case 48:    iconSizeInfoBoxes->setCurrentIndex(3); break;
    }

    iconDirSelecter->edit->setText(settings()->value("icons/iconsetpath",
                    "/usr/local/kde4/share/icons/default.kde4/").toString());
    
}

void KPortsConfigView::writeKPortsSettings()
{
    settings()->setValue("colors/filemodified", m_fileModified.name());
    settings()->setValue("colors/portsimprove", m_ports2BeImproved.name());
    settings()->setValue("colors/portsremove",  m_ports2BeRemoved.name());

    int iconsize = 0;
    switch(iconSizeToolbar->currentIndex())
    {
        case 0: iconsize = 16; break;
        case 1: iconsize = 22; break;
        case 2: iconsize = 32; break;
        case 3: iconsize = 48; break;
        default: iconsize = 32; break;
    }
    settings()->setValue("icons/toolbariconsize", iconsize);

    switch(iconSizeInfoBoxes->currentIndex())
    {
        case 0: iconsize = 16; break;
        case 1: iconsize = 22; break;
        case 2: iconsize = 32; break;
        case 3: iconsize = 48; break;
        default: iconsize = 48; break;
    }
    settings()->setValue("icons/infoboxiconsize", iconsize);

    settings()->setValue("icons/iconsetpath",
                         iconDirSelecter->edit->text());
    
}

void KPortsConfigView::readBpkSettings()
{

    /** paths page **/
    pkgSiteEdit->setText(QBsdPort::global->pkgSite());
    portsDirSelecter->edit->setText(QBsdPort::global->portsDir().path());
    pkgDbSelecter->edit->setText(QBsdPort::global->pkgDbDir().path());
    indexFileSelecter->edit->setText(QBsdPort::global->indexFile().fileName());

    /** misc page **/
    updDateEdit->setDate(QBsdPort::global->updDate());
    bigPkgSpinBox->setValue(QBsdPort::global->bigPkgSize() / (1024*1024));
    


}

void KPortsConfigView::writeBpkSettings()
{
    /** paths page **/
    QBsdPort::global->setPkgSite(pkgSiteEdit->text());
    QBsdPort::global->setPortsDir(QDir(portsDirSelecter->edit->text()));
    QBsdPort::global->setPkgDbDir(QDir(pkgDbSelecter->edit->text()));
    QBsdPort::global->setIndexFile(indexFileSelecter->edit->text());

    /** misc page **/
    QBsdPort::global->setUpdDate(updDateEdit->date());
    QBsdPort::global->setBigPkgSize(bigPkgSpinBox->value() * 1024 * 1024);

    if (!QMetaObject::invokeMethod(QBsdPort::global,
                                   "writeSettings",
                                   Qt::QueuedConnection))

        qWarning("Invokation of method failed");


}

void KPortsConfigView::colorChoserFileModified()
{
    QColor newColor = QColorDialog::getColor(m_fileModified, this);
    if (newColor.isValid())
    {
        m_fileModified = newColor;
/*        QSize size(colorButtonFileModified->width() ,
                   colorButtonFileModified->height() / 2);*/
        QSize size(12, 12);
        QPixmap pixmap(size);
        
        pixmap.fill(m_fileModified);
        colorButtonFileModified->setIcon(QIcon(pixmap));
    }    
}

void KPortsConfigView::colorChoserPortImprove()
{
    QColor newColor = QColorDialog::getColor(m_ports2BeImproved, this);
    if (newColor.isValid())
    {
        m_ports2BeImproved = newColor;
/*        QSize size(colorButtonFileModified->width() ,
                   colorButtonFileModified->height() / 2);*/
        QSize size(12, 12);
        QPixmap pixmap(size);

        pixmap.fill(m_ports2BeImproved);
        colorButtonPortImprove->setIcon(QIcon(pixmap));
    }
}

void KPortsConfigView::colorChoserPortRemove()
{
    QColor newColor = QColorDialog::getColor(m_ports2BeRemoved, this);
    if (newColor.isValid())
    {
        m_ports2BeRemoved = newColor;
/*        QSize size(colorButtonFileModified->width() ,
                   colorButtonFileModified->height() / 2);*/
        QSize size(12, 12);
        QPixmap pixmap(size);

        pixmap.fill(m_ports2BeRemoved);
        colorButtonPortRemove->setIcon(QIcon(pixmap));
    }
}


void KPortsConfigView::buttonInBoxClicked(QAbstractButton* button)
{
    if ( (buttonBox->buttonRole(button) & QDialogButtonBox::AcceptRole) ||
         (buttonBox->buttonRole(button) & QDialogButtonBox::ApplyRole) )
    accepted();
}

void KPortsConfigView::accepted()
{
    writeKPortsSettings();
    writeBpkSettings();
}
