/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "kportsview.h"

#include <QMessageBox>
#include <QMainWindow>
#include <QStatusBar>
#include <QHeaderView>
#include <QScrollArea>
#include <QPalette>
#include <QSpacerItem>
#include <QSettings>

#include <math.h>

#include "kportsglobal.h"


KPortsView::KPortsView(QWidget *parent)
    : QWidget(parent)
{

    this->setUpdatesEnabled(false);
    setupUi(this);
    label_2->setText("<span style=\" font-size:14pt; font-weight:600; "
    "color:#e85848;\">"
    APPTITLE
    "</span>");
    
    connect(bInstalled, SIGNAL(toggled(bool)),
            this, SLOT(bInstalledToggled(bool)));


    connect(QBsdPort::global, SIGNAL(portState( QBsdPort*,
                                                const QString&,
                                                const QString&,
                                                const QString&)),
            this,             SLOT(portStateReceived( QBsdPort*,
                                                const QString&,
                                                const QString&,
                                                const QString&)));
    connect(QBsdPort::global, SIGNAL(findAllAvSrcRunning()),
            this,               SLOT(findingAvSrc()));
    connect(QBsdPort::global, SIGNAL(findAllAvSrcFinished(int)),
            this,               SLOT(findingAvSrcDone()));

    connect(QBsdPort::global, SIGNAL(error(QString)),
            this,               SLOT(error(const QString&)));
    connect(QBsdPort::global, SIGNAL(warning(QString)),
            this,               SLOT(warning(const QString&)));
    connect(QBsdPort::global, SIGNAL(critical(QString)),
            this,               SLOT(critical(const QString&)));


    connect(bAvSrc,         SIGNAL(clicked(bool)),
            this,           SLOT(bAvSrcToggled(bool)));
    connect(bAvBin,         SIGNAL(clicked(bool)),
            this,           SLOT(bAvBinToggled(bool)));
    connect(bInst,          SIGNAL(clicked(bool)),
            this,           SLOT(bInstToggled(bool)));


    QWidgetFader *searchGroupFader = new QWidgetFader(searchOptionsGroup);

    connect(searchOptionsButton, SIGNAL(clicked(bool)),
            searchGroupFader, SLOT(setWidgetVisible(bool)));
    connect(searchButton,   SIGNAL(clicked(bool)),
            this,           SLOT(searchButtonClicked()));
    connect(KPortsGlobal::instance(), SIGNAL(searching()),
            this,           SLOT(searchStarted()));
    connect(KPortsGlobal::instance(), SIGNAL(searchResult(QBsdPort*)),
            this,           SLOT(searchResult(QBsdPort*)));
//     connect(QBsdPort::global, SIGNAL(searchProgress(int)),
//             this,           SLOT(raiseBusyWidget()));
    connect(KPortsGlobal::instance(), SIGNAL(searchFinished()),
            this,           SLOT(searchFinished()));
    searchGroupFader->hideWidget();
    cDescrAvBin->hide();
    cFilesAvBin->hide();


    setupModelsAndViews();
    setupInfoFrame();
    setupIcons();




    #ifdef PORTAUDIT
    tDebug->append("Portaudit-Support activated.");
    #endif

    /// uncomment before release
     release();

    QBsdPort::global->getGoing();
    
    setCurrentPort(0);
    this->setUpdatesEnabled(true);
}

void    KPortsView::release()
{
    fetchUpdatesButton->setEnabled(false);
    commitInstallsButton->setEnabled(false);
    commitDeinstallsButton->setEnabled(false);

    fetchUpdatesButton->setToolTip(tr("Not implemented or tested, please be"
    " patient."));
    commitInstallsButton->setToolTip(tr("Not implemented or tested, please be"
    " patient."));
    commitDeinstallsButton->setToolTip(tr("Not implemented or tested, please be"
    " patient."));
    bVulnerable->hide();

    mainTab->setTabEnabled(2, false);
//     mainTab->setTabEnabled(3, false);
    mainTab->removeTab(3);


}


void KPortsView::setupModelsAndViews()
{
    theModel = new QPortsTreeModel(this);
    browseViewFilterModel = new QBrowseViewFilterModel(this);

    browseViewFilterModel->setSourceModel(theModel);

    browseView->setModel(browseViewFilterModel);
//     browseView->setSortingEnabled(true);
    browseView->sortByColumn(0, Qt::AscendingOrder);

    browseView->header()->setResizeMode(0, QHeaderView::Stretch);
    browseView->header()->setResizeMode(1, QHeaderView::ResizeToContents);
    browseView->header()->setResizeMode(2, QHeaderView::ResizeToContents);
    browseView->header()->setStretchLastSection(false);

    connect(browseView,             SIGNAL(clicked(QModelIndex)),
            this,                   SLOT(currentItemChanged(QModelIndex)));
    connect(eBrowseFilter,          SIGNAL(textChanged(const QString&)),
            browseViewFilterModel,  SLOT(delayedSetFilterRegExp(const QString&)));
    connect(bInstalled,             SIGNAL(clicked(bool)),
            browseViewFilterModel,  SLOT(setInstalled(bool)));
    connect(bInstalled,             SIGNAL(clicked(bool)),
            browseViewFilterModel,  SLOT(invalidate()));
    connect(bVulnerable,            SIGNAL(clicked(bool)),
            browseViewFilterModel,  SLOT(setVulnerable(bool)));
    connect(bVulnerable,            SIGNAL(clicked(bool)),
            browseViewFilterModel,  SLOT(invalidate()));
    connect(bUpgradeable,           SIGNAL(clicked(bool)),
            browseViewFilterModel,  SLOT(setUpgradeable(bool)));
    connect(bUpgradeable,           SIGNAL(clicked(bool)),
            browseViewFilterModel,  SLOT(invalidate()));
    connect(bBrowseClear,           SIGNAL(clicked(bool)),
            eBrowseFilter,          SLOT(clear()));

    searchViewFilterModel = new QSearchViewFilterModel(this);
    searchViewFilterModel->setSourceModel(theModel);

    searchView->setModel(searchViewFilterModel);

    searchView->header()->setResizeMode(0, QHeaderView::Stretch);
    searchView->header()->setResizeMode(1, QHeaderView::ResizeToContents);
    searchView->header()->setResizeMode(2, QHeaderView::ResizeToContents);
    searchView->header()->setStretchLastSection(false);
    searchView->sortByColumn(0, Qt::AscendingOrder);

    connect(searchView, SIGNAL(clicked(QModelIndex)),
            this,       SLOT(currentItemChanged(QModelIndex)));
    connect(KPortsGlobal::instance(), SIGNAL(searchResult(QBsdPort*)),
            searchView, SLOT(expandAll()));
}

void KPortsView::setupIcons()
{
    QSettings settings("kports", "kports");
    int s = settings.value("icons/toolbariconsize", 32).toInt();
    QSize ts(s,s);

    /// "maintoolbar"
    fetchUpdatesButton->setIcon(
                        KPortsIcons::iconLoader.loadIcon("view-refresh"));
    fetchUpdatesButton->setIconSize(ts);
    commitInstallsButton->setIcon(QPixmap(QString::fromUtf8(
                    ":/custom_artwork/custom_artwork/apply_green.png")));
    commitInstallsButton->setIconSize(ts);
    commitDeinstallsButton->setIcon(QPixmap(QString::fromUtf8(
                    ":/custom_artwork/custom_artwork/apply_red.png")));
    commitDeinstallsButton->setIconSize(ts);
    aboutButton->setIcon(
        KPortsIcons::iconLoader.loadIcon("dialog-information"));
    aboutButton->setIconSize(ts);
    configureButton->setIcon(KPortsIcons::iconLoader.loadIcon("configure"));
    configureButton->setIconSize(ts);
    exitButton->setIcon(KPortsIcons::iconLoader.loadIcon("application-exit"));
    exitButton->setIconSize(ts);

    bInstalled->setIcon(KPortsIcons::iconLoader.loadIcon("drive-harddisk"));
    bVulnerable->setIcon(KPortsIcons::iconLoader.loadIcon("security-low"));
    bUpgradeable->setIcon(KPortsIcons::iconLoader.loadIcon("arrow-up"));
    bBrowseClear->setIcon(KPortsIcons::iconLoader.loadIcon(
                                                "edit-clear-locationbar-ltr"));
    bSearchClear->setIcon(KPortsIcons::iconLoader.loadIcon(
                                                "edit-clear-locationbar-ltr"));
    bExpandAll->setIcon(KPortsIcons::iconLoader.loadIcon("list-add"));
    bCollapseAll->setIcon(KPortsIcons::iconLoader.loadIcon("list-remove"));
    searchButton->setIcon(KPortsIcons::iconLoader.loadIcon("system-search"));
//     searchButton->setIconSize( QSize(searchButton->height(),
//                                      searchButton->height() ) );

    mainTab->setTabIcon(1, KPortsIcons::iconLoader.loadIcon("edit-find") );
    mainTab->setTabIcon(2, KPortsIcons::iconLoader.loadIcon(
                                                           "view-statistics") );



}



KPortsView::~KPortsView()
{
    QMetaObject::invokeMethod(QBsdPort::global,
                              "quit",
                              Qt::BlockingQueuedConnection);
    if (!QBsdPort::global->wait(100))
        QMetaObject::invokeMethod(QBsdPort::global,
                                  "terminate",
                                  Qt::BlockingQueuedConnection);
    QBsdPort::global->deleteLater();
}


/*!
    \fn KPortsView::bInstalledToggled(bool state)
 */
void KPortsView::bInstalledToggled(bool state)
{
    bVulnerable->setEnabled(state);
    bUpgradeable->setEnabled(state);
    if (!state)
    {
        bVulnerable->setChecked(false);
        bUpgradeable->setChecked(false);
    }
}






/*!
    \fn KPortsView::findingAvSrc()
 */
void KPortsView::findingAvSrc()
{
//     qDebug("findingAvSrc");
//     browseView->update();
//     browseView->setUpdatesEnabled(false);
// /* DEBUG
//     progressDlg->exec();
// 
//     progressDlg->deleteLater();
// */
//     this->setEnabled(true);
//     browseView->setUpdatesEnabled(true);
//     browseView->resizeColumnToContents(0);

}

void KPortsView::findingAvSrcDone()
{

}


/*!
    \fn KPortsView::findingInst()
 */
void KPortsView::findingInst()
{
}


/*!
    \fn KPortsView::findingInstDone()
 */
void KPortsView::findingInstDone()
{

}


/*!
    \fn KPortsView::critical(const QString&)
 */
void KPortsView::critical(const QString& str)
{
    tDebug->append("CRITICAL: " + str);
    qFatal("CRITICAL: " + str.toLatin1());
}


/*!
    \fn KPortsView::error(const QString&)
 */
void KPortsView::error(const QString& str)
{
    tDebug->append("ERROR: " + str);
    qWarning("ERROR: " + str.toLatin1());
}


/*!
    \fn KPortsView::warning(const QString&)
 */
void KPortsView::warning(const QString& str)
{
    tDebug->append("WARNING: " + str);
    qDebug("WARNING: " + str.toLatin1());
}

void KPortsView::currentItemChanged(QModelIndex newItemIndex)
{

    QBsdPort* newCurrent = 0;
    QStandardItem* newItem = theModel->itemFromIndex(
                             browseViewFilterModel->mapToSource(newItemIndex));
    if ( newItem )
        if (newItem->data(50).toString().contains('/'))/// port item
            newCurrent =
                QBsdPort::global->portWithOrigin(newItem->data(50).toString());
//                (QBsdPort*)newItem->data(50).toString().toUInt();
    /// break connections with old port
    if (newCurrent != currentPort() != 0)
        currentPort()->disconnect(this);

    if (newCurrent)
        setCurrentPort(newCurrent);
    else
        setCurrentPort(0);
}

void KPortsView::refreshPortInfo()
{
    bInst->setEnabled(false);
    if (bInst->isChecked())
        bInst->toggle();

    bAvSrc->setEnabled(false);
    if (bAvSrc->isChecked())
        bAvSrc->toggle();

    bAvBin->setEnabled(false);
    if (bAvBin->isChecked())
        bAvBin->toggle();

    if (currentPort())
    {
        tOrigin->setText("<b>" + currentPort()->origin() + "</b>");
        bInst->setText(tr("none"));
        bAvSrc->setText(tr("none"));
        bAvBin->setText(tr("none"));
/*
        connect(currentPort(),  SIGNAL(instState(const QString&)),
                this,           SLOT(currentInstStateReceived(const QString&)));
        connect(currentPort(),  SIGNAL(avSrcState(const QString&)),
                this,           SLOT(currentAvSrcStateReceived(const QString&)));
        connect(currentPort(),  SIGNAL(avBinState(const QString&)),
                this,           SLOT(currentAvBinStateReceived(const QString&)));

        QMetaObject::invokeMethod(currentPort(),"emitInstState",
                                  Qt::QueuedConnection );
        QMetaObject::invokeMethod(currentPort(),"emitAvSrcState",
                                  Qt::QueuedConnection );
        QMetaObject::invokeMethod(currentPort(),"emitAvBinState",
                                  Qt::QueuedConnection );*/
        QMetaObject::invokeMethod(  currentPort(),
                                    "emitPortState",
                                    Qt::QueuedConnection );
        pkgTab->setEnabled(true);

    } else
    {
        emit pkgChanged(0);
        tOrigin->setText(QString());
        bInst->setText(QString());
        bAvBin->setText(QString());
        bAvSrc->setText(QString());
        pkgTab->setEnabled(false);

    }
}

/*!
    \fn KPortsView::currentAvBinStateReceived(const QString&)
 */
void KPortsView::currentAvBinStateReceived(const QString& state)
{
    if (state.isEmpty())
        return;
    bAvBin->setText(state.section('-', -1, -1));
    bAvBin->setEnabled(true);
    //TODO debug why click is always called, isEmpty is workaround
    if ( (bAvSrc->text()==tr("none")) && (bAvSrc->text()==tr("none")) )
    {
            bAvBin->click();
    }
}

void KPortsView::portStateReceived( QBsdPort* p,
                                    const QString& inst,
                                    const QString& avsrc,
                                    const QString& avbin)
{
    if (p != currentPort())
        return;

    bInst->setText(inst.isEmpty() ? tr("none") : inst.section('-',-1,-1));
    bInst->setEnabled(!inst.isEmpty());

    bAvSrc->setText(avsrc.isEmpty() ? tr("none") : avsrc.section('-',-1,-1));
    bAvSrc->setEnabled(!avsrc.isEmpty());

    bAvBin->setText(avbin.isEmpty() ? tr("none") : avbin.section('-',-1,-1));
    bAvBin->setEnabled(!avbin.isEmpty());

    if (!inst.isEmpty())
        bInst->click();
    else if (!avsrc.isEmpty())
        bAvSrc->click();
    else if (!avbin.isEmpty())
        bAvBin->click();

}

void KPortsView::bAvBinToggled(bool state)
{
    if (!state)
    {
        emit pkgChanged(0);
        return;
    }
    if(!currentPort()->hasAvBin())
    {
        //TODO error
        return;
    }

    emit pkgChanged(currentPort()->avBin());

    QMetaObject::invokeMethod(currentPort()->avBin(), "allSlot");
}

/*!
    \fn KPortsView::currentAvSrcStateReceived(const QString&)
 */
void KPortsView::currentAvSrcStateReceived(const QString& state)
{
    if (state.isEmpty())
        return;
    bAvSrc->setText(state.section('-', -1, -1));
    bAvSrc->setEnabled(true);
//    if ( (!bAvBin->isDown()) && (!bInst->isDown()) )
    if (bInst->text()==tr("none"))
    {
            bAvSrc->click();
    }
}

void KPortsView::bAvSrcToggled(bool state)
{
    if (!state)
    {
        emit pkgChanged(0);
        // ...
        return;
    }
    if ( (!currentPort()) || (!currentPort()->hasAvSrc()) )
    {
        //TODO error
        return;
    }

    emit pkgChanged(currentPort()->avSrc());

    QMetaObject::invokeMethod(currentPort()->avSrc(), "allSlot");
}

/*!
    \fn KPortsView::currentInstStateReceived(const QString&)
 */
void KPortsView::currentInstStateReceived(const QString& state)
{
    if (state.isEmpty())
        return;
    bInst->setText(state.section('-', -1, -1));
    bInst->setEnabled(true);
//    if ( (!bAvSrc->isDown()) && (!bAvBin->isDown()) )
    {
        bInst->click();
//        bInst->setChecked(true);
    }
}

void KPortsView::bInstToggled(bool state)
{
    if (!state)
    {
        emit pkgChanged(0);
        return;
    }
    if(!currentPort()->hasInst())
    {
        //TODO error
        return;
    }

    emit pkgChanged(currentPort()->inst());

    QMetaObject::invokeMethod(currentPort()->inst(), "allSlot");
}

void KPortsView::setupInfoFrame()
{
    /// for dep and files tabs
    connect(this,       SIGNAL(pkgChanged(QBsdPortPkg*)),
            this, SLOT(pkgChangedSlot(QBsdPortPkg*)));

    infoPageLayout = new QVBoxLayout(infoPage);

    scrollArea = new QScrollArea(infoPage);
    scrollArea->setFrameShape(QFrame::StyledPanel);
    scrollArea->setFrameShadow(QFrame::Sunken);
    scrollArea->setWidget(new QWidget(scrollArea));
    scrollArea->setWidgetResizable(true);
    scrollArea->widget()->setBackgroundRole(QPalette::Base);

    infoPageLayout->addWidget(scrollArea);

    scrollAreaWidgetLayout = new QVBoxLayout(scrollArea->widget());
    scrollAreaWidgetLayout->setContentsMargins(0,0,0,0);
    scrollAreaWidgetLayout->setSpacing(0);


    /// name widget
    nameInfo = new QInfoPageNameItem(scrollArea->widget());
    scrollAreaWidgetLayout->addWidget(nameInfo);
    connect(this,       SIGNAL(pkgChanged(QBsdPortPkg*)),
            nameInfo, SLOT(pkgChanged(QBsdPortPkg*)));
    /// web widget
    webInfo = new QInfoPageWebItem(scrollArea->widget());
    scrollAreaWidgetLayout->addWidget(webInfo);
    connect(this,       SIGNAL(pkgChanged(QBsdPortPkg*)),
            webInfo, SLOT(pkgChanged(QBsdPortPkg*)));

    /// description widget
    descrInfo = new QInfoPageDescrItem(scrollArea->widget());
    scrollAreaWidgetLayout->addWidget(descrInfo);
    connect(this,       SIGNAL(pkgChanged(QBsdPortPkg*)),
            descrInfo, SLOT(pkgChanged(QBsdPortPkg*)));

    /// warning widget
    warningInfo = new QInfoPageWarningItem(scrollArea->widget());
    scrollAreaWidgetLayout->addWidget(warningInfo);
    connect(this,       SIGNAL(pkgChanged(QBsdPortPkg*)),
            warningInfo, SLOT(pkgChanged(QBsdPortPkg*)));


    /// kports widget
    kportsInfo = new QInfoPageKPortsItem(scrollArea->widget());
    scrollAreaWidgetLayout->addWidget(kportsInfo);
    connect(this,       SIGNAL(pkgChanged(QBsdPortPkg*)),
            kportsInfo, SLOT(pkgChanged(QBsdPortPkg*)));


    /// last
    QSpacerItem* spacerItem =
        new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);
    scrollAreaWidgetLayout->addItem(spacerItem);

    infoPage->update();
}

void    KPortsView::showAbout()
{
    QPortList results = QBsdPort::global->portWithPkgName("kports",
                                                         QBsdPortGlobal::INSTPKG,
                                                          false);
    if (!results.count())
        results = QBsdPort::global->portWithPkgName("kports",
                                                    QBsdPortGlobal::ANYPKG,
                                                    false);
    if (results.count() > 0)
    {
        if (currentPort() != results.at(0))
            setCurrentPort(results.at(0));
        if (!kportsInfo->bCredits->isChecked())
            kportsInfo->bCredits->toggle();
    } else /* if no port for kports is available ->seriously fucked up*/
    {
        /* display the about dialog as modal widget in the center of the app */
        QInfoPageKPortsItem* about = new QInfoPageKPortsItem(this,
                             Qt::Window | Qt::Popup |Qt::WindowStaysOnTopHint );
        about->move((this->width()-about->width())/2,
                    (this->height()/*-about->height()*/)/2);
        about->setWindowModality(Qt::ApplicationModal);
        about->show();
        if (!about->bCredits->isChecked())
            about->bCredits->toggle();
        /* this widget is not deleted before application close, but the case
           is rare anyway and the waste of space not big */
            
    }

}

void    KPortsView::pkgChangedSlot(QBsdPortPkg* pkg)
{
    pkgTab->setUpdatesEnabled(false);
    filesList->clear();
    depsList->clear();
    reqsList->clear();

//     filesPage->setEnabled(false);
    // prevent page-change if uneccessary
    if ((!pkg) || (qobject_cast<QBsdPortAvBinPkg *>(pkg)))
        pkgTab->setTabEnabled(pkgTab->indexOf(filesPage),false);
    lDeps->hide();
    depsList->hide();
    filesLabel->clear();
    filesSizeLabel->clear();
    filesSize->clear();
    filesSize->hide();
    checksumWarningLabel->hide();
    checksumWarningLabel->clear();
    checksumIcon->hide();
    checksumWarningLabel->setToolTip(QString());

    disconnect(this, SLOT(filesReceived(const QStringList&)));
    disconnect(this, SLOT(filesSizeReceived(qint64)));
    disconnect(this, SLOT(falseFilesReceived(const QStringList&)));
    disconnect(this, SLOT(reqsReceived(const QPortList&)));
    disconnect(this, SLOT(depsReceived(const QPortList&)));

    if (!pkg)
    {
        pkgTab->setEnabled(false);
        pkgTab->setUpdatesEnabled(true);        
        return;
    }

    QSettings settings("kports", "kports");
    int s = settings.value("icons/infoboxiconsize", 48).toInt();
    QSize iconsize(s,s);


    connect(pkg,    SIGNAL(reqsPtrListSignal(const QPortList&)),
            this,   SLOT(reqsReceived(const QPortList&)));
    reqsList->addItem(tr("loading..."));

    if (qobject_cast<QBsdPortAvSrcPkg *>(pkg))
    {
        pkgTab->setTabEnabled(pkgTab->indexOf(filesPage),true);
//         filesPage->setEnabled(true);
        filesList->addItem(tr("loading..."));
        filesLabel->setText(tr("Files available:"));
        filesLabel->setToolTip(
            tr("Pathnames relative to PREFIX (\"/usr/local\" by default)"));
        connect(pkg,    SIGNAL(filesSignal(QStringList)),
                this,   SLOT(filesReceived(const QStringList&)));


    } else if (qobject_cast<QBsdPortInstPkg *>(pkg))
    {
        pkgTab->setTabEnabled(pkgTab->indexOf(filesPage),true);
//         filesPage->setEnabled(true);
        filesList->addItem(tr("loading..."));
        filesLabel->setText(tr("Installed Files:"));
        filesLabel->setToolTip(QString());
        connect(pkg,    SIGNAL(filesSignal(QStringList)),
                this,   SLOT(filesReceived(const QStringList&)));

        filesSizeLabel->setText(tr("Total Size of Files:"));
        filesSize->show();
        filesSize->setText("...");
        connect(pkg,    SIGNAL(sizeSignal(qint64)),
                this,   SLOT(filesSizeReceived(qint64)));

        connect(pkg,    SIGNAL(falseFilesSignal(QStringList)),
                this,   SLOT(falseFilesReceived(const QStringList&)));
        checksumWarningLabel->setText(tr("calculating checksums...."));
        checksumIcon->setPixmap(
             KPortsIcons::iconLoader.loadIcon("chronometer").pixmap(iconsize));
        checksumWarningLabel->show();
        checksumIcon->show();

        lDeps->show();
        depsList->show();
        depsList->addItem(tr("loading..."));
        connect(pkg,    SIGNAL(depsPtrListSignal(const QPortList&)),
                this,   SLOT(depsReceived(const QPortList&)));

    } else
    {
//         pkgTab->setTabEnabled(2,false);
        filesLabel->setText(tr("Binary Packages do not provide information "
                               "about available files."));
    }
    pkgTab->setUpdatesEnabled(true);
    pkgTab->setEnabled(true);
}
void    KPortsView::filesReceived(const QStringList& files)
{
    filesList->clear();
    filesList->addItems(files);
}

void    KPortsView::filesSizeReceived(qint64 size)
{
    filesSize->setText(QString::number(size/1024) + " KB");
    if ((size >= QBsdPort::global->bigPkgSize()))
    {
        checksumWarningLabel->setText(
         tr("Checksums were not calculated because that would take too long."));
        checksumWarningLabel->setToolTip(
                           tr("You may change this behaviour in the Options."));
        checksumWarningLabel->show();

        QSettings settings("kports", "kports");
        int s = settings.value("icons/infoboxiconsize", 48).toInt();
        QSize iconsize(s,s);

        checksumIcon->setPixmap(
       KPortsIcons::iconLoader.loadIcon("dialog-information").pixmap(iconsize));
        checksumIcon->show();
    }
}

void    KPortsView::falseFilesReceived(const QStringList& files)
{
    if (checksumWarningLabel->text() ==
          tr("Checksums were not calculated because that would take too long."))
        return;

    QSettings settings("kports", "kports");
    int s = settings.value("icons/infoboxiconsize", 48).toInt();
    QSize iconsize(s,s);
    QColor color;
    color.setNamedColor( settings.value("colors/filemodified",
                         QColor(255,55,55).name()).toString() );


    bool falseType; /// 0 = missing, 1 = modified
    bool hasFalseFiles = false;
    foreach (QString str, files)
    {
        if (str.endsWith("checksum"))
        {
            falseType = 1;
            str.remove(" fails the original MD5 checksum");

        } else if (str.endsWith("exist"))
        {
            falseType = 0;
            str.remove("pkg_info: ");
            str.remove(" doesn't exist");
        }
        foreach(QListWidgetItem* item,
                                filesList->findItems(str, Qt::MatchFixedString))
        {
            hasFalseFiles = true;
            item->setBackground(color);
            if (falseType)
                item->setText(item->text() + tr(" (file modified)"));
            else
                item->setText(item->text() + tr(" (file missing)"));
        }
    }

    if (hasFalseFiles)
    {
        checksumWarningLabel->setText(tr("Some files were "
                        " modified or removed after Install!"));
        checksumWarningLabel->show();
        checksumWarningLabel->setToolTip(tr("You might want to reinstall the "
                        "Package."));
        checksumIcon->setPixmap( KPortsIcons::iconLoader.loadIcon(
                                 "dialog-warning").pixmap(iconsize) );
        checksumIcon->show();
    } else
    {
        checksumWarningLabel->setText(
                                tr("All files match the recorded Checksum!"));
        checksumWarningLabel->show();
        checksumWarningLabel->setToolTip(QString());
        checksumIcon->setPixmap( KPortsIcons::iconLoader.loadIcon(
                                 "face-smile").pixmap(iconsize)    );
        checksumIcon->show();
    }

}


void KPortsView::reqsReceived(const QPortList& reqs)
{
    reqsList->clear();
    foreach(QBsdPort* req, reqs)
    {
        if (req)
            reqsList->addItem(req->origin());
    }
}

void KPortsView::depsReceived(const QPortList& deps)
{
    depsList->clear();
    foreach(QBsdPort* dep, deps)
    {
        if (dep)
            depsList->addItem(dep->origin());
    }
}
void    KPortsView::searchButtonClicked()
{
    searchButton->setEnabled(false);
    if (searchButton->text() == tr("stop"))
    {
        if (m_currentSearch)
        {
            if (!QMetaObject::invokeMethod(m_currentSearch,
                                           "interrupt",
                                           Qt::QueuedConnection))
                emit error("Invokation of method failed");
        } else
            searchFinished();
        return;
    }
    
//     int flags = 0;


    qint64 mask = 0;

    if (cNameAvSrc->isChecked())
        mask += (qint64)pow(2,0);
    if (cCommentAvSrc->isChecked())
        mask += (qint64)pow(2,1);
    if (cFilesAvSrc->isChecked())
        mask += (qint64)pow(2,2);
    if (cDescrAvSrc->isChecked())
        mask += (qint64)pow(2,3);
    // 2^4 - 2^7 reserved

    if (cNameAvBin->isChecked())
        mask += (qint64)pow(2,8);
    if (cCommentAvBin->isChecked())
        mask += (qint64)pow(2,9);
    // 2^10 - 2^15 reserved

    if (cNameInst->isChecked())
        mask += (qint64)pow(2,16);
    if (cCommentInst->isChecked())
        mask += (qint64)pow(2,17);
    if (cFilesInst->isChecked())
        mask += (qint64)pow(2,18);
    if (cDescrInst->isChecked())
        mask += (qint64)pow(2,19);

    if ((!mask) || eSearchFilter->text().isEmpty())
    {
        searchButton->setEnabled(true);
        return;
    }


    m_currentSearch = new QBsdPortSearch(eSearchFilter->text(), mask);
    m_currentSearch->moveToThread(QBsdPort::global);

    if (!connect(m_currentSearch,       SIGNAL(searching()),
            KPortsGlobal::instance(),   SIGNAL(searching()),
            Qt::QueuedConnection))
            qDebug("could not connect search");
    connect(m_currentSearch,            SIGNAL(searchProgress(int)),
            KPortsGlobal::instance(),   SIGNAL(searchProgress(int)),
            Qt::QueuedConnection);
    connect(m_currentSearch,            SIGNAL(searchResult(QBsdPort*)),
            KPortsGlobal::instance(),   SIGNAL(searchResult(QBsdPort*)),
            Qt::QueuedConnection);
    connect(m_currentSearch,            SIGNAL(searchFinished()),
            KPortsGlobal::instance(),   SIGNAL(searchFinished()),
            Qt::QueuedConnection);
    

    
    if (!QMetaObject::invokeMethod(m_currentSearch,
                                   "search",
                                   Qt::QueuedConnection))
        emit error("Invokation of method failed");
    


}


