/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef KPORTSVIEW_H
#define KPORTSVIEW_H

#include <QWidget>
#include <QSortFilterProxyModel>

#include "ui_kportsview.h"
#include "bsdportglobal.h"
#include "extrawidgets.h"
#include "infopagewidget.h"
#include "portstreemodel.h"
#include "bsdportsearch.h"

#define APPTITLE "KPorts 0.8.1"

typedef QList <QBsdPort*> QPortList;
typedef QList <QBsdPortPkg*> QPkgList;

class QScrollArea;

/**
	@author Hannes Hauswedell <kports@soulrebel.in-berlin.de>
*/
class KPortsView : public QWidget, public Ui::KPortsView
{
Q_OBJECT
private:
    QString                 m_noAvBinReason;
    QBsdPort                *m_currentPort;


    QBsdPortSearch          *m_currentSearch;


    int toolbarsize;

    /** Model & View related **/
    void                    setupModelsAndViews();
    QBrowseViewFilterModel  *browseViewFilterModel;
    QSearchViewFilterModel  *searchViewFilterModel;
    QPortsTreeModel         *theModel;


    /**     infoPAGE STUFF  **/
    void        setupInfoFrame();
    QVBoxLayout *infoPageLayout;
    QScrollArea *scrollArea;
    QVBoxLayout *scrollAreaWidgetLayout;
    QInfoPageNameItem*      nameInfo;
    QInfoPageWebItem*       webInfo;
    QInfoPageDescrItem*     descrInfo;
    QInfoPageWarningItem*   warningInfo;
    QInfoPageKPortsItem*    kportsInfo;


    /// a function that is called only when KPorts is packaged
    /// it should hide elements not useable or not implemented yet
    void    release();

    void    setupIcons();
public:
    KPortsView(QWidget *parent = 0);

    ~KPortsView();

    /**     get calls for members   **/
    QString noAvBinReason() const
    {
        return m_noAvBinReason;
    }
    QBsdPort* currentPort() const { return m_currentPort; }

    /**     set calls for member    **/
    void    setCurrentPort(QBsdPort* newValue)
    {
        m_currentPort = newValue;
        refreshPortInfo();
    }



    void refreshPortInfo();
public slots:
    void bInstalledToggled(bool state);
    void portStateReceived( QBsdPort*,
                            const QString&,
                            const QString&,
                            const QString&);

    void findingAvSrc();
    void findingAvSrcDone();
    void findingInst();
    void findingInstDone();
    void critical(const QString&);
    void error(const QString&);
    void warning(const QString&);


    void showAbout();

    void searchButtonClicked();



    void searchResult(QBsdPort* p)
    {
        tDebug->append("Search Result: " + p->origin());
//        searchView->expandAll();
    }

    void searchStarted()
    {
        searchButton->setEnabled(true);
        searchButton->setText(tr("stop"));
        searchButton->setIcon(KPortsIcons::iconLoader.loadIcon("process-stop"));
    }


    void searchFinished()
    {
        if (m_currentSearch)
            m_currentSearch->deleteLater();
        searchButton->setEnabled(true);
        searchButton->setText(tr("search"));
        searchButton->setIcon(KPortsIcons::iconLoader.loadIcon("system-search"));
    }

    void currentItemChanged(QModelIndex);

    /** slots for current port **/
    void currentAvBinStateReceived(const QString&);
    void currentAvSrcStateReceived(const QString&);
    void currentInstStateReceived(const QString&);
    void bAvSrcToggled(bool);
    void bAvBinToggled(bool);
    void bInstToggled(bool);
    void pkgChangedSlot(QBsdPortPkg*);
    void filesReceived(const QStringList&);
    void falseFilesReceived(const QStringList&);
    void filesSizeReceived(qint64);
    void reqsReceived(const QPortList&);
    void depsReceived(const QPortList&);
signals:
    void pkgChanged(QBsdPortPkg*);
   
};

#endif
