/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "portstreemodel.h"
#include "kportsglobal.h"
#include "bsdportglobal.h"

QPortsTreeModel::QPortsTreeModel(QObject *parent = 0)
 : QStandardItemModel(0,3, parent)
{

    QStringList myHeader;
    myHeader.append(tr("Origin"));
    myHeader.append(tr("Installed Version"));
    myHeader.append(tr("Available Version"));

    setHorizontalHeaderLabels(myHeader);
    m_currentSearchId = 0;

// 1 central

    connect(QBsdPort::global, SIGNAL(portState(QBsdPort*, const QString&,
                                              const QString&, const QString&)),
            this,             SLOT(portStateReceived(QBsdPort*, const QString&,
                                            const QString&, const QString&)));
    connect(QBsdPort::global, SIGNAL(newPortAdded(QBsdPort*)),
            this,               SLOT(addPortToModel(QBsdPort*)));
    connect(KPortsGlobal::instance(), SIGNAL(searching()),
            this,               SLOT(incSearchId()));
    connect(KPortsGlobal::instance(), SIGNAL(searchResult(QBsdPort*)),
            this,               SLOT(searchResultReceived(QBsdPort*)));
//     connect(KPortsGlobal::instance(), SIGNAL(searching()),
//             this,               SLOT(incSearchId()));


}


QPortsTreeModel::~QPortsTreeModel()
{
    clear();
    qDeleteAll(list);
}


void QPortsTreeModel::addPortToModel(QBsdPort* p)
{
    if (!p)
        return;

    QPortMetaItem*    item = 0;
    QString cat     = p->origin().section('/',0,0);
    QString name    = p->origin().section('/',1,1);
    QStandardItem *cat_item = 0;


    //TODO debug the below to speed up category fidning
/*    if ( ((QPortsTreeCategoryItem*)invisibleRootItem()->child(
                invisibleRootItem()->rowCount() ))->name() == cat)
        cat_item = (QPortsTreeCategoryItem*)invisibleRootItem()->child(
                invisibleRootItem()->rowCount());
    else
*/        for (int i = 0; i < invisibleRootItem()->rowCount(); ++i)
        {
            if (invisibleRootItem()->child(i)->data(0) == cat )
                cat_item = invisibleRootItem()->child(i);
        }

    if (!cat_item)
    {
        cat_item = new QStandardItem(
                               KPortsIcons::iconLoader.loadIcon("folder"), cat);
        cat_item->setData(-1, 51);
        invisibleRootItem()->appendRow(cat_item);
    }

    item = new QPortMetaItem(p);
    list.append(item);
    cat_item->appendRow(item->cols());

    // 1 central
//     qRegisterMetaType<QBsdPort>();
//     if (!QMetaObject::invokeMethod(QBsdPort::global,
//                             "emitPortState",
//                             Qt::QueuedConnection,
//                             Q_ARG(QBsdPort*, p)))
//         qDebug("couldnt invokemethod for portstate");
}

void QPortsTreeModel::searchResultReceived(QBsdPort* p)
{
    foreach(QPortMetaItem *item, list)
    {
        if (item->port() == p)
        {
            item->col0()->setData(currentSearchId(), 51);
            return;
        }
    }
}
    // 1 central
void QPortsTreeModel::portStateReceived(QBsdPort* p, const QString& inst,
                                const QString& avsrc, const QString& avbin)
{
    QPortMetaItem* row = 0;
/*    foreach(QPortMetaItem *crow, list)
        if (crow->port() == p)
            row = crow;
*/    for (int i = list.count()-1; (i > 0) && (row==0); --i)
        if (list.at(i)->port() == p)
            row = list.at(i);

    if (!row)
        return;

    if (inst.isEmpty())
        row->col1()->setData(QString(), 0);
    else
        row->col1()->setData(inst.section('-',-1,-1),0);

    if (avsrc.isEmpty())
        row->col2()->setData(QString(), 33);
    else
    {
        row->col2()->setData(avsrc.section('-',-1,-1),33);
        if (row->col2()->data(33).toString()
                        > row->col2()->data(34).toString())
            row->col2()->setData(avsrc.section('-',-1,-1),0);
    }

    if (avbin.isEmpty())
        row->col2()->setData(QString(), 34);
    else
    {
        row->col2()->setData(avbin.section('-',-1,-1),34);
        if (row->col2()->data(34).toString()
                    > row->col2()->data(33).toString())
            row->col2()->setData(avbin.section('-',-1,-1),0);
    }
}

// // 2 new single
// void QPortMetaItem::pkgStateReceived(const QString& inst,
//                                 const QString& avsrc, const QString& avbin)
// {
//     if (inst.isEmpty())
//         col1()->setData(QString(), 0);
//     else
//         col1()->setData(inst.section('-',-1,-1),0);
// 
//     if (avsrc.isEmpty())
//         col2()->setData(QString(), 33);
//     else
//     {
//         col2()->setData(avsrc.section('-',-1,-1),33);
//         if (col2()->data(33).toString()
//                         > col2()->data(34).toString())
//             col2()->setData(avsrc.section('-',-1,-1),0);
//     }
// 
//     if (avbin.isEmpty())
//         col2()->setData(QString(), 34);
//     else
//     {
//         col2()->setData(avbin.section('-',-1,-1),34);
//         if (col2()->data(34).toString()
//                     > col2()->data(33).toString())
//             col2()->setData(avbin.section('-',-1,-1),0);
//     }
// }

///-----------------------------------------------------------------------------


bool QBrowseViewFilterModel::filterAcceptsRow(int sourceRow,
                                    const QModelIndex &sourceParent) const
{
    QModelIndex index0 = sourceModel()->index(sourceRow, 0, sourceParent);
    QModelIndex index1 = sourceModel()->index(sourceRow, 1, sourceParent);
    QModelIndex index2 = sourceModel()->index(sourceRow, 2, sourceParent);

    /// do bottom to top filtering
    if (sourceModel()->hasChildren(index0))
    {
        for (int i=0; i < sourceModel()->rowCount(index0); ++i)
            if (filterAcceptsRow(i, index0))
                return true;
        return false;
    }
    bool show = true;

    if (!sourceModel()->data(index0).toString().contains(
                               filterRegExp().pattern(), Qt::CaseInsensitive))
        show = false;

    else if (installed() && sourceModel()->data(index1).toString().isEmpty())
        show =  false;

    //TODO implement vulnerabilitycheck
    else if (upgradeable() && (sourceModel()->data(index2).toString()
                          <= sourceModel()->data(index1).toString()) )
        show = false;

    return show;
}



QSearchViewFilterModel::QSearchViewFilterModel(QObject *parent)
        : QSortFilterProxyModel(parent)
{
    connect(KPortsGlobal::instance(),   SIGNAL(searching()),
            this,               SLOT(invalidate()));
    connect(KPortsGlobal::instance(),   SIGNAL(searchResult(QBsdPort*)),
            this,               SLOT(invalidate()));
    connect(KPortsGlobal::instance(),   SIGNAL(searchFinished()),
            this,               SLOT(invalidate()));
    
    setSortCaseSensitivity(Qt::CaseInsensitive);
    setDynamicSortFilter(true);
}


bool QSearchViewFilterModel::filterAcceptsRow(int sourceRow,
                                    const QModelIndex &sourceParent) const
{
    QModelIndex index0 = sourceModel()->index(sourceRow, 0, sourceParent);

    /// do bottom to top filtering
    if (sourceModel()->hasChildren(index0))
    {
        for (int i=0; i < sourceModel()->rowCount(index0); ++i)
            if (filterAcceptsRow(i, index0))
                return true;
        return false;
    }

    return (sourceModel()->data(index0, 51).toInt() ==
            ((QPortsTreeModel*)sourceModel())->currentSearchId());
}

