/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TASKWIDGETS_H
#define TASKWIDGETS_H

#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QProgressBar>
#include <QtGui/QWidget>

#include <QMessageBox>
#include <QTimer>
#include <QSettings>

#include "bsdportglobal.h"
#include "kportsglobal.h"
#include "kportsicons.h"
#include "extrawidgets.h"


class QCurrentTaskItem : public QFrame
{
Q_OBJECT
protected:
    QHBoxLayout *mainLayout;
    QHBoxLayout *subLayout;
    QLabel      *head;
    QSlowLabel      *sub;
    QProgressBar   *progressBar;

    bool        showBar;

    QTimer      *fadeOutTimer;

    /// needed to reflect Visibility in case parent is invisible
    bool        m_wouldBeVisible;
public:
    QCurrentTaskItem(QWidget *parent=0)
        : QFrame(parent)
    {
        this->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
    }

    ~QCurrentTaskItem()
    {
        delete fadeOutTimer;
    }
    void    setupUi()
    {
        this->setBackgroundRole(QPalette::Base);
        this->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
        mainLayout = new QHBoxLayout(this);
//         int left = 11;
//         int right = 11;
//         mainLayout->getContentsMargins(&left, 0, &right, 0);
//         mainLayout->setContentsMargins(left,0,right,0);
        head = new QLabel(this);
        mainLayout->addWidget(head);

        subLayout = new QHBoxLayout;
        subLayout->setContentsMargins(0,0,0,0);
        sub = new QSlowLabel(this);
        subLayout->addWidget(sub);
        if (showBar)
        {
            progressBar = new QProgressBar(this);
            progressBar->setValue(0);
            progressBar->setRange(0,100);
//             progressBar->setFixedHeight(progressBar->height()*0.8);
            progressBar->setFixedHeight(head->minimumSizeHint().height());
            subLayout->addWidget(progressBar);
        }
        mainLayout->addLayout(subLayout);
//        mainLayout->addStretch();

        QMetaObject::connectSlotsByName(this);
    }

    void    retranslateUi()
    {
    }

    inline bool wouldBeVisible() const { return m_wouldBeVisible; }

public slots:
    inline void show()
    {
        m_wouldBeVisible = true;
        this->QWidget::show();
        emit visibilityChange();
    }

    inline void hide()
    {
        m_wouldBeVisible = false;
        this->QWidget::hide();
        emit visibilityChange();
    }
    void    fadeOut()
    {
/*        qDebug("fadeOut() called");
        fadeOutTimer = new QTimer(this);
        connect(fadeOutTimer,   SIGNAL(timeout()),
                this,           SLOT(timeout()));
        fadeOutTimer->start(100); */
//DEBUG
        hide();
    }

    void timeout()
    {
        qDebug("fadeOut()-Timer timeout");
        if (windowOpacity() > 0.05)
            setWindowOpacity(windowOpacity()-0.05);
        else
        {
            hide();
            fadeOutTimer->stop();
        }
    }
signals:
    void    visibilityChange();
};

class QSearchTaskItem : public QCurrentTaskItem
{
Q_OBJECT
private:
    /// delay progressbar update to prevent freeze of parent
    QTimer  *unblockUpdateTimer;
    bool    blockUpdate;
public:
    QSearchTaskItem(QWidget* parent=0)
        : QCurrentTaskItem(parent)
    {
        showBar = true;
        setupUi();
        sub->hide();
        retranslateUi();

        blockUpdate = false;
        unblockUpdateTimer = new QTimer(this);
        unblockUpdateTimer->setInterval(300);

//         connect(QBsdPort::global,   SIGNAL(searching()),
//                 this,               SLOT(show()));
        if (!connect(KPortsGlobal::instance(),  SIGNAL(searching()),
                unblockUpdateTimer, SLOT(start())))
            qDebug("could not connect taskwidget to parent");
        connect(KPortsGlobal::instance(),   SIGNAL(searching()),
                this,               SLOT(show()));
        connect(unblockUpdateTimer, SIGNAL(timeout()),
                this,               SLOT(unblockUpdate()));
        connect(KPortsGlobal::instance(),   SIGNAL(searchProgress(int)),
                this,               SLOT(setProgressBuffered(int)));
    //    connect(QBsdPort::global,   SIGNAL(searchResult(QBsdPort*)),
    //            this,               SLOT(searchResult(QBsdPort*)));
        connect(KPortsGlobal::instance(),   SIGNAL(searchFinished()),
                this,               SLOT(hide()));
        connect(KPortsGlobal::instance(),   SIGNAL(searchFinished()),
                progressBar,        SLOT(reset()));
        connect(KPortsGlobal::instance(),   SIGNAL(searchFinished()),
                unblockUpdateTimer, SLOT(stop()));

        this->hide();
    }

    void    retranslateUi()
    {
        head->setText(tr("<b>Search Progress:</b>"));
        sub->setText(QString());
    }
public slots:
    void    unblockUpdate()
    {
        blockUpdate = false;
        this->update();
    }

    void    setProgressBuffered(int newValue)
    {
        if (!blockUpdate)
        {
            blockUpdate=true;
            progressBar->setValue(newValue);
        }
    }
};

class QIndexTaskItem : public QCurrentTaskItem
{
Q_OBJECT
public:
    QIndexTaskItem(QWidget* parent=0)
        : QCurrentTaskItem(parent)
    {
        showBar = false;
        setupUi();
        retranslateUi();

        connect(QBsdPort::global,   SIGNAL(findAllAvSrcRunning()),
                this,               SLOT(started()));
        connect(QBsdPort::global,   SIGNAL(findAllAvSrcFinished(int)),
                this,               SLOT(done(int)));

//         this->hide();
    }

    void    retranslateUi()
    {
        head->setText(tr("<b>INDEX:</b>"));
        head->setToolTip(tr("Software available as Source-Code"));
        sub->setText(QString());
    }
public slots:
    void    started()
    {
        sub->setText(tr("reading..."));
        update();
    }

    void    done(int num)
    {
        if (num)
            sub->setText(tr("%1 Ports read").arg(num));
        else
        {
            sub->setText(tr("no Ports read").arg(num));
            QSettings settings("kports", "kports");
            int s = settings.value("icons/infoboxiconsize", 48).toInt();
            QSize size(s,s);
            QMessageBox msgBox;
            msgBox.setWindowTitle(tr("KPorts - Error"));
            msgBox.setStandardButtons(QMessageBox::Ok);
            msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                                "dialog-error").pixmap(size));
            msgBox.setText(tr("Failed to read the INDEX (information about "
                              "software available from Source). You will not "
                              "be able to install software from Source.\n\n"
                              "Adjust the path to INDEX in the settings or "
                              "refetch/update your portstree "
                              "(\"Fetch Updates\") to solve this."));
            //msgBox.setDetailedText(error);
            msgBox.exec();
        }
    }
};

class QPkgDbTaskItem : public QCurrentTaskItem
{
Q_OBJECT
public:
    QPkgDbTaskItem(QWidget* parent=0)
        : QCurrentTaskItem(parent)
    {
        showBar = false;
        setupUi();
        retranslateUi();

//         connect(QBsdPort::global,   SIGNAL(findAllInstRunning()),
//                 this,               SLOT(show()));
//         connect(QBsdPort::global,   SIGNAL(findAllInstFinished()),
//                 this,               SLOT(fadeOut()));
//
//         this->hide();

//         connect(QBsdPort::global,       
//                 SIGNAL(instPkgDbReaderChanged(QInstPkgDbReader*)),
//                 this,
//                 SLOT(instPkgDbReaderChanged(QInstPkgDbReader*)));

        connect(QBsdPort::global->instPkgDbReader(),
                SIGNAL(started()),
                this,
                SLOT(started()));
        connect(QBsdPort::global->instPkgDbReader(),
                SIGNAL(finishedSuccessfully(uint)),
                this,
                SLOT(done(uint)));
        connect(QBsdPort::global->instPkgDbReader(),
                SIGNAL(failed(const QString&)),
                this,
                SLOT(failed(const QString&)));
    }

    void    retranslateUi()
    {
        head->setText(tr("<b>Package Database:</b>"));
        head->setToolTip(tr("Software installed on your Computer."));
        sub->setText(tr("waiting..."));

    }
public slots:
//     void    instPkgDbReaderChanged(QInstPkgDbReader* newValue)
//     {
//         disconnect(0,0,this, SLOT(started()));
//         disconnect(0,0,this, SLOT(done(uint)));
//         disconnect(0,0,this, SLOT(failed(const QString&)));
// 
// 
//     }
    void    started()
    {
        sub->setText(tr("reading..."));
    }

    void    done(uint num)
    {
        sub->setText(tr("%1 Pkgs read").arg(num));
    }

    void    failed(const QString& error)
    {
        sub->setText(tr("failed."));
        QSettings settings("kports", "kports");
        int s = settings.value("icons/infoboxiconsize", 48).toInt();
        QSize size(s,s);
        QMessageBox msgBox;
        msgBox.setWindowTitle(tr("KPorts - Error"));
        msgBox.setStandardButtons(QMessageBox::Ok);
        msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                            "dialog-error").pixmap(size));
        msgBox.setText(tr("Failed to read information about installed packages"));
        msgBox.setDetailedText(error);
        msgBox.exec();
    }

};

class QVulnaDbTaskItem : public QCurrentTaskItem
{
Q_OBJECT
public:
    QVulnaDbTaskItem(QWidget* parent=0)
        : QCurrentTaskItem(parent)
    {
        showBar = false;
        setupUi();
        retranslateUi();

        connect(QBsdPort::global,   SIGNAL(updatingVulnaDb()),
                this,               SLOT(show()));
        connect(QBsdPort::global,   SIGNAL(vulnaDbUpdateFinished()),
                this,               SLOT(fadeOut()));
        connect(QBsdPort::global,   SIGNAL(vulnaDbUpdateFailed(const QString&)),
                this,               SLOT(fadeOut()));
        connect(QBsdPort::global,   SIGNAL(vulnaDbUpdateFailed(const QString&)),
                this,               SLOT(updateFailed(const QString&)));

        this->hide();
    }

    void    retranslateUi()
    {
        head->setText(tr("<b>Vulnerability Database:</b>"));
        sub->setText(tr("updating..."));
    }
public slots:
    void    updateFailed(const QString& log)
    {
        QSettings settings("kports", "kports");
        int s = settings.value("icons/infoboxiconsize", 48).toInt();
        QSize size(s,s);
        QMessageBox msgBox;
        msgBox.setWindowTitle(tr("KPorts - Error"));
        msgBox.setStandardButtons(QMessageBox::Ok);
        msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                            "dialog-error").pixmap(size));
        msgBox.setText(tr("Failed to update Vulnerability Database."));
        msgBox.setDetailedText(log);
        msgBox.exec();

    }

};

class QFtpIndexTaskItem : public QCurrentTaskItem
{
Q_OBJECT
public:
    QFtpIndexTaskItem(QWidget* parent=0)
        : QCurrentTaskItem(parent)
    {
        showBar = true;
        setupUi();
        retranslateUi();

        connect(QBsdPort::global, SIGNAL(pkgSiteLoggingIn()),
                this,               SLOT(loggingIn()));
        connect(QBsdPort::global, SIGNAL(pkgSiteFoundIndex()),
                this,               SLOT(indexFound()));

        connect(QBsdPort::global, SIGNAL(findAllAvBinFinished(int)),
                this,               SLOT(readingDone(int)));
        connect(QBsdPort::global, SIGNAL(findAllAvBinRunning()),
                this,               SLOT(reading()));

        connect(QBsdPort::global, SIGNAL(ftpIndexAboutToFetch()),
                this,               SLOT(aboutToFetch()));
        connect(QBsdPort::global, SIGNAL(ftpIndexFail(const QString&)),
                this,               SLOT(failure(const QString&)));
        connect(QBsdPort::global, SIGNAL(ftpIndexNowUpToDate()),
                this,               SLOT(nowUpToDate()));
        connect(QBsdPort::global, SIGNAL(ftpIndexFetchProgress(qint64, qint64)),
                this,               SLOT(fetchProgress(qint64, qint64)));

//         this->hide();
        progressBar->hide();
    }

    void    retranslateUi()
    {
        head->setText(tr("<b>FTP-INDEX:</b>"));
        head->setToolTip(tr("Software available as binary Package"));
        sub->setText(tr("waiting..."));
    }
public slots:
    void    loggingIn()
    {
        if (!this->isVisible())
            this->show();
        sub->setVisible(true);
        sub->setText(tr("Logging in to Packagesite..."));
        progressBar->setVisible(false);

    }
    void    indexFound()
    {
        if (!this->isVisible())
            this->show();
        sub->setVisible(true);
        sub->setText(tr("Found on Packagesite..."));
        progressBar->setVisible(false);
    }
    void    aboutToFetch()
    {
        if (!this->isVisible())
            this->show();
        sub->setVisible(true);
        sub->setText(tr("Preparing to fetch..."));
        progressBar->setVisible(false);
    }
    void    nowUpToDate()
    {
        if (!this->isVisible())
            this->show();
        sub->setVisible(true);
        sub->setText(tr("up-to-date."));
        progressBar->setVisible(false);
    }
    void    fetchProgress(qint64 progress, qint64 total)
    {
//         qDebug("Progress: " + QString::number(progress).toLatin1() + "/" +
//                 QString::number(total).toLatin1());
        if (!this->isVisible())
            this->show();
//        if (progressBar->maximum()==0)
//            progressBar->setRange(0, total);

        if ( (total > 0) && (total >= progress) )
        {
            if (progressBar->maximum() != total)
                progressBar->setRange(0, total);
            progressBar->setVisible(true);
            progressBar->setValue(progress);
            progressBar->setFormat(tr("updating: %p"));
            sub->setVisible(false);
        } else
        {
            progressBar->hide();
            sub->setVisible(true);
            sub->setText(tr("updating..."));
        }
    }
    void    reading()
    {
        if (!this->isVisible())
            show();
        sub->setVisible(true);
        sub->setText(tr("reading..."));
        progressBar->setVisible(false);
    }
    void    readingDone(int num)
    {
        if (!this->isVisible())
            this->show();
        sub->setVisible(true);
        sub->setText(tr("%1 Pkgs read").arg(num));
        progressBar->setVisible(false);
//         fadeOut();
    }
    void    failure(const QString& error)
    {
        if (!this->isVisible())
            this->show();
        sub->setVisible(true);
        sub->setText(tr("failed to read."));
        progressBar->setVisible(false);

        QSettings settings("kports", "kports");
        int s = settings.value("icons/infoboxiconsize", 48).toInt();
        QSize size(s,s);
        QMessageBox msgBox;
        msgBox.setWindowTitle(tr("KPorts - Error"));
        msgBox.setStandardButtons(QMessageBox::Ok);
        msgBox.setIconPixmap(KPortsIcons::iconLoader.loadIcon(
                            "dialog-error").pixmap(size));
        msgBox.setText(tr("Failed to update/read the FTPINDEX. "
                          "You will not be able to use Packages."));
        msgBox.setDetailedText(error);
        msgBox.exec();
//         fadeOut();
    }
};

#endif //NDEF TASKWIDGETS_H
